#include <klineeditdlg.h>
#include <klocale.h>
#include "activitywidgetdata.h"
#include "umlwidget.h"
#include "umlview.h"
#include "dialogs/activitydialog.h"
#include "activitywidget.h"

ActivityWidget::ActivityWidget(UMLView * view, CUMLWidgetData* pData) : UMLWidget(view, pData)
{	
}

ActivityWidget::ActivityWidget(UMLView * view, ActivityType activityType ) : UMLWidget(view, new CActivityWidgetData())
{
  dynamic_cast<CActivityWidgetData *>( m_pData ) -> setActivityType( activityType );
	m_pData->setType(wt_Activity);
	calculateSize();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ActivityWidget::ActivityWidget(UMLView * view) : UMLWidget(view, new CActivityWidgetData())
{
	m_pData->setType(wt_Activity);
	calculateSize();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ActivityWidget::~ActivityWidget()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ActivityWidget::draw(QPainter & p, int offsetX, int offsetY)
{
  int w = width();
  int h = height();
  ActivityType type = dynamic_cast<CActivityWidgetData *>( m_pData ) -> getActivityType();
  if( type == Normal )
  {
    p.setPen(m_LineColor);
    if(m_pData->getUseFillColor())
      p.setBrush(m_FillColor);
    QFontMetrics fm = QFontMetrics( m_pData -> getFont() );
    int fontHeight  = fm.lineSpacing();
    //int middleX = w / 2;
    int textStartY = (h / 2) - (fontHeight / 2);
		p.drawRoundRect(offsetX, offsetY, w, h, 75, 75);
   	p.setPen(black);
		QFont font = m_pData -> getFont();
		p.setFont( font );
   	p.drawText(offsetX + ACTIVITY_MARGIN, offsetY + textStartY, w - ACTIVITY_MARGIN * 2, fontHeight, AlignCenter, GetName());
		p.setPen(m_LineColor);
  }
  else if( type == Initial )
  {
    p.setPen(m_LineColor);
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX, offsetY, w, h );
  }
  else if( type == End )
  {
    p.setPen(m_LineColor);
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX, offsetY, w, h );
    p.setBrush( white );
    p.drawEllipse( offsetX + 1, offsetY + 1, w - 2, h - 2 );
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX + 3, offsetY + 3, w - 6, h - 6 );
  }
	else if( type == Branch )
	{
		QPointArray array( 4 );
		array[ 0 ] = QPoint( offsetX + w / 2, offsetY );
		array[ 1 ] = QPoint( offsetX + w, offsetY  + h / 2 );
		array[ 2 ] = QPoint( offsetX + w / 2, offsetY + h );  
		array[ 3 ] = QPoint( offsetX, offsetY + h / 2 );
		p.setPen(m_LineColor);
		p.setBrush( m_FillColor );
		p.drawPolygon( array );
		p.drawPolyline( array );
	}
	else if( type == Fork )
	{
		QBrush b = QBrush( black );
		p.fillRect( offsetX, offsetY, width(), height(), b );
	}
  if(m_bSelected)
    drawSelected(&p, offsetX, offsetY);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ActivityWidget::calculateSize()
{
	int width = 10, height = 10;
	ActivityType type = dynamic_cast<CActivityWidgetData *>( m_pData ) -> getActivityType();
  if( type == Normal )
  {
		QFont font = m_pData -> getFont();
    QFontMetrics fm = QFontMetrics( font );
    int fontHeight  = fm.lineSpacing();
    int textWidth = fm.width(GetName());
		height = fontHeight;
	  width = textWidth > ACTIVITY_WIDTH?textWidth:ACTIVITY_WIDTH;
		height = height > ACTIVITY_HEIGHT?height:ACTIVITY_HEIGHT;
    width += ACTIVITY_MARGIN * 2;
		height += ACTIVITY_MARGIN * 2;
  }
	else if( type == Branch )
	{
		width = height = 20;
	}
	else if( type == Fork )
	{
		width = 40;
		height = 4;
	}
  setSize(width, height);
}

void ActivityWidget::SynchronizeData()
{


	//Nothing to synchronize
	UMLWidget::SynchronizeData();
}

void ActivityWidget::SetName(QString strName)
{
  dynamic_cast<CActivityWidgetData *>( m_pData ) -> setName( strName );
  calculateSize();
  AdjustAssocs( (int)x(), (int)y() );
}

QString ActivityWidget::GetName()
{
  return dynamic_cast<CActivityWidgetData *>( m_pData ) -> getName();
}

QString ActivityWidget::getDoc()
{
  return dynamic_cast<CActivityWidgetData *>( m_pData ) -> getDoc();
}

void ActivityWidget::setDoc( QString doc )
{
  dynamic_cast<CActivityWidgetData *>( m_pData ) -> setDoc( doc );
}

ActivityWidget::ActivityType ActivityWidget::getActivityType()
{
  return dynamic_cast<CActivityWidgetData *>( m_pData ) -> getActivityType();
}

void ActivityWidget::setActivityType( ActivityType activityType )
{
  dynamic_cast<CActivityWidgetData *>( m_pData ) -> setActivityType( activityType );
}

void ActivityWidget::slotMenuSelection(int sel)
{
  bool done = false;

  bool ok = false;
  QString name = GetName();

  switch( sel )
  {
    case ListPopupMenu::mt_Rename:
      name = KLineEditDlg::getText( i18n("Enter Activity name."), i18n("Enter the name of the new Activity."), GetName(), &ok );
      if( ok && name.length() > 0 )
        SetName( name );
      done = true;
    break;

    case ListPopupMenu::mt_Properties:
      mouseDoubleClickEvent( 0 );
      done = true;
    break;
  }

  if( !done )
    UMLWidget::slotMenuSelection( sel );
}


void ActivityWidget::mouseDoubleClickEvent(QMouseEvent * /*me*/)
{
  m_pView -> updateDocumentation( false );
  ActivityDialog dialog( m_pView, this );
  if( dialog.exec() && dialog.getChangesMade() )
  {
    //put here anything that needs to be done for changes
    //nothing at the moment.
  }
  m_pView -> showDocumentation( this, true );
}






