#include <qdatastream.h>

#include "activitywidgetdata.h"

CActivityWidgetData::CActivityWidgetData()
{
  m_ActivityType = ActivityWidget::Normal;
  m_Name = "Activity";
  m_Doc = "";
  m_Type = Uml::wt_Activity;
}

CActivityWidgetData::CActivityWidgetData(CActivityWidgetData & Other) : CUMLWidgetData(Other)
{
  *this = Other;
}

CActivityWidgetData::~CActivityWidgetData()
{

}

CActivityWidgetData & CActivityWidgetData::operator=(CActivityWidgetData & Other)
{
  *((CUMLWidgetData*)this) = (CUMLWidgetData)Other;
  m_ActivityType = Other.m_ActivityType;
  m_Name = Other.m_Name;
  return *this;
}

bool CActivityWidgetData::operator==(CActivityWidgetData & Other)
{
  if(!(*((CUMLWidgetData*)this) == (CUMLWidgetData)Other))
  {
    return false;
  }
  if( m_ActivityType != Other.m_ActivityType )
    return false;
  if( m_Name != Other.m_Name )
    return false;
  return true;
}

long CActivityWidgetData::getClipSizeOf()
{
  long l_size = ( CUMLWidgetData::getClipSizeOf() + sizeof( int ) );
  if( m_Name.length() == 0 )
    l_size += sizeof( Q_UINT32 );
  else
    l_size += ( sizeof( QChar ) * m_Name.length() );
  if( m_Doc.length() == 0 )
    l_size += sizeof( Q_UINT32 );
  else
    l_size += ( sizeof( QChar ) * m_Doc.length() );
  return l_size;
}

bool CActivityWidgetData::serialize(QDataStream *s, bool archive)
{
  int nActivity = (int)m_ActivityType;
  if( archive )
  {
    *s << nActivity
        << m_Name
        << m_Doc;
  }
  else
  {
    *s >> nActivity
        >> m_Name
        >> m_Doc;
    m_ActivityType = (ActivityWidget::ActivityType)nActivity;
  }
  return CUMLWidgetData::serialize(s, archive);
}

bool CActivityWidgetData::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement activityElement = qDoc.createElement( "UML:ActivityWidget" );
  bool status = CUMLWidgetData::saveToXMI( qDoc, activityElement );
  activityElement.setAttribute( "activityname", m_Name );
  activityElement.setAttribute( "documentation", m_Doc );
  activityElement.setAttribute( "activitytype", m_ActivityType );
  qElement.appendChild( activityElement );
  return status;
}

bool CActivityWidgetData::loadFromXMI( QDomElement & qElement )
{
  if( !CUMLWidgetData::loadFromXMI( qElement ) )
    return false;
  m_Name = qElement.attribute( "activityname", "" );
  m_Doc = qElement.attribute( "documentation", "" );
  QString type = qElement.attribute( "activitytype", "1" );
  m_ActivityType = (ActivityWidget::ActivityType)type.toInt();
  return true;
}







