#ifndef ASSOCIATIONWIDGETDATA_H
#define ASSOCIATIONWIDGETDATA_H

#include <qpointarray.h>
#include "umlnamespace.h"
#include "linepath.h"
#include <qdom.h>

class CFloatingTextData;

/**
	*	This class holds all the information
	*	from an association
  	*	@author Gustavo Madrigal
  	*/

class CAssociationWidgetData
{
friend class CAssociationWidget;
friend class LinePath;
public:
	/**
	* 		Constructor
	*/
  CAssociationWidgetData( );

	/**
	* 		Deconstructor
	*/
	virtual ~CAssociationWidgetData();


	/**
	* 		Copy Constructor 
	*/
	CAssociationWidgetData(CAssociationWidgetData & Other);

	/**
	* 		Overloaded '=' operator
	*/
	virtual CAssociationWidgetData & operator=(CAssociationWidgetData & Other);

	/**
	* 		Overloaded '==' operator
	*/
	virtual bool  operator==(CAssociationWidgetData & Other);
	/** 
	*		Save/load data
	*/
 	virtual bool serialize(QDataStream *s, bool archive);

  	/**
	* Returns the amount of bytes needed to serialize this object
	*	If the serialization method of this class is changed this function will have to be CHANGED TOO
	* This function is used by the Copy and Paste Functionality
	*	The Size in bytes of a serialized QString Object is long sz:
	*	if ( (sz =str.length()*sizeof(QChar)) && !(const char*)str.unicode() )
	*	{
	*		sz = size of Q_UINT32; //  typedef unsigned int	Q_UINT32;		// 32 bit unsigned
	*	}
	*	This calculation is valid only for QT 2.1.x or superior, this is totally incompatible with QT 2.0.x or QT 1.x or inferior

	*	That means the copy and paste functionality will work on with QT 2.1.x or superior 
	*/
  	virtual long getClipSizeOf();
	
  /**
	* Write property of CFloatingTextData* m_pMultiDataA. 
	*/
  virtual void setMultiDataA( CFloatingTextData* pMultiData);
	
  /**
	* Read property of CFloatingTextData* m_pMultiDataA. 
	*/
  virtual CFloatingTextData* getMultiDataA();

  /**
	* Write property of CFloatingTextData* m_pMultiDataB. 
	*/
  virtual void setMultiDataB( CFloatingTextData* pData);

  /**
	* Read property of CFloatingTextData* m_pMultiDataB. 
	*/
  virtual CFloatingTextData* getMultiDataB();
	
  /**
	* Write property of CFloatingTextData* m_pRoleData. 
	*/
  virtual void setRoleData( CFloatingTextData* pData);
	
  /**
	* Read property of CFloatingTextData* m_pRoleData. 
	*/
  virtual CFloatingTextData* getRoleData();
	
  /**
	* Write property of AssociationType AssocType. 
	*/
  virtual void setAssocType( Uml::Association_Type Type);
	
  /**
	* Read property of AssociationType AssocType. 
	*/
  virtual  Uml::Association_Type getAssocType();

  /**
	* Write property of int m_nWidgetAID. 
	*/
  virtual void setWidgetAID( int AID);

  /**
	* Read property of int m_nWidgetAID. 
	*/
  virtual int getWidgetAID();
	
  /**
	* Write property of int m_nWidgetBID. 
	*/
  virtual void setWidgetBID( int BID);

  /**
	* Read property of int m_nWidgetBID. 
	*/
  virtual int getWidgetBID();
	
  /**
	* Prints the association's data to stderr 
	*/
  void print2cerr();

  /**
  *  Sets the Association line index for widgetA
  */
  void setIndexA(int index){ m_nIndexA = index; }

  /**
  *  Returns the Association line index for widgetA
  */
  int getIndexA(){ return m_nIndexA; }

  /**
  *  Sets the Association line index for widgetB
  */
  void setIndexB(int index){ m_nIndexB = index; }

  /**
  *  Returns the Association line index for widgetB
  */
  int getIndexB(){ return m_nIndexB; }

  /**
  *  Sets the total count on the Association region for widgetA
  */
  void setTotalCountA(int count){ m_nTotalCountA = count; }

  /**
  *  Returns the total count on the Association region for widgetA
  */
  int getTotalCountA(){ return m_nTotalCountA; }

  /**
  *  Sets the total count on the Association region for widgetB
  */
  void setTotalCountB(int count){ m_nTotalCountB = count; }

  /**
  *  Region the total count on the Association region for widgetB
  */
  int getTotalCountB(){ return m_nTotalCountB; }

	/**
	*  	Returns the associations documentation.
	*/
	QString getDoc(){ return m_Doc; }

	/**
	*		Sets the associations documentation.
	*/
	void setDoc( QString doc){ m_Doc = doc; }

  bool saveToXMI( QDomDocument & qDoc, QDomElement & qElement );

  bool loadFromXMI( QDomElement & qElement );

  void cleanup();

protected: // Private attributes
	
  	/**
	* The Data from the Multiplicity Text widget at the starting side of the association 
	*/
  	CFloatingTextData* m_pMultiDataA;
	
  	/**

	* The Data from the Multiplicity Text widget at the ending side of the association 
	*/
  	CFloatingTextData* m_pMultiDataB;

	/**
	* The Data from the Association's Role FloatingText widget 
	*/
  	CFloatingTextData* m_pRoleData;
	
  	/**
	* This is the type of the association 
	*/
  	Uml::Association_Type m_AssocType;

	/**
	* The definition points for the association line 
	*/
	LinePath m_LinePath;

  /**
	* This member holds the identification number the the UMLWidget at the starting side
	*	of the association 
	*/
  int m_nWidgetAID;

	/**
	* This member holds the identification number the the UMLWidget at the ending side
	*	of the association 
	*/
  int m_nWidgetBID;

	/**
	* 		The index of where the line is on the region for widget a
	*/
  int m_nIndexA;

	/**
	*   	The index of where the line is on the region for widget b
	*/
  int m_nIndexB;

	/**
	* 		The total amount of associations on the region widget a line is on.
	*/
  int m_nTotalCountA;

	/**
	*		The total amount of associations on the region widget b line is on.
	*/
  int m_nTotalCountB;

	/**
	* 		The documentation for the association.
	*/
	QString	m_Doc;
};

#endif
