#ifndef CUMLDRAG_H
#define CUMLDRAG_H


#include <qdragobject.h>
#include <qobject.h>

#include "../mylistviewitemdatalist.h"
#include "../associationwidgetdatalist.h"
#include "../umlobjectlist.h"
#include "../umlviewlist.h"
#include "../umlwidgetlist.h"
#include "../umlnamespace.h"

/**
	*	This class provides encoding and 
	*	decoding for the uml data that will be
	*	use in a drag_n_drop operation or
	*	in a copy paste operation
  *	@author Gustavo Madrigal
  */

class MyListView;
class UMLDoc;

class CUMLDragPrivate;


class Q_EXPORT CUMLDrag : public QDragObject  {
Q_OBJECT
	CUMLDragPrivate *d;
public:
	/**	
	*		For use when the user selects only UML Objects from the
	*		ListView but no diagrams to be copied, Mime	type = "application/x-uml-clip1
	*/
	CUMLDrag( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems,	QWidget * dragSource = 0, const char * name = 0 );

   /**	
	*		For use when the user selects UML Object and Diagrams from
	*		the ListView to be copied, Mime type = "application/x-uml-clip2
	*/
	CUMLDrag( CUMLObjectList &Objects, CMyListViewItemDataList& MyListViewItems, UMLViewDataList& Diagrams,	QWidget * dragSource = 0, const char * name = 0 );

	/**
	*		For use when the user selects only empty folders from the ListView to be copied, Mime
	*		type = "application/x-uml-clip3
	*/
	CUMLDrag( CMyListViewItemDataList& MyListViewItems, QWidget * dragSource = 0, const char * name = 0 );

   /*
	*		For use when the user selects UMLObjects from a Diagram. The Selected widegets and the relationships
	*		between only selected widgets will be copied and also its respective
	*		ListView Items, Mime type = "application/x-uml-clip4
	*/
	CUMLDrag( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems, CUMLWidgetDataList& WidgetDatas,
			CAssociationWidgetDataList& AssociationDatas, QPixmap& PngImage, Uml::Diagram_Type dType, QWidget * dragSource = 0, const char * name = 0 );

   /**
	*		For use when the user selects only Operations and/or Attributes from the
	*		ListView, Mime 	type = "application/x-uml-clip5
	*/
	CUMLDrag( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems,  int, QWidget * dragSource = 0, const char * name = 0 );

	/**
	* 		Constructor
	*/
	CUMLDrag( QWidget * dragSource = 0, const char * name = 0 );

	/**
	* 		Deconstructor
	*/
	~CUMLDrag();

	/**
	*		For use when the user selects only UMLObjects from the ListView but no diagrams to be copied
	*/
	void setUMLData( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems );

   /**
	*		For use when the user selects UML Object and Diagrams from the ListView to be copied
	*/
	void setUMLData( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems, UMLViewDataList& Diagrams );

	/**
	*		For use when the user selects only empty folders from the ListView to be copied.
	*/
	void setUMLData( CMyListViewItemDataList& MyListViewItems );

	/**
	*		For use when the user selects UML Objects from a Diagram. The Selected widegets
	*		and the relationships	between only selected widgets will be copied and also its respective
	*		ListView Items
	*/
	void setUMLData( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems, CUMLWidgetDataList& WidgetDatas,
							CAssociationWidgetDataList& AssociationDatas, QPixmap& PngImage, Uml::Diagram_Type dType );

	/**
	*		For use when the user selects only Attirbutes and/or Operation from the ListView
	*/
	void setUMLData( CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems, int );

	/**
	* 		Returns whether it can decode the given mimesource
	*/
	static bool canDecode( const QMimeSource* e );

	/**
	*
	*/
	virtual void setSubType( const QCString & sub, int index);

	/**
	*
	*/
	virtual void setEncodedData( const QByteArray &, int index );

	/**
	*
	*/
	const char * format(int i) const;

	/**
	*
	*/
    virtual QByteArray encodedData(const char*) const;

	/**
	*		For use when the user selects only UML Objects from the ListView but no diagrams to be
	*		copied, decodes Mime type = "application/x-uml-clip1
	*/
	static bool decode( const QMimeSource* e, CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems, UMLDoc* Doc);

	/**
	*		For use when the user selects UML Object and Diagrams from the ListView to be copied, decodes Mime
	*		type = "application/x-uml-clip2
	*/
	static bool decode( const QMimeSource* e, CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems,
																				UMLViewDataList& Diagrams, UMLDoc* Doc  );

   /**
	*		For use when the user selects only empty folders from the ListView to be copied, decodes Mime
	*		type = "application/x-uml-clip3
	*/
	static bool decode( const QMimeSource* e, CMyListViewItemDataList& MyListViewItems);

	/**
	*		For use when the user selects UML Objects from a Diagram. The Selected widegets and the relationships between only
	*		selected widgets will be copied and also its respective ListView Items,
	*		decodes Mime type = "application/x-uml-clip4
	*/
	static bool decode( const QMimeSource* e, CUMLObjectList& Objects, CMyListViewItemDataList& MyListViewItems,
									CUMLWidgetDataList& WidgetDatas, CAssociationWidgetDataList& AssociationDatas, Uml::Diagram_Type & dType, UMLDoc* Doc );

	/**
	*		For use when the user selects only Attributes and/or Operations from the ListView
	*		copied, decodes Mime type = "application/x-uml-clip5
	*/
	static bool decode( const QMimeSource* e, CUMLObjectList& Objects,	CMyListViewItemDataList& MyListViewItems, UMLDoc* Doc, int);

	/**
	* No descriptions 
	*/
  static int getCodingType(const QMimeSource* e);
};

#endif
