#ifndef IDCHANGELOG_H
#define IDCHANGELOG_H


/**
*		This class contains all the ID translations done for each UMLObject pasted. It caontains for
*		each old id its new assigned id.
*
*		@author Gustavo Madrigal
*/

#include <qpointarray.h>

class CIDChangeLog
{
public: 
	/**
	* 		Constructor
	*/
	CIDChangeLog();

	/**
	*	Copy constructor
	*/
	CIDChangeLog(const CIDChangeLog& Other);

	/**
	* 		Deconstructor
	*/
	~CIDChangeLog();

	/**
	*  	Overloaded '=' operator
	*/
	CIDChangeLog& operator=(const CIDChangeLog& Other);

	/**
	*  	Overloaded '==' operator
	*/
	bool operator==(const CIDChangeLog& Other);

	/**
	* Adds a new ID Change to the log 
	*/
  void AddIDChange(int OldID, int NewID);

	/** 
	*	Appends another CIDChangeLog to this instance of CIDChangeLog and
	*	returns a reference to itself 
	*/
  CIDChangeLog& operator+=(const CIDChangeLog& Other);

	/**
	* Returns the new assigned ID of the object that had OldID as its previous id 
	*/
  int FindNewID(int OldID);

	/** 
	*	Returns the old ID of an UMLobject given its new one 
	*/
  int FindOldID(int NewID);

  /**
	*	 Removes a change giving an New ID 
	*/
  void RemoveChangeByNewID( int OldID);

	enum SpecialIDs
	{
		NullID = -1000 //A not posible id
	};
	
public: 
  /**
	* Each change is a QPoint (x=newID, y=oldID) 
	*/
  QPointArray m_LogArray;
private: 
  /**
	* Finds a specific change in the log  
	*/
  bool FindIDChange(int OldID, int NewID, uint& pos);
};

#endif
