#include <qmime.h>

#include "../uml.h"
#include "../umldoc.h"
#include "../umlview.h"
#include "../mylistview.h"
#include "../mylistviewitem.h"
#include "../associationwidgetdata.h"
#include "../associationwidgetdatalist.h"
#include "../umlwidgetlist.h"
#include "../umlobjectlist.h"
#include "../umlviewlist.h"
#include "../mylistviewitemdatalist.h"
#include "../mylistviewitemdata.h"
#include "../floatingtextdata.h"
#include "idchangelog.h"
#include "umlclipboard.h"
#include "cumldrag.h"
#include "../mylistviewitem.h"
#include "../umlnamespace.h"
#include <kdebug.h>

CUMLClipboard::CUMLClipboard()
{
	m_pObjectList = new CUMLObjectList;
	m_pObjectList->setAutoDelete( FALSE );
	m_pItemDataList = new CMyListViewItemDataList;;
	m_pItemDataList->setAutoDelete( FALSE );
	m_pWidgetDataList = new CUMLWidgetDataList;
	m_pWidgetDataList->setAutoDelete( FALSE );
	m_pAssociationDataList = new CAssociationWidgetDataList;
	m_pAssociationDataList->setAutoDelete( FALSE );
	m_pViewDataList = new UMLViewDataList;
	m_pViewDataList->setAutoDelete( FALSE );
	m_type = clip1;
}


CUMLClipboard::~CUMLClipboard()
{
	if(m_pObjectList)
	{
		delete m_pObjectList;
	}
	if(m_pItemDataList)
	{
		delete m_pItemDataList;
	}
	if(m_pWidgetDataList)
	{
		delete m_pWidgetDataList;
	}
	if(m_pAssociationDataList)
	{
		delete m_pAssociationDataList;
	}
	if(m_pViewDataList)
	{
		delete m_pViewDataList;
	}

}
/** Copies the selected stuff from an UMLDocument to a QMimeSource
ready to be put in the clipboard. fromView speifies when the copy
action is performed from a diagram or from the ListView */
QMimeSource* CUMLClipboard::copy(UMLDoc * Doc, bool fromView/*=false*/)
{
	if(!Doc)
	{
		return 0;
	}
	//Clear previous copied data
	m_pAssociationDataList->clear();
	m_pItemDataList->clear();
	m_pObjectList->clear();
	m_pViewDataList->clear();

	CUMLDrag *data = 0;
	QPixmap* png = 0;

	MyListView * listView = Doc->listView;
   CMyListViewItemList selectedItems;
	selectedItems.setAutoDelete(FALSE);

	if(fromView)
	{
		m_type = clip4;
		Doc->getCurrentView() -> checkSelections();
		if(!Doc->getCurrentView()->GetSelectedWidgetDatas(*m_pWidgetDataList))
		{
			return 0;
		}
		//if there is no selected widget then there is no copy action
		if(!m_pWidgetDataList->count())
		{
			return 0;
		}
		CUMLWidgetDataListIt widgetdata_it(*m_pWidgetDataList);
		CUMLWidgetData* widgetdata = widgetdata_it.current();
		UMLObject* object = 0;
		while(widgetdata)
		{
			++widgetdata_it;
			if(widgetdata->getType() >= Uml::wt_Actor && widgetdata->getType() <= Uml::wt_Object)
			{
				object = Doc->findUMLObject(widgetdata->getId());
				//if the object is not already on the list
			 	if(m_pObjectList->find(object) == -1)
			 	{
					m_pObjectList->append(object);
			 	}
			}
			widgetdata = widgetdata_it.current();
		}
		if(!Doc->getCurrentView()->GetSelectedAssocDatas(*m_pAssociationDataList))
		{
			return 0;
		}

		//For each Selected UMLObject get its CMyListViewItemData and children 
		CUMLObjectListIt object_it(*m_pObjectList);
		object = object_it.current();
		MyListViewItem* item = 0;
		while(object)
		{
			++object_it;
			item = listView->findItem(object->getID());
			if(item)
			{
				m_pItemDataList->append( item->getdata() );
				insertItemDataChildren( item );
			}
			else
			{
				return 0;
			}
			object = object_it.current();
		}

		
		Doc->getCurrentView()->copyAsImage(png);
	}
	else //if the copy action is being performed from the ListView
	{
		if(!listView->GetSelectedItems(selectedItems))
		{
			return 0;
		}
      //Set What type of copy operation are we performing and
		//also fill m_pViewList with all the selected Diagrams
		SetCopyType(selectedItems, Doc);

		//if we are copying a diagram or part of a diagram, select the items
		//on the ListView that correspond to a UseCase, Actor or Concept
		//in the Diagram	
		if(m_type == clip2)
  		//Fill the member lists with all the object and stuff to be copied	
		//to the clipboard
		{
			selectedItems.clear();
			MyListViewItem* item = 0;
			UMLViewDataListIt view_it(*m_pViewDataList);
			//For each selected view select all the Actors, USe Cases and Concepts
			//widgets in the ListView
			CUMLObjectList* objects = 0;
			for( ; view_it.current(); ++view_it)
			{
    			objects = Doc -> findView( view_it.current()->getID() ) -> getUMLObjects();
				CUMLObjectListIt object_it(*objects);
    			for( ; object_it.current(); ++object_it)
    			{
    				item = listView->findUMLObject(object_it.current());
    				if(item)
    				{
    					listView->setSelected(item, true);
    				}
    			}
				delete objects;
				objects = 0;
			}
			if(!listView->GetSelectedItems(selectedItems))
			{
				return 0;
			}
		}
		if(!FillSelectionLists(selectedItems))
		{
			return 0;
		}
	}
	int i =0;
	switch(m_type)
	{
		case clip1:
			data = new CUMLDrag(*m_pObjectList, *m_pItemDataList);
		break;
		case clip2:
			data = new CUMLDrag(*m_pObjectList, *m_pItemDataList, *m_pViewDataList);
		break;
		case clip3:
			data = new CUMLDrag(*m_pItemDataList);
		break;
		case clip4:
			if(png)
			{
				data = new CUMLDrag(*m_pObjectList, *m_pItemDataList, *m_pWidgetDataList,
																*m_pAssociationDataList, *png, Doc -> getCurrentView() -> getType() );
			}
			else
			{
				return 0;
			}
		break;
		case clip5:
			data = new CUMLDrag(*m_pObjectList, *m_pItemDataList, i); //The int i is used to differenciate
			//which CUMLDrag Constructor gets called
		break;
	}
                         	
	return (QMimeSource*)data;
}
/** Inserts the clipboard's contents into an UML Document */
bool CUMLClipboard::paste(UMLDoc * Doc, QMimeSource* Data)
{
	if(!Doc)
	{
		return false;
	}
	bool result = false;
	Doc->BeginPaste();
	switch(CUMLDrag::getCodingType(Data))
	{
		case 1:
			result = pasteClip1(Doc, Data);
		break;
		case 2:

			result = pasteClip2(Doc, Data);
		break;
		case 3:
			result = pasteClip3(Doc, Data);
		break;
		case 4:
			result = pasteClip4(Doc, Data);
		break;
		case 5:
			result = pasteClip5(Doc, Data);
		break;
		default:
		break;
	}
   Doc->EndPaste();


	return result;
}
/** Fills the member lists with all the objects and other stuff to
be copied to the clipboard when the selection is made using the ListView*/
bool CUMLClipboard::FillSelectionLists(CMyListViewItemList& SelectedItems)
{
 	CMyListViewItemListIt it(SelectedItems);
	MyListViewItem* item = 0;
	Uml::ListView_Type type;
	switch(m_type)
 	{
		case clip4:
  		break;
		case clip3:
       for ( ; it.current(); ++it )
       {
         item = (MyListViewItem*)it.current();
         type = item->getType();
         if((type != Uml::lvt_Attribute) && (type != Uml::lvt_Operation))
         /*If an attribute or operation was selected without selecting its parent,
         	i.e it should not be copied*/
         {
          	m_pItemDataList->append(item->getdata());
         	InsertItemChildren(item, SelectedItems);//Because it is being called when m_type is 3
         	//it will insert only child empty folders of other folders. If a child folder
         	//its not empty that means m_type wouldn't be 3 because if a folder is
         	//selected then its complete contents are treated as if they were selected
         }
       }
  		break;
		case clip2:
		case clip1:
         for ( ; it.current(); ++it )
        {
         	item = (MyListViewItem*)it.current();
     			type = item->getType();
     			if((type != Uml::lvt_Attribute) && (type != Uml::lvt_Operation))

     			/*If an attribute or operation was selected then:
     				1- its parent is not selected -> It should not be copied,
     				2- its parent is selected -> there is no need to added to m_pItemDataList
     					because when its parent is processed it gets inside the list*/
     			{
      				m_pItemDataList->append(item->getdata());
               
      				if(type == Uml::lvt_Actor || type == Uml::lvt_UseCase || type == Uml::lvt_Class )
      				{
      					m_pObjectList->append(item->getUMLObject());
      				}
      				InsertItemChildren(it.current(), SelectedItems);
     			}
        }
 		break;
		case clip5:
			for ( ; it.current(); ++it )
        	{
         		item = (MyListViewItem*)it.current();
     			type = item->getType();
     			if((type == Uml::lvt_Attribute) || (type == Uml::lvt_Operation))
     			{
      				m_pItemDataList->append(item->getdata());
      				m_pObjectList->append(item->getUMLObject());

     			}
				else
				{
					return false;
				}
        	}
		break;
  	}

	return true;
}
/** Checks the whole list to determine the copy action type to be
performed, sets the type in the m_type member variable */
void CUMLClipboard::SetCopyType(CMyListViewItemList& SelectedItems, UMLDoc* Doc)
{
	bool withDiagrams = false; //If the selection includes diagrams
	bool	 withObjects = false; //If the selection includes objects
	bool onlyAttsOps = false; //If the selection only includes Attributes and/or Operations
	CMyListViewItemListIt it(SelectedItems);
  	for ( ; it.current(); ++it )
	{
  		 CheckItemForCopyType(it.current(), withDiagrams, withObjects, onlyAttsOps, Doc);
  	}
	if(onlyAttsOps)
	{
		m_type = clip5;
	}
	else if(withDiagrams)
	{
		m_type = clip2;
	}
	else
	{
		if(withObjects)
		{
			m_type = clip1;
		}
		else
		{
			m_type = clip3;
		}
	}
}
/** Searches the child items of a MyListViewItem to establish
with Copy type is to be perfomed */
void CUMLClipboard::CheckItemForCopyType(MyListViewItem* Item, bool & WithDiagrams, bool &WithObjects,
																							bool &OnlyAttsOps, UMLDoc* Doc)
{
	if(!Item)

	{
		return;
	}
	OnlyAttsOps = true;
	UMLView * view = 0;
	MyListViewItem * child = 0;
	int type = Item->getType();
   switch( type )
	{
		case Uml::lvt_Actor:
		case Uml::lvt_UseCase:
		case Uml::lvt_Class:
			WithObjects = true;
			OnlyAttsOps = false;
		break;

		case Uml::lvt_UseCase_Diagram:
		case Uml::lvt_Class_Diagram:
		case Uml::lvt_Collaboration_Diagram:
		case Uml::lvt_Sequence_Diagram:
		case Uml::lvt_State_Diagram:
		case Uml::lvt_Activity_Diagram:
			WithDiagrams = true;
			OnlyAttsOps = false;
			view= Doc->findView( Item->getID() );
			view -> SynchronizeData(); //make sure all data up to date
			m_pViewDataList->append( view -> getData() );
		break;
	
		case Uml::lvt_UseCase_Folder:
		case Uml::lvt_Logical_Folder:
			OnlyAttsOps = false;
			if(Item->childCount())
			{
				child = (MyListViewItem*)Item->firstChild();
				while(child)
				{
					CheckItemForCopyType(child, WithDiagrams, WithObjects, OnlyAttsOps, Doc);
					child = (MyListViewItem*)child->nextSibling();
				}
			}
		break;
	}//end switch	
}

/** Adds the children of a MyListViewItem to m_pItemDataList */
bool CUMLClipboard::InsertItemChildren(MyListViewItem * Item, CMyListViewItemList& SelectedItems)
{
	if(Item->childCount())
	{
		MyListViewItem * child = (MyListViewItem*)Item->firstChild();
		int type;
		while(child)
		{
			m_pItemDataList->append(child->getdata());
			type = child->getType();
			if(type == Uml::lvt_Actor || type == Uml::lvt_UseCase || type == Uml::lvt_Class)
    		{
    			m_pObjectList->append(child->getUMLObject());
    		}
			//if the child is selected, remove it from the list of selected items otherwise it will be
			//inserted twice in m_pObjectList
			if(child->isSelected())
			{
				SelectedItems.remove(SelectedItems.find(child) );
			}
			InsertItemChildren(child, SelectedItems);
			child = (MyListViewItem*)child->nextSibling();
		}
	}
	return true;
}/** Pastes the children of a MyListViewItem (The Parent) */


bool CUMLClipboard::pasteChildren(MyListViewItem* Parent, CMyListViewItemDataListIt* It,
																		CIDChangeLog& ChangeLog, UMLDoc * Doc)
{
	if(!It || !Parent || !Doc)
	{
      kdDebug()<<"Paste Children Error, maybe: no It, Parent or Doc"<<endl;
		return false;
	}
	CMyListViewItemData* itemdata = It->current();//itemdata gets the parent's info
	//of children to be added
	MyListViewItem* item = 0;
	int children = itemdata -> getChildren();
	for(int i = 0; i < children; i++)
 	{
		++(*It);
		itemdata = It->current();
		if(!itemdata)
		{
			kdDebug()<<"PasteChildren called with no itemData"<<endl;
			return false; //Error this function gets called only if the parent Item
									//has children and that means It->current() shouldn't be null
		}
 		item = Doc->listView->CreateItem(*itemdata, ChangeLog, Parent);
      int count = itemdata -> getChildren();
		if( item && count )
 		{
        kdDebug()<<count<<endl;
 			if(!pasteChildren(item, It, ChangeLog, Doc))
 			{
 				return false;
 			}
 		}
 	}


	return true;
}
/** Cleans the list of associations taking out the ones that point to an object not in 
m_pObjectList */
void CUMLClipboard::CleanAssociations(CAssociationWidgetDataList& AssociationDatas)
{


	CAssociationWidgetDataListIt it(AssociationDatas);
	CAssociationWidgetData* assoc = it.current();
	
	while(assoc)
	{
		++it;
		
		assoc = it.current();
	}
}

/** If clipboard has mime type application/x-uml-clip1,
Pastes the data from the clipboard into the current Doc */
bool CUMLClipboard::pasteClip1(UMLDoc * Doc, QMimeSource* Data)
{
	CMyListViewItemDataList itemdatalist;
	CUMLObjectList objects;

	objects.setAutoDelete(FALSE);
	CIDChangeLog* idchanges = 0;
	bool result = CUMLDrag::decode(Data, objects, itemdatalist, Doc);
   if(!result)
	{
		return false;
	}
	UMLObject *obj = 0;
  	CUMLObjectListIt object_it(objects);
  	while ( (obj=object_it.current()) != 0 )
  	{
  		++object_it;
  		if(!Doc->AddUMLObject(obj))
  		{
  			return false;

  		}
  		idchanges = Doc->getChangeLog();
  		if(!idchanges)
  		{
  			return false;
  		}
  	}


	MyListViewItem* item = 0;
	CMyListViewItemData* itemdata = 0;
  	CMyListViewItemDataListIt it(itemdatalist);
  	while ( (itemdata=it.current()) != 0 )
  	{
  		item = Doc->listView->CreateItem(*itemdata, *idchanges);
  		if(!item)
  		{
  			return false;
  		}
  		if(itemdata -> getChildren())
  		{
  			if(!pasteChildren(item, &it, *idchanges, Doc))
  			{
  				return false;
  			}

  		}
  		++it;
  	}

	return result;
}

/** If clipboard has mime type application/x-uml-clip2,
Pastes the data from the clipboard into the current Doc */
bool CUMLClipboard::pasteClip2(UMLDoc * Doc, QMimeSource* Data)
{

	CMyListViewItemDataList itemdatalist;
	CUMLObjectList objects;
	objects.setAutoDelete(FALSE);
	UMLViewDataList		views;
	CIDChangeLog* idchanges = 0;

	bool result = CUMLDrag::decode(Data, objects, itemdatalist, views, Doc);
	if(!result)
	{
		return false;
	}
	UMLObject *obj = 0;
  	CUMLObjectListIt object_it(objects);
  	idchanges = Doc->getChangeLog();
  	if(!idchanges)
  	{
  		return false;
  	}
  	while ( (obj=object_it.current()) != 0 )
  	{
  		++object_it;
  		if(!Doc->AddUMLObject(obj))
  		{
			kdDebug()<<"CUMLClipboard: error adding umlobject"<<endl;
       	return false;
  		}
  	}

	UMLViewData * pViewData = 0;
  	UMLViewDataListIt view_it( views );
  	while ( ( pViewData =view_it.current()) != 0 )
  	{
  		++view_it;
  		if( !Doc->AddUMLView( pViewData ) )
		{
			return false;
		}
  	}

	MyListViewItem* item = 0;
	CMyListViewItemData* itemdata = 0;
  	CMyListViewItemDataListIt it(itemdatalist);
  	while ( (itemdata=it.current()) != 0 )
  	{
  		item = Doc->listView->CreateItem(*itemdata, *idchanges);
  		if(!item)
  			{
  				return false;
  			}
  		if(itemdata -> getChildren())
  		{
  			if(!pasteChildren(item, &it, *idchanges, Doc))
  			{
  				return false;
  			}
  		}
  		++it;
  	}

	return result;
}

/** If clipboard has mime type application/x-uml-clip3,
Pastes the data from the clipboard into the current Doc */
bool CUMLClipboard::pasteClip3(UMLDoc * Doc, QMimeSource* Data)
{
	CMyListViewItemDataList itemdatalist;

	MyListViewItem* item = 0;
	CMyListViewItemData* itemdata = 0;
	CIDChangeLog* idchanges = Doc->getChangeLog();
	if(!idchanges)
	{
		return false;
	}

	bool result = CUMLDrag::decode(Data, itemdatalist);
	if(!result)
	{

		return false;
	}
  	CMyListViewItemDataListIt it(itemdatalist);
  	while ( (itemdata=it.current()) != 0 )
  	{
  		item = Doc->listView->CreateItem(*itemdata, *idchanges);
  		if(itemdata -> getChildren())
  		{
  			if(!pasteChildren(item, &it, *idchanges, Doc))
  			{
  				return false;
  			}
  		}
  		++it;
  	}

	return result;
}

/** If clipboard has mime type application/x-uml-clip4,
Pastes the data from the clipboard into the current Doc */
bool CUMLClipboard::pasteClip4(UMLDoc * Doc, QMimeSource* Data)
{
	if(!Doc->getCurrentView())
		return false;
	CMyListViewItemDataList itemdatalist;
	itemdatalist.setAutoDelete(FALSE);

	CUMLObjectList objects;
	objects.setAutoDelete(FALSE);


	CUMLWidgetDataList		widgetdatas;
	widgetdatas.setAutoDelete(FALSE);

	CAssociationWidgetDataList	assocs;
	assocs.setAutoDelete(FALSE);

	CIDChangeLog* idchanges = 0;

	Uml::Diagram_Type diagramType;


	if( !CUMLDrag::decode(Data, objects, itemdatalist, widgetdatas, assocs, diagramType, Doc) )
		return false;

	if( diagramType != Doc -> getCurrentView() -> getType() )
	{
		if( !checkPasteWidgets( widgetdatas ) )
			return false;
	}
  CUMLObjectListIt object_it(objects);
  idchanges = Doc->getChangeLog();
  if(!idchanges)
  		return false;
  //make sure the file we are pasting into has the objects
	//we need if there are widgets to be pasted
	UMLObject *obj = 0;
  while ( (obj=object_it.current()) != 0 )
  {
    	++object_it;
    	if(!Doc->AddUMLObject(obj))
    		return false;
  }
	//now add any widget we are want to paste
	Doc->getCurrentView()->BeginPartialWidgetPaste();
	CUMLWidgetData*	widgetdata =0;
  	CUMLWidgetDataListIt widgetdata_it(widgetdatas);
  	while ( (widgetdata=widgetdata_it.current()) != 0 )
  	{

   	++widgetdata_it;
		if( ! 	Doc->getCurrentView()->addWidget(widgetdata) )
		{
			Doc->getCurrentView()->EndPartialWidgetPaste();
			return false;
		}
  	}

	//now paste the associations
  CAssociationWidgetData* assoc;
  CAssociationWidgetDataListIt assoc_it(assocs);
  while ( (assoc=assoc_it.current()) != 0 )
  {
   		++assoc_it;
   		if( !Doc->getCurrentView()->addAssociation(assoc) )

			{
				Doc->getCurrentView()->EndPartialWidgetPaste();
				return false;
			}
  }
	//Activate all the pasted associations and widgets

	if(!Doc->getCurrentView()->Activate())
	{
		Doc->getCurrentView()->EndPartialWidgetPaste();
		return false;
	}
	Doc->getCurrentView()->EndPartialWidgetPaste();

  MyListViewItem* item = 0;
	CMyListViewItemData* itemdata = 0;
  CMyListViewItemDataListIt it(itemdatalist);
  while ( (itemdata=it.current()) != 0 )
  {
		item = Doc->listView->CreateItem(*itemdata, *idchanges);
  		if(!item)
			return false;
  		if(itemdata -> getChildren())
  			if(!pasteChildren(item, &it, *idchanges, Doc))
  				return false;
  		++it;
  }
	return true;
}

/** If clipboard has mime type application/x-uml-clip5,
Pastes the data from the clipboard into the current Doc */
bool CUMLClipboard::pasteClip5(UMLDoc * Doc, QMimeSource* Data)
{
	MyListViewItem* lvitem = dynamic_cast<MyListViewItem *>( Doc->listView->currentItem() );
	if(!lvitem || (lvitem->getType() != Uml::lvt_Class))
	{
		return false;
	}
	CMyListViewItemDataList itemdatalist;
	CUMLObjectList objects;
	objects.setAutoDelete(FALSE);
	CIDChangeLog* idchanges = 0;
	int I = 0;
	bool result = CUMLDrag::decode(Data, objects, itemdatalist, Doc, I);
   if(!result)
	{
		return false;
	}
	UMLObject *obj = 0;
  	CUMLObjectListIt object_it(objects);
	Doc->setModified(true);
	idchanges = Doc->getChangeLog();

  	while ( (obj=object_it.current()) != 0 )
  	{
  		++object_it;
		obj->setID(Doc->AssignNewID(obj->getID()));
		if(obj->getBaseType() == Uml::ot_Attribute)
		{
            ((UMLConcept*) lvitem->getUMLObject())->addAttribute((UMLAttribute*)obj, idchanges);
		}
		else if (obj->getBaseType() == Uml::ot_Operation)
		{
			  ((UMLConcept*) lvitem->getUMLObject())->addOperation((UMLOperation*)obj, idchanges);
		}
		else
		{
        kdDebug()<<"pasteClip5: Unknown type: not Att or Op"<<endl;
			return false;
		}
  	}

	MyListViewItem* item = 0;
	CMyListViewItemData* itemdata = 0;
  	CMyListViewItemDataListIt it(itemdatalist);
  	while ( (itemdata=it.current()) != 0 )
  	{
  		item = Doc->listView->CreateItem(*itemdata, *idchanges, lvitem);
  		++it;
  	}

	return result;
}

bool CUMLClipboard::insertItemDataChildren( MyListViewItem * item )
{
	if( item -> childCount() )
	{
		MyListViewItem * child =dynamic_cast<MyListViewItem *>( item -> firstChild() );
		while( child )
		{
			m_pItemDataList -> append( child->getdata() );
			insertItemDataChildren( child );
			child = dynamic_cast<MyListViewItem *>( child->nextSibling() );
		}
	}
	return true;
}

bool CUMLClipboard::checkPasteWidgets( CUMLWidgetDataList & widgetList )
{
	CUMLWidgetData * pData = 0;
	CUMLWidgetDataListIt it( widgetList );
	while ( ( pData = it.current()) != 0 )
	{
		++it;
		switch( pData -> getType() )
		{
			case Uml::wt_Note:
			break;

			case Uml::wt_Text:
				if( dynamic_cast<CFloatingTextData *>( pData )-> getRole() != Uml::tr_Floating )
					return false;
			break;

			default:
				return false;
			break;
		}
	}
	return true;
}



