#include "conceptwidget.h"
#include "concept.h"
#include "umlview.h"
#include <kdebug.h>

ConceptWidget::ConceptWidget(UMLView * view, UMLObject *o, CUMLWidgetData *pData) : UMLWidget(view,  o, pData)
{
	m_pMenu = 0;
	if( m_pObject )
	{
		calculateSize();
	 	update();
	}
}

ConceptWidget::ConceptWidget(UMLView * view, UMLObject *o) : UMLWidget(view,o, new CConceptWidgetData())
{
	init();
   setSize(100,30);
	calculateSize();
	m_pData->setType(wt_Class);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ConceptWidget::ConceptWidget(UMLView * view) : UMLWidget(view, new CConceptWidgetData())
{
	init();
	setSize(100,30);
	m_pData->setType(wt_Class);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::init()
{   
	m_pMenu = 0;
  //set defaults from m_pView
  if(m_pView && m_pData)
  {
    //check to see if correct
    SettingsDlg::OptionState ops = m_pView -> getOptionState();

    setShowAttSigs( ops.classState.showAttSig );
    setShowOpSigs( ops.classState.showOpSig );

    ( (CConceptWidgetData *) m_pData ) -> m_bShowAttributes = ops.classState.showAtts;
    ( (CConceptWidgetData *) m_pData ) -> m_bShowOperations = ops.classState.showOps;
    ( (CConceptWidgetData *) m_pData ) -> m_bShowPackage = ops.classState.showPackage ;
    ( (CConceptWidgetData *) m_pData ) -> m_bShowStereotype = ops.classState.showStereoType;
    ( (CConceptWidgetData *) m_pData ) -> m_bShowScope = ops.classState.showScope ;
    updateSigs();
	}
	//maybe loading and this may not be set.
	if( m_pObject )
	{
		calculateSize();
	 	update();
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ConceptWidget::~ConceptWidget()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::draw(QPainter & p, int offsetX, int offsetY)
{
	p.setPen(m_LineColor);
	if(m_pData->getUseFillColor())
		p.setBrush(m_FillColor);
  else
     p.setBrush(m_pView -> viewport() -> backgroundColor());
	int na = ((UMLConcept*)m_pObject)->attributes();
	int w = width();
	int h = height();
	QFontMetrics fm = QFontMetrics( m_pData -> getFont() );
	int fontHeight  = fm.lineSpacing();
   QString name;
   if( ((CConceptWidgetData*)m_pData)->m_bShowPackage )
   	name = m_pObject -> getPackage() + "::" + this -> GetName();

   else name = this -> GetName();

	p.drawRect(offsetX, offsetY, w, h);
	p.setPen(QPen(black));
	if(!((CConceptWidgetData*)m_pData)->m_bShowOperations &&
			! ((CConceptWidgetData*)m_pData)->m_bShowAttributes &&
			! ((CConceptWidgetData*)m_pData)->m_bShowStereotype)
	{
		QFont font = m_pData -> getFont();
		font.setBold( true ); 
		p.setFont( font );
		p.drawText(offsetX + MARGIN, offsetY, w - MARGIN * 2, h, AlignCenter, name);
		font.setBold( false );
		p.setFont( font );
	}
	else
	{
		if(((CConceptWidgetData*)m_pData)->m_bShowStereotype)
		{
			QFont f( m_pData -> getFont() );
			f.setBold( true );
			p.setFont( f );
			p.drawText(offsetX + MARGIN, offsetY ,w-MARGIN * 2,fontHeight,AlignCenter, "<< " + m_pObject -> getStereotype() + " >>");
			f.setItalic( m_pObject -> getAbstract() );
			p.setFont( f );
			p.drawText(offsetX + MARGIN, offsetY + fontHeight,w-MARGIN * 2,fontHeight,AlignCenter, name);
			f.setItalic( false );
			f.setBold( false );
			p.setFont( f );
			p.setPen(m_LineColor);
			p.drawLine(offsetX , offsetY + fontHeight * 2, offsetX + w - 1, offsetY + fontHeight * 2);
		}
		else
		{
			QFont f( m_pData -> getFont() );
			f.setBold( true );
			f.setItalic( m_pObject -> getAbstract() );
			p.setFont( f );
			p.drawText(offsetX + MARGIN, offsetY, w-MARGIN * 2,fontHeight,AlignCenter, name);
			f.setItalic( false );
			f.setBold( false );
			p.setFont( f );
			p.setPen(m_LineColor);
			p.drawLine(offsetX , offsetY + fontHeight, offsetX + w - 1, offsetY + fontHeight);
		}
	}
	int aStart = fontHeight;
	int oStart = na * fontHeight + aStart;
	if(((CConceptWidgetData*)m_pData)->m_bShowStereotype)
	{
		aStart += fontHeight;
		oStart += fontHeight;
	}


	if(na==0)
		oStart = aStart + fontHeight / 2;
	int y;
	//change so only going through list once
	p.setPen(QPen(black));
	if(((CConceptWidgetData*)m_pData)->m_bShowAttributes)
	{
		y = aStart;
		UMLAttribute * obj=0;
		QList<UMLAttribute> *list = ((UMLConcept*)m_pObject)->getAttList();
		for(obj=list->first();obj != 0;obj=list->next())
	 	{
	 		p.drawText(offsetX + MARGIN, offsetY + y, w-MARGIN * 2, fontHeight, AlignVCenter, obj-> toString(((CConceptWidgetData*)m_pData)->m_ShowAttSigs));
	 		y+=fontHeight;
	 	}

	}//end if att
	if(((CConceptWidgetData*)m_pData)->m_bShowOperations)	
	{
      QFont f( p.font() );
		y = oStart;	
	   	p.setPen(m_LineColor);
		if(((CConceptWidgetData*)m_pData)->m_bShowAttributes)
			p.drawLine(offsetX, offsetY + y, offsetX + w - 1, offsetY + y);
		else
			y = aStart;
		UMLOperation * obj = 0;
		QList<UMLOperation> *list = ((UMLConcept*)m_pObject)->getOpList();
		for(obj=list->first();obj != 0;obj=list->next())
	 	{
	 		QString op = obj -> toString(((CConceptWidgetData*)m_pData)->m_ShowOpSigs);
	 		p.setPen(QPen(black));
          f.setItalic( obj -> getAbstract() );
          p.setFont( f );
	 		p.drawText(offsetX + MARGIN, offsetY + y, w-MARGIN * 2,fontHeight,AlignVCenter,op);
          f.setItalic( false );
          p.setFont( f );
	 		y+=fontHeight;
	 	}
	}//end if op
	if(m_bSelected)
		drawSelected(&p, offsetX, offsetY);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::calculateSize()
{
	if( !m_pData || !m_pObject)
		return;
	int width, height;
	QFont font = m_pData -> getFont();
	font.setBold( true );//use bold for all calculations
	font.setItalic( true );
	QFontMetrics fm = QFontMetrics( font );
	int fontHeight  = fm.lineSpacing();

	int lines = 1;//always have one line - for name
	int numAtts = ((UMLConcept*)m_pObject)->attributes();
	int numOps = ((UMLConcept*)m_pObject)->operations();
	
	if(((CConceptWidgetData*)m_pData)->m_bShowStereotype)
		lines++;

	height = width = 0;
	//set the height of the concept
	if(((CConceptWidgetData*)m_pData)->m_bShowAttributes)
	{
		lines += numAtts;
		if(numAtts == 0)
			height += fontHeight / 2;//no atts, so just add a bit of space
	}

	if(((CConceptWidgetData*)m_pData)->m_bShowOperations)
	{
		lines += numOps;
		if(numOps == 0)

			height += fontHeight / 2;//no ops, so just add a but of space
	}
	height += lines * fontHeight;
	//now set the width of the concept
	//set width to name to start with
	if(((CConceptWidgetData*)m_pData)->m_bShowPackage)
		width = fm.width(m_pObject -> getPackage() + "::" + GetName());
	else
		width = fm.width(GetName());
	int w = fm.width("<< " + m_pObject -> getStereotype() + " >>");


	width = w > width?w:width;
	
	if(((CConceptWidgetData*)m_pData)->m_bShowAttributes)
	{
		QList<UMLAttribute> * list = ((UMLConcept*)m_pObject)->getAttList();
		UMLAttribute * a = 0;
		for(a = list->first();a != 0; a = list->next())
		{
			int w = fm.width(a -> toString(((CConceptWidgetData*)m_pData)->m_ShowAttSigs));
			width = w > width?w:width;
		}
	}
	if(((CConceptWidgetData*)m_pData)->m_bShowOperations)
	{		
		QList<UMLOperation> * list = ((UMLConcept*)m_pObject)->getOpList();
		UMLOperation * o = 0;
		for(o = list->first();o != 0; o = list->next())
		{
			int w = fm.width(o -> toString(((CConceptWidgetData*)m_pData)->m_ShowOpSigs));
			width = w > width?w:width;
		}
	}
	//allow for width margin
	width += MARGIN * 2;
	if(!((CConceptWidgetData*)m_pData)->m_bShowOperations &&
			!((CConceptWidgetData*)m_pData)->m_bShowAttributes &&
			!((CConceptWidgetData*)m_pData)->m_bShowStereotype)
	{
		height += MARGIN * 2;
	}
	setSize(width, height);
	AdjustAssocs( (int)x(), (int)y() );//adjust assoc lines
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::slotMenuSelection(int sel)
{

	switch(sel)
	{
		case ListPopupMenu::mt_Attribute:
		case ListPopupMenu::mt_Operation:
			m_pView->getDocument() -> createUMLObject(m_pObject, ListPopupMenu::convert_MT_OT( (ListPopupMenu::Menu_Type)sel) );
		break;
		
		case ListPopupMenu::mt_Show_Operations:
			((CConceptWidgetData*)m_pData)->m_bShowOperations = ((CConceptWidgetData*)m_pData)->m_bShowOperations?false:true;
			updateSigs();
			calculateSize();

			update();
		break;
		
		case ListPopupMenu::mt_Show_Attributes:
			((CConceptWidgetData*)m_pData)->m_bShowAttributes = ((CConceptWidgetData*)m_pData)->m_bShowAttributes?false:true;
			updateSigs();
			calculateSize();
			update();

		break;
		
		case ListPopupMenu::mt_Show_Operation_Signature:
			if(((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_ShowSig || ((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_SigNoScope)
			{
				if(((CConceptWidgetData*)m_pData)->m_bShowScope)
					((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSig;
				else
					((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSigNoScope;
			}
			else if(((CConceptWidgetData*)m_pData)->m_bShowScope)
				((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_ShowSig;

			else
				((CConceptWidgetData*)m_pData)->m_ShowOpSigs =	Uml::st_SigNoScope;			
			calculateSize();
			update();
		break;
		
		case ListPopupMenu::mt_Show_Attribute_Signature:
			if(((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_ShowSig ||
					((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_SigNoScope)
			{
				if(((CConceptWidgetData*)m_pData)->m_bShowScope)


					((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSig;
				else

					((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSigNoScope;
			}

			else if(((CConceptWidgetData*)m_pData)->m_bShowScope)
				((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_ShowSig;
			else
				((CConceptWidgetData*)m_pData)->m_ShowAttSigs =	Uml::st_SigNoScope;		
			calculateSize();
			update();
		break;
		
		case ListPopupMenu::mt_Scope:
			((CConceptWidgetData*)m_pData)->m_bShowScope = ((CConceptWidgetData*)m_pData)->m_bShowScope?false:true;
			updateSigs();
			calculateSize();
			update();
		break;
	}
	UMLWidget::slotMenuSelection(sel);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::updateSigs()
{
	if(((CConceptWidgetData*)m_pData)->m_bShowScope)
	{
		if(((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_NoSigNoScope)
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSig;
		else if(((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_SigNoScope)
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_ShowSig;
			
		if(((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_NoSigNoScope)
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSig;
		else if(((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_SigNoScope)
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_ShowSig;
	}
	else
	{
		 if(((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_ShowSig)
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_SigNoScope;
		else if(((CConceptWidgetData*)m_pData)->m_ShowOpSigs == Uml::st_NoSig)
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSigNoScope;
			
		if(((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_ShowSig)
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_SigNoScope;
		else if(((CConceptWidgetData*)m_pData)->m_ShowAttSigs == Uml::st_NoSig)
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSigNoScope;
	}
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setShowScope(bool _scope)
{
	((CConceptWidgetData*)m_pData)->m_bShowScope = _scope;
	updateSigs();
	calculateSize();
	update();	
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setShowAtts(bool _show)
{
	((CConceptWidgetData*)m_pData)->m_bShowAttributes = _show;
	updateSigs();

	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setShowOps(bool _show)
{
	((CConceptWidgetData*)m_pData)->m_bShowOperations = _show;
	updateSigs();
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setOpSignature(Signature_Type sig)
{
	((CConceptWidgetData*)m_pData)->m_ShowOpSigs = sig;
	updateSigs();
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setAttSignature(Signature_Type sig)
{
	((CConceptWidgetData*)m_pData)->m_ShowAttSigs = sig;
	updateSigs();
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setShowStereotype(bool _status)
{
	((CConceptWidgetData*)m_pData)->m_bShowStereotype = _status;
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConceptWidget::setShowPackage(bool _status)
{
	((CConceptWidgetData*)m_pData)->m_bShowPackage = _status;
	calculateSize();
	update();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
bool ConceptWidget::Activate(CIDChangeLog* ChangeLog /* = 0 */)
{
	bool status = UMLWidget::Activate(ChangeLog);
	if(status)
	{
		calculateSize();
	}


	return status;
}

/** Synchronizes the Widget's m_pData member with its display properties, for example:

 the X and Y positions of the widget, etc */
void ConceptWidget::SynchronizeData()
{
	//Nothing to synchronize
	UMLWidget::SynchronizeData();
}

void ConceptWidget::setShowAttSigs(bool _status)
{
	if( !_status )
	{
		if(((CConceptWidgetData*)m_pData)->m_bShowScope)
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSig;
		else
			((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_NoSigNoScope;
	}

	else if(((CConceptWidgetData*)m_pData)->m_bShowScope)
		((CConceptWidgetData*)m_pData)->m_ShowAttSigs = Uml::st_ShowSig;
	else
		((CConceptWidgetData*)m_pData)->m_ShowAttSigs =	Uml::st_SigNoScope;		
	calculateSize();
	update();
}

void ConceptWidget::setShowOpSigs(bool _status)
{
	if( !_status )
	{
		if(((CConceptWidgetData*)m_pData)->m_bShowScope)
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSig;
		else
			((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_NoSigNoScope;

	}
	else if(((CConceptWidgetData*)m_pData)->m_bShowScope)
		((CConceptWidgetData*)m_pData)->m_ShowOpSigs = Uml::st_ShowSig;
	else
		((CConceptWidgetData*)m_pData)->m_ShowOpSigs =	Uml::st_SigNoScope;			
	calculateSize();
	update();
}
