#include <qstrlist.h>
#include <qlayout.h>
#include <qobjcoll.h>
#include <qstringlist.h>

#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>

#include "../umldoc.h"
#include "classattpage.h"
#include "classpropdlg.h"
#include "umlattributedialog.h"

ClassAttPage::ClassAttPage(QWidget *parent, UMLConcept * c, UMLDoc * doc) : QWidget(parent)
{
	m_bSigWaiting = false;
	m_pDoc = doc;
	m_pConcept = c;
	m_pMenu = 0;
	int margin = fontMetrics().height();
	setMinimumSize(310,330);
	
	QVBoxLayout * mainLayout = new QVBoxLayout( this );
	mainLayout -> setSpacing(10);
	mainLayout -> setMargin(margin);
	
	m_pAttsGB = new QGroupBox(i18n("UMLAttributes"), this );
	QHBoxLayout * attsLayout = new QHBoxLayout( m_pAttsGB );
	m_pAttsLB = new QListBox(m_pAttsGB );
	attsLayout -> setMargin(margin);
	attsLayout -> setSpacing ( 10 );
	attsLayout -> addWidget(m_pAttsLB);

	QVBoxLayout * buttonLayout = new QVBoxLayout( attsLayout );
	m_pUpArrowB = new KArrowButton( m_pAttsGB );
	m_pUpArrowB -> setEnabled( false );
	buttonLayout -> addWidget( m_pUpArrowB );

	m_pDownArrowB = new KArrowButton( m_pAttsGB, Qt::DownArrow );
	m_pDownArrowB -> setEnabled( false );
	buttonLayout -> addWidget( m_pDownArrowB );
	mainLayout -> addWidget(m_pAttsGB);

	m_pDocGB = new QGroupBox(i18n("Documentation"), this);
	QVBoxLayout * docLayout = new QVBoxLayout( m_pDocGB );
	m_pDocTE = new QTextEdit( m_pDocGB );
	docLayout -> setMargin(margin);
	docLayout -> setSpacing ( 10 );
	docLayout -> addWidget( m_pDocTE );
	mainLayout -> addWidget(m_pDocGB);
	
	//add attributes to list
	UMLAttribute *a;
	m_pAttList = c -> getAttList();
	for(a=m_pAttList->first();a != 0;a=m_pAttList->next())
	 		m_pAttsLB -> insertItem(a->GetName());

	enableWidgets(false);//disable widgets until an att is chosen
	m_pOldAtt = 0;
	connect(m_pAttsLB, SIGNAL(clicked(QListBoxItem *)), this, SLOT(slotClicked(QListBoxItem *)));
	
	connect(m_pAttsLB, SIGNAL(rightButtonPressed(QListBoxItem *, const QPoint &)),
		this, SLOT(slotRightButtonPressed(QListBoxItem *, const QPoint &)));
		
	connect(m_pAttsLB, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)),
		this, SLOT(slotRightButtonClicked(QListBoxItem *, const QPoint &)));
	connect(m_pDoc, SIGNAL(sigChildObjectCreated(UMLObject *)), this, SLOT(slotAttributeCreated(UMLObject *)));
	connect(m_pDoc, SIGNAL(sigChildObjectChanged(UMLObject *)), this, SLOT(slotAttributeRenamed(UMLObject *)));

	connect( m_pUpArrowB, SIGNAL( clicked() ), this, SLOT( slotUpClicked() ) );
	connect( m_pDownArrowB, SIGNAL( clicked() ), this, SLOT( slotDownClicked() ) );
	connect( m_pAttsLB, SIGNAL( doubleClicked( QListBoxItem * ) ), this, SLOT( slotDoubleClick( QListBoxItem * ) ) );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ClassAttPage::~ClassAttPage()
{


}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::enableWidgets(bool state)
{
	m_pDocTE -> setEnabled( state );
	//if disabled clear contents
	if( !state )
	{
		m_pDocTE -> setText( "" );
		m_pUpArrowB -> setEnabled( false );
		m_pDownArrowB -> setEnabled( false );
		return;
	}
	/*now check the order buttons.
		Double check an item is selected
	   If only one att. in list make sure there disabled.
		If at top item,only allow down arrow to be enabled.
		If at bottom item. only allow up arrow to be enabled.
	*/
	int index = m_pAttsLB -> currentItem();
	if( m_pAttsLB -> count() == 1 || index == -1 )
	{
		m_pUpArrowB -> setEnabled( false );
		m_pDownArrowB -> setEnabled( false );
	}	
	else if( index == 0 )
	{
		m_pUpArrowB -> setEnabled( false );
		m_pDownArrowB -> setEnabled( true );
	}
	else if( index == (int)m_pAttsLB -> count() - 1 )
	{
		m_pUpArrowB -> setEnabled( true );
		m_pDownArrowB -> setEnabled( false );
	}
	else
	{
		m_pUpArrowB -> setEnabled( true );
		m_pDownArrowB -> setEnabled( true );
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotClicked(QListBoxItem *item)
{
	//make sure clicked on an item
	if(!item)
	{
		enableWidgets(false);
		m_pOldAtt = 0;
		m_pAttsLB -> clearSelection();
		return;
	}

	QString name = m_pAttsLB ->currentText();
	UMLAttribute * pAtt = m_pAttList -> at( m_pAttsLB -> index( item ) );
	//if not first time an item is highlighted
	//save old highlighted item first
	if(m_pOldAtt)
	{
		m_pOldAtt -> setDoc( m_pDocTE -> text() );
	}//end if old att
	//now update screen
	m_pDocTE -> setText( pAtt -> getDoc() );
	enableWidgets(true);
	m_pOldAtt = pAtt;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::updateObject()
{
	QListBoxItem *i = m_pAttsLB -> item(m_pAttsLB -> currentItem());
	slotClicked(i);
	QStringList stringList;	
	int count = m_pAttsLB -> count();
	for( int j = 0; j < count ; j++ )
		stringList.append( m_pAttsLB -> text( j ) );
/*	if( count > 0 )
		m_pConcept -> reorderAttributes( stringList );             */
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotAttributeCreated(UMLObject * object)
{
	if(!m_bSigWaiting)
		return;
	int index = m_pAttsLB -> count();
	m_pAttsLB ->insertItem(object -> GetName(), index);
	m_bSigWaiting = false;
	slotClicked(0);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotAttributeRenamed(UMLObject * object)
{

	if(!m_bSigWaiting)
		return;
	int index = m_pAttsLB -> currentItem();
	m_pAttsLB -> changeItem(object -> GetName(), index);
	m_bSigWaiting = false;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotRightButtonClicked(QListBoxItem * /*item*/, const QPoint &/* p*/)
{
	if(m_pMenu)
	{
		m_pMenu -> hide();
		disconnect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotPopupMenuSel(int)));
		delete m_pMenu;
		m_pMenu = 0;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotRightButtonPressed(QListBoxItem * item, const QPoint & p)

{
	ListPopupMenu::Menu_Type type = ListPopupMenu::mt_Undefined;
	if(item)//pressed on an item
	{
		type = ListPopupMenu::mt_Attribute_Selected;
	}
	else//pressed into fresh air
	{
		type = ListPopupMenu::mt_New_Attribute;
	}
	if(m_pMenu)
	{
		m_pMenu -> hide();
		disconnect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotPopupMenuSel(int)));
		delete m_pMenu;
		m_pMenu = 0;
	}
	m_pMenu = new ListPopupMenu(this, type);


	m_pMenu->popup(p);
	connect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotPopupMenuSel(int)));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassAttPage::slotPopupMenuSel(int id)
{
	QString s = m_pAttsLB ->currentText();
	UMLAttribute * a = m_pAttList -> at( m_pAttsLB -> currentItem() );
	if(!a && id != ListPopupMenu::mt_New_Attribute)
	{
		kdDebug() << "can't find att from selection" << endl;
		return;
	}
	switch(id)
	{
		case ListPopupMenu::mt_New_Attribute:
			m_bSigWaiting = true;
			m_pDoc -> createUMLObject(m_pConcept, Uml::ot_Attribute);
		break;
		
		case ListPopupMenu::mt_Delete:
			//should really wait for signal back
			//but really shouldn't matter
			m_pDoc -> removeUMLObject(a);
			m_pAttsLB -> removeItem( m_pAttsLB -> currentItem());
			slotClicked(0);
		break;
		
		case ListPopupMenu::mt_Rename:
			m_bSigWaiting = true;
			m_pDoc -> renameChildUMLObject(a);
		break;
		
		case ListPopupMenu::mt_Properties:
			slotDoubleClick( m_pAttsLB -> item( m_pAttsLB -> currentItem() ) );
		break;
	}
}

void ClassAttPage::slotUpClicked()
{
	int count = m_pAttsLB -> count();
	int index = m_pAttsLB -> currentItem();
	//shouldn't occur, but just in case
	if( count <= 1 || index <= 0 )
		return;

	//swap the text around ( meaning attributes )
	QString aboveString = m_pAttsLB -> text( index - 1 );
	QString currentString = m_pAttsLB -> text( index );
	m_pAttsLB -> changeItem( currentString, index -1 );  
	m_pAttsLB -> changeItem( aboveString, index );
	//set the moved atttribute selected
	QListBoxItem * item = m_pAttsLB -> item( index - 1 );
	m_pAttsLB -> setSelected( item, true );
	//now change around in the list
	UMLAttribute * aboveAtt = m_pAttList -> at( index - 1 );
	UMLAttribute * currentAtt = m_pAttList -> take( index );
	m_pAttList -> insert( m_pAttList -> findRef( aboveAtt ), currentAtt ); 
	slotClicked( item );
}

void ClassAttPage::slotDownClicked()
{   
	int count = m_pAttsLB -> count();
	int index = m_pAttsLB -> currentItem();
	//shouldn't occur, but just in case
	if( count <= 1 || index >= count - 1 )
		return;

	//swap the text around ( meaning attributes )
	QString belowString = m_pAttsLB -> text( index + 1 );
	QString currentString = m_pAttsLB -> text( index );
	m_pAttsLB -> changeItem( currentString, index + 1 );  
	m_pAttsLB -> changeItem( belowString, index );
	//set the moved atttribute selected
	QListBoxItem * item = m_pAttsLB -> item( index + 1 );
	m_pAttsLB -> setSelected( item, true );
	//now change around in the list
	UMLAttribute * aboveAtt = m_pAttList -> at( index + 1 );
	UMLAttribute * currentAtt = m_pAttList -> take( index );
	m_pAttList -> insert( m_pAttList -> findRef( aboveAtt ) + 1, currentAtt ); 
	slotClicked( item );
}

void ClassAttPage::slotDoubleClick( QListBoxItem * item )
{
	if( !item )
		return;
	QString name = item -> text();
	UMLAttribute * pAtt  = m_pAttList -> at( m_pAttsLB -> index( item ) );
	if( !pAtt )
	{
		kdDebug() << "can't find att from selection" << endl;
		return;
	}
	UMLAttributeDialog dlg( this, pAtt );
	if( dlg.exec() )
		m_pAttsLB -> changeItem( pAtt -> GetName(), m_pAttsLB -> index( item ) );
}






