#ifndef CLASSOPSPAGE_H
#define CLASSOPSPAGE_H

#include <qwidget.h>
#include <qgroupbox.h>
#include <qlistbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qtextedit.h>

#include <karrowbutton.h>

#include "../concept.h"
#include "../listpopupmenu.h"
class UMLDoc;

/**
	*	A dialog page to display class operation properties.  This is not normally setup
	*	called by you.  It is used by the @ref ClassPropDlg.
	*

	*	@short	A dialog page to display class operation properties.
 	*	@author Paul Hensgen <phensgen@techie.com>
  *	@version	1.0
  *	@see	ClassPropDlg
  */

class ClassOpsPage : public QWidget
{
Q_OBJECT
public:

	/**
		*	Sets up the ClassOpsPage.
		*
		*	@param	parent	The parent to the ClassOpsPage.
		*	@param	c	The Concept to display the properties of.
		*/
	ClassOpsPage(QWidget *parent, UMLConcept *c, UMLDoc * doc);
	
	/**
		*	Standard deconstructor.
		*/
	~ClassOpsPage();

	
	/**
		*	Will move information from the dialog into the object.
		*	Call when the ok or apply button is pressed.
		*/
	void updateObject();
	
private:
	UMLConcept * m_pConcept;		
	QGroupBox * m_pOpsGB, * m_pDocGB;
	QListBox * m_pOpsLB;
	QTextEdit * m_pDocTE;
	KArrowButton * m_pUpArrowB, * m_pDownArrowB;
	
	UMLOperation * m_pOldOp;
	ListPopupMenu * m_pMenu;
	UMLDoc * m_pDoc;
	bool m_bSigWaiting;
	QPtrList<UMLOperation> * m_pOpList;
	/**
		*	Set the state of the widgets on the page with the given value.
		*
		*	@param	state	The state to set the widgets as.
		*/
	void enableWidgets(bool state);
	
signals:
	void sigUpdateChildObject(int id);
public slots:
	void slotClicked(QListBoxItem *i);
	void slotDoubleClick( QListBoxItem * item );
	void slotOpPopupMenuSel(int id);
	void slotOpRightButtonPressed(QListBoxItem *item, const QPoint &p);
	void slotOpRightButtonClicked(QListBoxItem *item, const QPoint &p);
	void slotOperationCreated(UMLObject * object);
	void slotOperationRenamed(UMLObject * object);
	void slotUpClicked();
	void slotDownClicked();
};

#endif
