#include <qlayout.h>

#include <klocale.h>
#include <kdebug.h>

#include "classpropdlg.h"
#include "../uml.h"
#include "../umlview.h"

ClassPropDlg::ClassPropDlg(QWidget *parent, UMLObject * c, int pageNum, bool assoc) : KDialogBase(IconList, i18n("Properties"), Ok | Apply | Cancel | Help,
	Ok, parent, "_CLASSDLG_", true, true)
{
	m_pWidget = 0;
	m_pGenPage = 0;
	m_pAttPage = 0;
	m_pOpsPage = 0;
	m_pOptionsPage = 0;
	m_Type = pt_Object;
	m_pDoc = ((UMLApp *)parent) -> getDocument();
	m_pObject = c;
	setupPages(c, assoc);
	showPage(pageNum);		
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ClassPropDlg::ClassPropDlg(QWidget *parent, ObjectWidget * o) : KDialogBase(IconList, i18n("Properties"), Ok | Apply | Cancel | Help,
	Ok, parent, "_CLASSDLG_", true, true)
{
	m_pWidget = o;
	m_pGenPage = 0;
	m_pAttPage = 0;
	m_pOpsPage = 0;
	m_pOptionsPage = 0;
	m_Type = pt_ObjectWidget;
	m_pObject = 0;
	m_pDoc = ((UMLApp *)parent) -> getDocument();
	QFrame *page = addPage( i18n("General"), i18n("General settings"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
	page -> setMinimumSize(310, 330);
	QHBoxLayout * topLayout = new QHBoxLayout(page);
	m_pGenPage = new ClassGenPage(m_pDoc, page, o);
	topLayout -> addWidget(m_pGenPage);

	QFrame * newPage = addPage( i18n("Color"), i18n("Color"), m_IconLoader.loadIcon( "colors", KIcon::Desktop ) );
	QHBoxLayout * m_pColorLayout = new QHBoxLayout(newPage);
	m_pColorPage = new UMLWidgetColorPage(newPage, o);
	m_pColorLayout -> addWidget(m_pColorPage);

	setupFontPage();
	showPage(0);	
	setMinimumSize(340,420);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ClassPropDlg::ClassPropDlg(QWidget *parent, UMLWidget * w) : KDialogBase(IconList, i18n("Properties"), Ok | Apply | Cancel | Help,
	Ok, parent, "_CLASSDLG_", true, true)
{
	m_pWidget = w;
	m_pGenPage = 0;
	m_pAttPage = 0;
	m_pOpsPage = 0;
	m_pOptionsPage = 0;
	m_Type = pt_Widget;
	m_pObject = w -> getUMLObject();
	m_pDoc = ((UMLApp *)parent) -> getDocument();
	if(w -> getBaseType() == Uml::wt_Class)
		setupPages(m_pObject, true);
	else setupPages(m_pObject);
	//now setup the options page for classes
	if( w -> getBaseType() == Uml::wt_Class )
	{
		QFrame * newPage = addPage( i18n("Options"), i18n("Misc. Options"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
		QHBoxLayout * m_pOptionsLayout = new QHBoxLayout(newPage);
		m_pOptionsPage = new ClassOptionsPage(newPage, w);
		m_pOptionsLayout -> addWidget(m_pOptionsPage);
	}

	QFrame * colorPage = addPage( i18n("Color"), i18n("Color"), m_IconLoader.loadIcon( "colors", KIcon::Desktop ) );
	QHBoxLayout * m_pColorLayout = new QHBoxLayout(colorPage);
	m_pColorPage = new UMLWidgetColorPage(colorPage, w);
	m_pColorLayout -> addWidget(m_pColorPage);
	setupFontPage();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ClassPropDlg::~ClassPropDlg()
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassPropDlg::slotOk()
{
	if( m_Type == pt_Object && m_pObject -> getBaseType() == Uml::ot_Concept)
	{
		m_pGenPage -> updateObject();
		m_pAttPage -> updateObject();	
		m_pOpsPage -> updateObject();
	}
	else if(m_Type == pt_Widget)
	{
		m_pGenPage -> updateObject();
		if(m_pAttPage)
			m_pAttPage -> updateObject();	
		if(m_pOpsPage)
			m_pOpsPage -> updateObject();
		if( m_pOptionsPage )
			m_pOptionsPage -> updateUMLWidget();
		m_pColorPage -> updateUMLWidget();
	}
	else
	{
		m_pGenPage -> updateObject();
		if(m_pOptionsPage)
			m_pOptionsPage -> updateUMLWidget();
		m_pColorPage -> updateUMLWidget();
	}
	if( m_pWidget )
	{
		m_pWidget -> setFont( m_pChooser -> font() );
	}	
	KDialogBase::accept();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassPropDlg::slotUpdateChildObject(int id)
{
	UMLObject * o = ((UMLConcept*)m_pObject) -> findChildObject(id);
	if(o)
		m_pDoc -> signalChildUMLObjectUpdate(o);
	else
		kdDebug() << "slot dlg: can't find child" << endl;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassPropDlg::slotApply()
{
	switch(activePageIndex())
	{
		case page_gen:
			m_pGenPage -> updateObject();
		break;
		
		case page_att:
			//it may be after options page on a use case or actor as
			//it will then be page 2. so check
			if(m_pAttPage)
				m_pAttPage -> updateObject();
			else
				m_pOptionsPage -> updateUMLWidget();
		break;
		
		case page_op:
			m_pOpsPage -> updateObject();
		break;
		
		case page_options:
			m_pOptionsPage -> updateUMLWidget();
		break;
		
		case page_color:
			m_pColorPage -> updateUMLWidget();
		break;

		case page_font: //page number will chang so make it default
			if( m_pWidget )
				m_pWidget -> setFont( m_pChooser -> font() );	
		break;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ClassPropDlg::setupPages(UMLObject * c, bool assoc)
{
	QFrame *page = addPage(i18n("General"), i18n("General settings"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
	QHBoxLayout * genLayout = new QHBoxLayout(page);
	page -> setMinimumSize(310, 330);
	m_pGenPage = new ClassGenPage(m_pDoc, page, c);
	genLayout -> addWidget(m_pGenPage);
	//add extra pages for concept
	if(c->getBaseType() == Uml::ot_Concept)
	{
		QFrame *newPage = addPage(i18n("Attributes"), i18n("Attribute settings"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
		m_pAttPage = new ClassAttPage(newPage, (UMLConcept*)c, m_pDoc);
		QHBoxLayout * attLayout = new QHBoxLayout(newPage);
		attLayout -> addWidget(m_pAttPage);
		connect(m_pAttPage, SIGNAL(sigUpdateChildObject(int)), this, SLOT(slotUpdateChildObject(int)));
		
		newPage = addPage(i18n("Operations"), i18n("Operation settings"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
		m_pOpsPage = new ClassOpsPage(newPage, (UMLConcept*)c, m_pDoc);
		QHBoxLayout * m_pOpsLayout = new QHBoxLayout(newPage);
		m_pOpsLayout -> addWidget(m_pOpsPage);
		connect(m_pOpsPage, SIGNAL(sigUpdateChildObject(int)), this, SLOT(slotUpdateChildObject(int)));
		if(assoc)
		{
			newPage = addPage(i18n("Associations"), i18n("Class associations"), m_IconLoader.loadIcon( "misc", KIcon::Desktop ) );
			m_pAssocPage = new AssocPage(newPage, m_pDoc -> getCurrentView(), m_pObject);
			QHBoxLayout * assocLayout = new QHBoxLayout(newPage);
			assocLayout -> addWidget(m_pAssocPage);
		}
		else m_pAssocPage = 0;
	}
}

void ClassPropDlg::setupFontPage()
{
	if( !m_pWidget )
		return;
	QVBox * page = addVBoxPage( i18n("Font"), i18n("Font settings"), m_IconLoader.loadIcon( "fonts", KIcon::Desktop  )  );	
	m_pChooser = new KFontChooser( (QWidget*)page, "font", false, QStringList(), false);
	m_pChooser -> setFont( m_pWidget -> getFont() );
	m_pChooser -> setSampleText( " UML Modeler font" );
}


