/***************************************************************************
                          codegenerationwizard.cpp  -  description
                             -------------------
    begin                : Wed Jul 24 2002
    copyright            : (C) 2002 by Paul Hensgen
    email                : phensgen@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlistview.h>
#include <kdebug.h>
#include <klocale.h>

#include "codegenerationwizard.h"
#include "codegenerationoptionspage.h"
#include "../concept.h"
#include "../codegenerator.h"
#include "../umldoc.h"

CodeGenerationWizard::CodeGenerationWizard(CodeGenerator *generator, UMLDoc *doc, QPtrList<UMLConcept> *classList,
                                           SettingsDlg::CodeGenState codegenState, QWidget *parent, const char *name)
                     :CodeGenerationWizardBase(parent,name)
{
  m_generator = generator;
  m_doc = doc;
  m_availableList -> setAllColumnsShowFocus(true);
  m_availableList -> setResizeMode(QListView::AllColumns);
  m_selectedList  -> setAllColumnsShowFocus(true);
  m_selectedList  -> setResizeMode(QListView::AllColumns);
  m_statusList    -> setAllColumnsShowFocus(true);
  m_statusList    -> setResizeMode(QListView::AllColumns);

  insertPage(new CodeGenerationOptionsPage(codegenState,this),i18n("Code Generation Options"),1);

  QList<UMLConcept> cList = m_doc->getConcepts();
  for(UMLConcept *c = cList.first(); c ; c = cList.next())
       new QListViewItem( m_availableList, c->GetName());
  if(classList)
    for(UMLConcept *c = classList->first(); c ; c = classList->next())
       new QListViewItem( m_selectedList, c->GetName());
  else
    for(UMLConcept *c = cList.first(); c ; c = cList.next())
       new QListViewItem( m_selectedList, c->GetName());

  setNextEnabled(page(0),m_selectedList->childCount() > 0);

  setFinishEnabled(page(2),true);
  finishButton()->disconnect();
  finishButton()->setText(i18n("Generate"));
  connect(finishButton(),SIGNAL(clicked()),this,SLOT(generateCode()));
}

CodeGenerationWizard::~CodeGenerationWizard(){
}


void CodeGenerationWizard::selectClass()
{
  if( !m_availableList -> currentItem() )
    return;
  QString name = m_availableList ->currentItem()->text(0);
  if( !m_selectedList -> findItem( name,0 ) )
    new QListViewItem(m_selectedList, name);
  setNextEnabled(currentPage(),true);
}

void CodeGenerationWizard::deselectClass()
{
  if( !m_selectedList -> currentItem() )
    return;
  m_selectedList->removeItem( m_selectedList->currentItem() );
  if(m_selectedList->childCount() == 0)
    setNextEnabled(currentPage(),false);
}

void CodeGenerationWizard::generateCode()
{
  backButton()->setEnabled(false);

  //get current Code Generation Options...
  SettingsDlg::CodeGenState codegenState;
  ((CodeGenerationOptionsPage*)page(1))->state(codegenState);

  m_generator->setForceDoc(codegenState.forceDoc);
  m_generator->setForceSections(codegenState.forceSections);
  m_generator->setIncludeHeadings(codegenState.includeHeadings);
  m_generator->setHeadingFileDir(codegenState.headingsDir);
  m_generator->setModifyNamePolicy(codegenState.modname);
  m_generator->setOutputDirectory(codegenState.outputDir);
  m_generator->setOverwritePolicy(codegenState.overwritePolicy);

  connect(m_generator,SIGNAL(codeGenerated(UMLConcept*)),
          this,SLOT(classGenerated(UMLConcept*)));

  QList<UMLConcept> cList;
  cList.setAutoDelete(false);
  QListViewItem *item = m_statusList->firstChild();

  for(; item; item = item-> nextSibling())
    cList.append((UMLConcept*)m_doc->findUMLObject(Uml::ot_Concept,item->text(0)));
  m_generator->generateCode(cList);
  finishButton()->setText(i18n("Finish"));
  finishButton()->disconnect();
  connect(finishButton(),SIGNAL(clicked()),this,SLOT(accept()));
}

void CodeGenerationWizard::classGenerated(UMLConcept *c)
{
 QListViewItem *item = m_statusList -> findItem( c->GetName(), 0 );
 if( !item )
      kdDebug()<<"GenerationStatusPage::Error finding class in list view"<<endl;
 else
      item->setText( 1, i18n("Code Generated") );


}

void CodeGenerationWizard::populateStatusList()
{
  m_statusList->clear();
  QListViewItem *item;
  item = m_selectedList->firstChild();
  for(; item; item = item-> nextSibling())
    new QListViewItem(m_statusList,item->text(0),i18n("No Code Generated"));
}

void CodeGenerationWizard::showPage(QWidget *page)
{
  if (indexOf(page) == 2)
    populateStatusList();
  QWizard::showPage(page);
}

