//qt includes
#include <qlayout.h>
//kde includes
#include <klocale.h>
#include <kdebug.h>
#include <kmessagebox.h>
//app includes
#include "../attribute.h"
#include "../concept.h"
#include "../umldoc.h"
#include "umlattributedialog.h"

UMLAttributeDialog::UMLAttributeDialog( QWidget * pParent, UMLAttribute * pAttribute ) : KDialogBase( Plain, i18n("Attribute Properties"), Help | Ok | Cancel , Ok, pParent, "_UMLATTRIBUTEDLG_", true, true)
{
	m_pAttribute = pAttribute;
	setupDialog();
}

UMLAttributeDialog::~UMLAttributeDialog()
{
}

void UMLAttributeDialog::setupDialog()
{
	UMLConcept * pConcept = dynamic_cast<UMLConcept *>( m_pAttribute -> parent() );
	UMLDoc * pDoc = dynamic_cast<UMLDoc *>( pConcept -> parent() );
	int margin = fontMetrics().height();

	QVBoxLayout * mainLayout = new QVBoxLayout( plainPage() );

	m_pValuesGB = new QGroupBox(i18n("General Properties"), plainPage() );
	QGridLayout * valuesLayout = new QGridLayout(m_pValuesGB, 3, 2);
	valuesLayout -> setMargin(margin);
	valuesLayout -> setSpacing(10);

	m_pTypeL = new QLabel(i18n("Type"), m_pValuesGB);
	valuesLayout -> addWidget(m_pTypeL, 0, 0);

	m_pNameL = new QLabel(i18n("Name"), m_pValuesGB);
	valuesLayout -> addWidget(m_pNameL, 1, 0);

	m_pInitialL = new QLabel(i18n("Initial Value"), m_pValuesGB);
	valuesLayout -> addWidget(m_pInitialL, 2, 0);

	m_pTypeCB = new QComboBox(m_pValuesGB);
	valuesLayout -> addWidget(m_pTypeCB, 0, 1);

	m_pNameLE = new QLineEdit(m_pValuesGB);
	valuesLayout -> addWidget(m_pNameLE, 1, 1);
	m_pNameLE -> setText( m_pAttribute -> GetName() );

	m_pInitialLE = new QLineEdit(m_pValuesGB);
	valuesLayout -> addWidget(m_pInitialLE, 2, 1);
	mainLayout -> addWidget(m_pValuesGB);
	m_pInitialLE -> setText( m_pAttribute -> getInitialValue() );


	m_pScopeBG = new QButtonGroup(i18n("Visibility"), plainPage() );
	QHBoxLayout * scopeLayout = new QHBoxLayout(m_pScopeBG);
	scopeLayout -> setMargin(margin);

	m_pPublicRB = new QRadioButton(i18n("Public"), m_pScopeBG);
	scopeLayout -> addWidget(m_pPublicRB);

	m_pPrivateRB = new QRadioButton(i18n("Private"), m_pScopeBG);
	scopeLayout -> addWidget(m_pPrivateRB);

	m_pProtectedRB = new QRadioButton(i18n("Protected"), m_pScopeBG);
	scopeLayout -> addWidget(m_pProtectedRB);
	mainLayout -> addWidget(m_pScopeBG);
	Uml::Scope scope = m_pAttribute -> getScope();
	if( scope == Uml::Public )
		m_pPublicRB -> setChecked( true );
	else if( scope == Uml::Private )
		m_pPrivateRB -> setChecked( true );
	else
		m_pProtectedRB -> setChecked( true );

	//add some standard attribute types to combo box
	QString types[] ={i18n("int"), i18n("long"), i18n("bool"), i18n("string"), i18n("double"), i18n("float"), i18n("date")};
	for(int i=0;i<7;i++)
		m_pTypeCB -> insertItem(types[i]);
	m_pTypeCB -> setDuplicatesEnabled(false);//only allow one of each type in box
	//now add the Concepts
	QList<UMLConcept> namesList(pDoc -> getConcepts());
	UMLConcept * obj;
	for(obj=namesList.first(); obj!=0 ;obj=namesList.next())
		m_pTypeCB -> insertItem(obj -> GetName());
	m_pTypeCB -> setEditable(true);
	m_pTypeCB -> setAutoCompletion(true);
	m_pTypeCB -> insertItem( m_pAttribute -> getTypeName(), 0 );
	m_pTypeCB -> setCurrentItem(0);
}

bool UMLAttributeDialog::apply()
{
	UMLConcept * pConcept = dynamic_cast<UMLConcept *>( m_pAttribute -> parent() );
	UMLDoc * pDoc = dynamic_cast<UMLDoc *>( pConcept -> parent() );

	if(m_pPublicRB -> isChecked())
		m_pAttribute -> setScope( Uml::Public );
	else if(m_pPrivateRB -> isChecked())
		m_pAttribute -> setScope( Uml::Private );
	else
		m_pAttribute -> setScope( Uml::Protected );

	m_pAttribute -> setTypeName( m_pTypeCB -> currentText() );
	m_pAttribute -> setInitialValue( m_pInitialLE -> text() );
	QString name = m_pNameLE -> text();
	if( name.length() == 0 )
	{
		KMessageBox::error(this, i18n("You have entered an invalid attribute name."),
			i18n("Attribute name Invalid."), false);
		m_pNameLE -> setText( m_pAttribute -> GetName() );
		return false;
	}

	QPtrList<UMLObject> list= pConcept -> findChildObject( Uml::ot_Attribute, name );
	if( list.count() != 0 && list.findRef( m_pAttribute ) )
	{
		KMessageBox::error(this, i18n("The Attribute name you have chosen is already being used in this operation."),
				i18n("Attribute name not unique."), false);
		m_pNameLE -> setText( m_pAttribute -> GetName() );
		return false;
	}
	m_pAttribute -> SetName( name );
	pDoc -> signalChildUMLObjectUpdate( m_pAttribute );
        return true;
}

void UMLAttributeDialog::slotApply() {
    apply();
}

void UMLAttributeDialog::slotOk() {
    if ( apply() ) {
	accept();
    }
}


