//qt includes
#include <qlayout.h>
//kde includes
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
//app includes
#include "../operation.h"
#include "../concept.h"
#include "../umldoc.h"
#include "../listpopupmenu.h"
#include "parmpropdlg.h"
#include "umloperationdialog.h"

UMLOperationDialog::UMLOperationDialog( QWidget * parent, UMLOperation * pOperation ) : KDialogBase( Plain, i18n("Operation Properties"), Help | Ok | Cancel , Ok, parent, "_UMLOPERATIONDLG_", true, true)
{
	m_pOperation = pOperation;
	m_pMenu = 0;
	setupDialog();
}

UMLOperationDialog::~UMLOperationDialog()
{
}

void UMLOperationDialog::setupDialog()
{
	UMLConcept * pConcept = dynamic_cast<UMLConcept *>( m_pOperation -> parent() );
	UMLDoc * pDoc = dynamic_cast<UMLDoc *>( pConcept -> parent() );
	int margin = fontMetrics().height();
	QVBoxLayout * topLayout = new QVBoxLayout( plainPage() );

	m_pGenGB = new QGroupBox(i18n("General Properties"), plainPage() );
	QGridLayout * genLayout = new QGridLayout(m_pGenGB, 2, 4 );
	genLayout -> setMargin(margin);
	genLayout -> setSpacing(10);

	m_pNameL = new QLabel(i18n("Name"), m_pGenGB);
	genLayout -> addWidget(m_pNameL, 0, 0 );

	m_pNameLE = new QLineEdit(m_pGenGB );
	m_pNameLE -> setText( m_pOperation -> GetName() );
	genLayout -> addWidget(m_pNameLE, 0, 1);

	m_pRtypeL = new QLabel(i18n("Type"), m_pGenGB );
	genLayout -> addWidget(m_pRtypeL, 0, 2);

	m_pRtypeCB = new QComboBox(m_pGenGB );
	genLayout -> addWidget(m_pRtypeCB, 0, 3);

	m_pAbstractCB = new QCheckBox( i18n("Abstract Operation"), m_pGenGB );
	m_pAbstractCB -> setChecked( m_pOperation -> getAbstract() );
	genLayout -> addWidget( m_pAbstractCB, 1, 0 );
	topLayout -> addWidget( m_pGenGB );


	m_pScopeBG = new QButtonGroup(i18n("Visibility"), plainPage() );

	QHBoxLayout * scopeLayout = new QHBoxLayout(m_pScopeBG);
	scopeLayout -> setMargin(margin);

	m_pPublicRB = new QRadioButton(i18n("Public"), m_pScopeBG);
	scopeLayout -> addWidget(m_pPublicRB);

	m_pPrivateRB = new QRadioButton(i18n("Private"), m_pScopeBG);
	scopeLayout -> addWidget(m_pPrivateRB);

	m_pProtectedRB = new QRadioButton(i18n("Protected"), m_pScopeBG);
	scopeLayout -> addWidget(m_pProtectedRB);
	topLayout -> addWidget(m_pScopeBG);

	m_pParmsGB = new QGroupBox(i18n("Parameters"), plainPage() );
	QHBoxLayout * parmsLayout = new QHBoxLayout(m_pParmsGB);
	parmsLayout -> setMargin(margin);

	m_pParmsLB = new QListBox(m_pParmsGB);
	parmsLayout -> addWidget(m_pParmsLB);
	topLayout -> addWidget(m_pParmsGB);

	//add some standard return types to combo box
	m_pRtypeCB -> setDuplicatesEnabled(false);//only allow one of each type in box
	m_pRtypeCB -> setCurrentItem(0);//select void
	m_pRtypeCB -> setEditable(true);
	m_pRtypeCB -> setAutoCompletion(true);
	QString types[] ={i18n("void"), i18n("int"), i18n("long"), i18n("bool"), i18n("string"), i18n("double"),
			i18n("float"), i18n("date")};
	for(int i=0;i<8;i++)
		m_pRtypeCB -> insertItem(types[i]);
	//now add the Concepts
	QPtrList<UMLConcept> namesList( pDoc -> getConcepts() );
	pConcept = 0;
	for(pConcept=namesList.first(); pConcept!=0 ;pConcept=namesList.next())
		m_pRtypeCB -> insertItem(pConcept -> GetName());
	m_pRtypeCB -> insertItem( m_pOperation -> getReturnType(), 0 );
	m_pRtypeCB -> setCurrentItem( 0 );
	//fill in parm list box
	QList<UMLAttribute> * list = m_pOperation -> getParmList();
	UMLAttribute * pAtt = 0;
	for( pAtt = list -> first(); pAtt != 0; pAtt = list -> next() )
		m_pParmsLB -> insertItem( pAtt -> GetName() );

	//set scope
	Uml::Scope scope = m_pOperation -> getScope();
	if( scope == Uml::Public )
		m_pPublicRB -> setChecked( true );
	else if( scope == Uml::Private )
		m_pPrivateRB -> setChecked( true );
	else
		m_pProtectedRB -> setChecked( true );

	//setup parm list box signals
	connect(m_pParmsLB, SIGNAL(rightButtonPressed(QListBoxItem *, const QPoint &)),
		this, SLOT(slotParmRightButtonPressed(QListBoxItem *, const QPoint &)));

	connect(m_pParmsLB, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)),
		this, SLOT(slotParmRightButtonClicked(QListBoxItem *, const QPoint &)));


	connect(m_pParmsLB, SIGNAL(doubleClicked(QListBoxItem *)),
		this, SLOT(slotParmDoubleClick(QListBoxItem *)));
}

void UMLOperationDialog::slotParmRightButtonPressed(QListBoxItem *item, const QPoint &p)
{
	ListPopupMenu::Menu_Type type = ListPopupMenu::mt_Undefined;
	if(item)//pressed on an item
	{
		type = ListPopupMenu::mt_Parameter_Selected;
	}
	else//pressed into fresh air
	{
		type = ListPopupMenu::mt_New_Parameter;
	}
	if(m_pMenu)
	{
		m_pMenu -> hide();
		disconnect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotParmPopupMenuSel(int)));
		delete m_pMenu;
		m_pMenu = 0;
	}
	m_pMenu = new ListPopupMenu(this, type);
	m_pMenu->popup(p);
	connect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotParmPopupMenuSel(int)));

}

void UMLOperationDialog::slotParmRightButtonClicked(QListBoxItem */*item*/, const QPoint &/*p*/)
{
	if(m_pMenu)
	{
		m_pMenu -> hide();
		disconnect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotParmPopupMenuSel(int)));
		delete m_pMenu;
		m_pMenu = 0;
	}
}

void UMLOperationDialog::slotParmDoubleClick(QListBoxItem *item)
{
	if(!item)
		return;
	slotParmPopupMenuSel(ListPopupMenu::mt_Properties);
}

void UMLOperationDialog::slotParmPopupMenuSel( int id )
{
	int result = 0;
	UMLAttribute * pAtt = 0, * pOldAtt = 0;
	pOldAtt = m_pOperation -> findParm( m_pParmsLB -> currentText() );
	UMLConcept * pConcept = dynamic_cast<UMLConcept *>( m_pOperation -> parent() );
	UMLDoc * pDoc = dynamic_cast<UMLDoc *>( pConcept -> parent() );
	if( id == ListPopupMenu::mt_Rename || id == ListPopupMenu::mt_Properties )
	{
		if( !pOldAtt )
		{
			kdDebug() << "THE impossible has occurred for:" << m_pParmsLB -> currentText() << endl;
			return;
		}//should never occur
		ParmPropDlg dlg(this, pDoc, pOldAtt);
		result = dlg.exec();
		QString name = dlg.getName();
		pAtt = m_pOperation -> findParm( name );
		if( result )
		{
			if( name.length() == 0 )
			{
				KMessageBox::error(this, i18n("You have entered an invalid parameter name."),
				i18n("Parameter name Invalid."), false);
				return;
			}
			if( !pAtt || ( pOldAtt -> getTypeName() != dlg.getTypeName() ) )
			{
				pOldAtt -> SetName( name );
				pOldAtt -> setTypeName( dlg.getTypeName() );
				m_pParmsLB -> changeItem( dlg.getName(), m_pParmsLB -> currentItem() );
				pOldAtt -> setDoc( dlg.getDoc() );
				pOldAtt -> setInitialValue( dlg.getInitialValue() );
				pDoc -> setModified( true );
			}
			else if( pAtt != pOldAtt )
			{
				KMessageBox::error(this, i18n("The Parameter name you have chosen is already being used in this operation."),
				i18n("Parameter name not unique."), false);
			}
		}
	}
	else if( id == ListPopupMenu::mt_New_Parameter )
	{
		ParmPropDlg dlg( this, pDoc, 0 );
		result = dlg.exec();
		QString name = dlg.getName();
		pAtt = m_pOperation -> findParm( name );
		if( result )
		{
			if( name.length() == 0 )
			{
				KMessageBox::error(this, i18n("You have entered an invalid parameter name."),
				i18n("Parameter name Invalid."), false);
				return;
			}
			if( !pAtt )
			{
				m_pOperation-> addParm( dlg.getTypeName(), name, dlg.getInitialValue(), dlg.getDoc() );
				m_pParmsLB -> insertItem( dlg.getName() );
				pDoc -> setModified( true );
			}
			else
			{
				KMessageBox::sorry(this, i18n("The Parameter name you have chosen\nis already being used in this operation."),
				i18n("Parameter name not unique."), false);
			}
		}//end if result
	}//end if PARM
	else if( id == ListPopupMenu::mt_Delete )
	{
		m_pOperation -> removeParm( pOldAtt );
		m_pParmsLB -> removeItem( m_pParmsLB -> currentItem() );
		pDoc -> setModified(true);
	}//end id DELETE
}

bool UMLOperationDialog::apply()
{
	QString name = m_pNameLE -> text();
	UMLConcept * pConcept = dynamic_cast<UMLConcept *>( m_pOperation -> parent() );
	UMLDoc * pDoc = dynamic_cast<UMLDoc *>( pConcept -> parent() );
	if( m_pPublicRB -> isChecked() )
		m_pOperation -> setScope( Uml::Public );
	else if( m_pPrivateRB -> isChecked() )
		m_pOperation -> setScope( Uml::Private );
	else
		m_pOperation -> setScope( Uml::Protected );
	m_pOperation -> setReturnType( m_pRtypeCB -> currentText() );
	m_pOperation -> setAbstract( m_pAbstractCB -> isChecked() );
	QPtrList<UMLObject> list= pConcept -> findChildObject( Uml::ot_Operation, name );
	if( name.length() == 0 )
	{
		KMessageBox::error(this, i18n("You have entered an invalid operation name."),
			i18n("Operation name Invalid."), false);
		m_pNameLE -> setText( m_pOperation -> GetName() );
		return false;
	}
	if( list.count() != 0 && list.findRef( m_pOperation ) )
	{
		if( KMessageBox::warningYesNo( this , i18n( "The name you entered was not unique!\nIs this what you wanted?" ),
				i18n( "Name not unique!" ) ) == KMessageBox::No )
		{
			m_pNameLE -> setText( m_pOperation -> GetName() );
			return false;
		}
	}
	m_pOperation -> SetName( name );
	pDoc -> signalChildUMLObjectUpdate( m_pOperation );
        return true;
}

void UMLOperationDialog::slotApply() {
    apply();
}

void UMLOperationDialog::slotOk() {
    if ( apply() ) {
	accept();
    }
}






