#include <qdragobject.h>
#include <qlist.h>

#include <klocale.h>
#include <klineeditdlg.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kdebug.h>
#include <kglobal.h>
#include  <kstddirs.h>

#include "mylistview.h"
#include "umlview.h"
#include "dialogs/classpropdlg.h"
#include "mylistviewitemdatalist.h"
#include "clipboard/idchangelog.h"
#include "clipboard/cumldrag.h"
#include "dialogs/umlattributedialog.h"
#include "dialogs/umloperationdialog.h"

#include "uml.h"
#include "cclassimport.h"


MyListView::MyListView(QWidget *parent,const char *name) : KListView(parent,name)
{
  loadPixmaps();
  init();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListView::~MyListView()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::contentsMousePressEvent(QMouseEvent *me)
{
  if( isRenaming() )
    return;
    if( doc -> getCurrentView() )
		doc -> getCurrentView() -> clearSelected();
	if( me -> state() != ShiftButton )
		clearSelection();
	//MyListViewItem *item = (MyListViewItem *)currentItem();
   QPoint pt = ((MyListView*)this) -> contentsToViewport( me -> pos() );
	MyListViewItem * item = (MyListViewItem*)itemAt(pt);
	if(!item)
		return;

	if( !item || ( me -> button() != RightButton && me -> button() != LeftButton ) )
	{
		doc -> getDocWindow() -> updateDocumentation( true );
		return;
	}
	Uml::ListView_Type type = item -> getType();

	switch( type )
	{
		//If is necesary activate the view
		case Uml::lvt_UseCase_Diagram:
		case Uml::lvt_Class_Diagram:
		case Uml::lvt_Collaboration_Diagram:
		case Uml::lvt_Sequence_Diagram:
		case Uml::lvt_State_Diagram:
		case Uml::lvt_Activity_Diagram:
			if( ! doc->activateView( item->getID()))
				KMessageBox::error( kapp -> mainWidget(), i18n("Could not activate the diagram."), i18n("ERROR: Diagram load!"));
			else
				doc -> changeCurrentView(item->getID());
				doc -> getDocWindow() -> showDocumentation( doc -> findView( item -> getID() ), false );
		break;

		case Uml::lvt_UseCase:
		case Uml::lvt_Class:
		case Uml::lvt_Actor:
		case Uml::lvt_Attribute:
		case Uml::lvt_Operation:
			doc -> getDocWindow() -> showDocumentation( item -> getUMLObject(), false );
		break;

		default:
			doc -> getDocWindow() -> updateDocumentation( true );
		break;
	}//end switch
	if(me->button() == RightButton)
	{
		//setSelected( item, true);
		if(menu != 0)
		{
			menu->hide();
			disconnect(menu, SIGNAL(activated(int)), this, SLOT(popupMenuSel(int)));
			delete menu;
			menu = 0;
		}
		//if on root of tree just return - no popup
		if(type == Uml::lvt_View)
			return;
		menu = new ListPopupMenu(this, type);
		menu->popup(me->globalPos());
		connect(menu, SIGNAL(activated(int)), this, SLOT(popupMenuSel(int)));
	}//end if right button

	/*
		We don't need to do anything for Left button.
		But if we do in future, just put and else statement in here
	*/
  if( !isRenaming() )
	this->KListView::contentsMousePressEvent(me);


}

void MyListView::contentsMouseReleaseEvent(QMouseEvent *me)
{
  if( isRenaming() )
    return;
  this->KListView::contentsMouseReleaseEvent(me);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::popupMenuSel(int sel)
{
	MyListViewItem * temp = (MyListViewItem*)currentItem();
	int id = temp -> getID();
	UMLObject * object = temp -> getUMLObject();
	Uml::ListView_Type lvtType = temp -> getType();
  Uml::UMLObject_Type umlType = Uml::ot_UMLObject;
	QString name;

	switch(sel)
	{
		case ListPopupMenu::mt_Activity_Diagram:
			addNewItem( temp, Uml::lvt_Activity_Diagram );
		break;

      case ListPopupMenu::mt_UseCase_Diagram:
        addNewItem( temp, Uml::lvt_UseCase_Diagram );
      break;

		case ListPopupMenu::mt_Sequence_Diagram:
        addNewItem( temp, Uml::lvt_Sequence_Diagram );
      break;

		case ListPopupMenu::mt_Collaboration_Diagram:
        addNewItem( temp, Uml::lvt_Collaboration_Diagram );
      break;

		case ListPopupMenu::mt_Class_Diagram:
			addNewItem( temp, Uml::lvt_Class_Diagram );
		break;

      case ListPopupMenu::mt_State_Diagram:
			addNewItem( temp, Uml::lvt_State_Diagram );
		break;

      case ListPopupMenu::mt_Concept:
        addNewItem( temp, Uml::lvt_Class );
      break;

		case ListPopupMenu::mt_Actor:
        addNewItem( temp, Uml::lvt_Actor );
      break;

		case ListPopupMenu::mt_UseCase:
        addNewItem( temp, Uml::lvt_UseCase );
      break;

      case ListPopupMenu::mt_Attribute:
        addNewItem( temp, Uml::lvt_Attribute );
      break;

		case ListPopupMenu::mt_Operation:
        addNewItem( temp, Uml::lvt_Operation );
      break;

		case ListPopupMenu::mt_Import_Classes:
                    ((CClassImport*)doc)->importCPP(
                        KFileDialog::getOpenFileNames(QString::null,
                                                      i18n("*.h|Header Files (*.h)\n*.*|All Files"),
                                                      this, i18n("Select Classes to Import...") ));
                break;



		case ListPopupMenu::mt_Export_Image:
                    doc->getCurrentView()->exportImage(
                        KFileDialog::getSaveURL(QDir::currentDirPath(),
                                                i18n("*.png|PNG Image\n*.*|All Files"),
                                                this,
                                                i18n("Save as...") ));
		break;

		case ListPopupMenu::mt_Rename:
        temp-> startRename(0);
		break;

		case ListPopupMenu::mt_Delete:
			if(lvtType >= Uml::lvt_UseCase_Diagram && lvtType <= Uml::lvt_Sequence_Diagram)
				doc->removeDiagram(id);
			else if(lvtType == Uml::lvt_Logical_Folder || lvtType == Uml::lvt_UseCase_Folder)
			{
				if(temp -> firstChild())
				{
					KMessageBox::error( kapp -> mainWidget(), i18n("The folder must be emptied before it can be deleted."), i18n("Folder not Empty!"));
					return;
				}
				delete temp;
			}
			else
				doc ->removeUMLObject(object);
			return;
		break;

		case ListPopupMenu::mt_Properties:
			/* first check if we are on a diagram */
			if( lvtType >= Uml::lvt_UseCase_Diagram &&
							lvtType <= Uml::lvt_Sequence_Diagram )
			{
				UMLView * pView = doc -> findView( temp -> getID() );
				if( !pView )
					return;
				doc -> getDocWindow() -> updateDocumentation( false );
				pView -> showPropDialog();
				doc -> getDocWindow() -> showDocumentation( pView, true );
				temp -> cancelRename( 0 );
				return;
			}

			/* ok, we are on another object, so find out on which one */
			umlType = object -> getBaseType();

			if(umlType >= Uml::ot_Actor && umlType <= Uml::ot_Concept)
				doc -> showProperties(object, ClassPropDlg::page_gen);
			else if(umlType == Uml::ot_Attribute)
			{
                            // show the attribute dialogue
                            UMLAttribute* selectedAttribute = (UMLAttribute*)object;
                            UMLAttributeDialog dialogue( this, selectedAttribute );
                            dialogue.exec();
			}
			else if(umlType == Uml::ot_Operation)
			{
                            // show the operation dialogue
                            UMLOperation* selectedOperation = (UMLOperation*)object;
                            UMLOperationDialog dialogue( this, selectedOperation );
                            dialogue.exec();
			}
          temp -> cancelRename( 0 );
		break;

		case ListPopupMenu::mt_Logical_Folder:
        addNewItem( temp, Uml::lvt_Logical_Folder );
			doc -> setModified( true );
        break;

		case ListPopupMenu::mt_UseCase_Folder:
			addNewItem( temp, Uml::lvt_UseCase_Folder );
			doc -> setModified( true );
		break;

		case ListPopupMenu::mt_Cut:
			m_bStartedCut = true;
			doc -> EditCut();
		break;

		case ListPopupMenu::mt_Copy:
			doc -> EditCopy();
		break;

		case ListPopupMenu::mt_Paste:
			doc -> EditPaste();
		break;

		default:
			return;//must be something we don't want to do
		break;
		}//end switch
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotDiagramCreated( int id )
{
	if( loading )
		return;
	MyListViewItem * temp = 0, *p = 0;
	UMLView *v = doc -> findView( id );
	//See if we wanted to create diagram in folder
	MyListViewItem * current = (MyListViewItem *) currentItem();
	if( current -> getType() == Uml::lvt_Logical_Folder || current -> getType() == Uml::lvt_UseCase_Folder )
		p = current;
	else if( v->getType() == Uml::dt_UseCase )
		p = ucv;
	else
		p = lv;
	temp = new MyListViewItem( p, v->GetName(), convert_DT_LVT( v->getType() ),  id );
	setSelected( temp, true );
	doc -> getDocWindow() -> showDocumentation( v , false );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotObjectCreated(UMLObject * o)
{
	if(loading)
		return;
	MyListViewItem * temp = 0, * item = 0;
	Uml::UMLObject_Type type = o -> getBaseType();

	//See if we wanted to create diagram in folder
	MyListViewItem * current = (MyListViewItem *) currentItem();
	if( current -> getType() == Uml::lvt_Logical_Folder || current -> getType() == Uml::lvt_UseCase_Folder )
		item = current;
	else if( type == Uml::ot_Concept )
		item = lv;
	else
		item = ucv;
	temp = new MyListViewItem( item, o->GetName(), convert_OT_LVT( type ), o );
	temp->setOpen( true );
	setSelected( temp, true );
	doc -> getDocWindow() -> showDocumentation( o, false );
}
////////////////////////////////////////////////////////////////////////////////////////////////////

void MyListView::slotChildObjectCreated(UMLObject * o)
{
	if(loading)
		return;
	MyListViewItem *item = findUMLObject((UMLObject*)o->parent());
	MyListViewItem *temp = new MyListViewItem(item, o->GetName(), convert_OT_LVT( o->getBaseType() ), o);
	setSelected(temp, true);
	doc -> getDocWindow() -> showDocumentation( o, false );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotDiagramRenamed(int id)
{
	MyListViewItem * temp;
	UMLView *v = doc->findView(id);
	temp = findView(v);
	temp -> setText(v->GetName());
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotObjectChanged(UMLObject * o)
{
	if(loading)//needed for class wizard
		return;
	MyListViewItem *item = findUMLObject(o);
	if(item)
		item -> updateObject();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotChildObjectChanged(UMLObject * o)
{
	if(loading)//needed for class wizard
		return;
	MyListViewItem * item = findChildUMLObject(o);
	if(item)
		item -> updateObject();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::setDocument(UMLDoc *d)
{
	doc = d;
	connect( (UMLApp *)doc -> parent(), SIGNAL( sigCutSuccessful() ), this, SLOT( slotCutSuccessful() ) );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotObjectRemoved(UMLObject * o)
{
	if(loading)//needed for class wizard
		return;
	MyListViewItem * t = findItem(o -> getID());
	if(t)
		delete t;
	doc -> getDocWindow() -> updateDocumentation( true );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::slotDiagramRemoved(int id)
{
	MyListViewItem * temp = findItem(id);
	delete temp;
	doc -> getDocWindow() -> updateDocumentation( true );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
QDragObject* MyListView::dragObject()
{
	CMyListViewItemList selecteditems;
	GetSelectedItems(selecteditems);
	selecteditems.setAutoDelete( FALSE );
	CMyListViewItemListIt it(selecteditems);
	MyListViewItem * item = 0;
	CMyListViewItemDataList  list;
	list.setAutoDelete( FALSE );
	Uml::ListView_Type type;
	while((item=it.current()) != 0)
	{
		++it;
		type = item->getType();
		if((type < Uml::lvt_Actor || type > Uml::lvt_Class) && (type < Uml::lvt_UseCase_Diagram || type > Uml::lvt_Sequence_Diagram) &&
						(type != Uml::lvt_UseCase_Folder) && (type != Uml::lvt_Logical_Folder))
		{
			return 0;
    	}
		list.append(item->getdata());
	}
	CUMLDrag *t = new CUMLDrag(list, this);

	return t;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::startDrag()
{
	QTextDrag * o=(QTextDrag*)dragObject();
	if(o)
		o->dragCopy();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem * MyListView::findUMLObject(UMLObject *p)
{
	Uml::UMLObject_Type pt = p->getBaseType();
	MyListViewItem *item, *temp;

	if(pt == Uml::ot_Concept) {
		item = lv;
	} else if(pt == Uml::ot_Actor || pt == Uml::ot_UseCase) {
		item = ucv;
	} else {
		kdDebug() << "ERROR:listview:find" << endl;
		return 0;
	}
	temp = (MyListViewItem *)item->firstChild();
	while(temp)
	{
		Uml::ListView_Type type = temp->getType();
		if(type == Uml::lvt_Logical_Folder || type == Uml::lvt_UseCase_Folder)
		{
			item = (MyListViewItem *)temp->firstChild();
			while(item)
			{
				type = item->getType();
				if(type >= Uml::lvt_Actor && type <=Uml::lvt_Class)
				{
					if(item->getUMLObject() == p)
						return item;
				}
				item = (MyListViewItem *)item->nextSibling();
			}
		}
		else if(type >= Uml::lvt_Actor && type <=Uml::lvt_Class)
		{
			if(temp->getUMLObject() == p)
				return temp;
		}
		temp = (MyListViewItem *)temp->nextSibling();
	}
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem * MyListView::findChildUMLObject(UMLObject *c)
{
	MyListViewItem *item, *child, *temp;
	//must be an att or op we are looking for here
	item = (MyListViewItem *)lv->firstChild();
	//go though all classes
	while(item)
	{
		//item = (MyListViewItem *)lv->firstChild();
		if(item -> getType() == Uml::lvt_Logical_Folder)
		{
			temp = (MyListViewItem *)item->firstChild();
			while(temp)
			{
				if(temp->getType() == Uml::lvt_Class)
				{
					child = (MyListViewItem *)temp->firstChild();
					while(child)
					{
						if(child->getUMLObject() == c)
							return child;
						child = (MyListViewItem *)child ->nextSibling();
					}//end while child
				}//end if
				temp =(MyListViewItem *)temp -> nextSibling();
			}//end while temp
		}//end if Uml::lvt_Logical_Folder
		//check each classes children for the id
		else if(item->getType() == Uml::lvt_Class)
		{
			child = (MyListViewItem *)item->firstChild();
			while(child)
			{
				if(child->getUMLObject() == c)
					return child;
				child = (MyListViewItem *)child ->nextSibling();
			}//end while child
		}
		item =(MyListViewItem *)item -> nextSibling();
	}//end while
	kdDebug() << "Error findchild" << endl;
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem* MyListView::findView(UMLView *v)
{
	if(!v)
		return 0;
	MyListViewItem *item, *temp, * child;
   Uml::Diagram_Type dType = v->getType();
	Uml::ListView_Type type = convert_DT_LVT( dType );
	int id = v->getID();
	if(dType == Uml::dt_UseCase)
		item = ucv;
	else
		item = lv;

	temp = (MyListViewItem *)item->firstChild();
	while(temp)
	{
		if(temp -> getType() == Uml::lvt_Logical_Folder || temp -> getType() == Uml::lvt_UseCase_Folder)
		{
			child = (MyListViewItem *)temp -> firstChild();
			while(child)
			{
				if( child -> getType() == type && child -> getID() == id)
					return child;
				child = (MyListViewItem *)child-> nextSibling();
			}
		}
		else
		{
			if(temp->getType() == type && temp->getID() == id)
				return temp;
		}
		temp = (MyListViewItem *)temp->nextSibling();
	}
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem* MyListView::findItem(int id)
{
	MyListViewItem *temp;
	QListViewItemIterator it(this);
	for( ; (temp = (MyListViewItem*)it.current()); ++it )
		if(temp->getID() == id)
			return temp;
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::init()
{
    //setup list view
  setBackgroundColor(white);
  setAcceptDrops(true);
  setDropVisualizer(false);
  setItemsMovable(true);
  setItemsRenameable( true );
  setSelectionModeExt(FileManager);
  setShowToolTips( true );
  setTooltipColumn( 0 );
  setFocusPolicy(QWidget::StrongFocus);
  setDragEnabled(TRUE);
  setColumnWidthMode( 0, Manual );
  setDefaultRenameAction( Reject );
  setResizeMode( LastColumn );
  //add columns and initial item
  clear();
  if(columns()  == 0)
    addColumn(i18n("UML Diagrams"));
  rv =  new MyListViewItem(this, i18n("Views"), Uml::lvt_View);
  ucv = new MyListViewItem(rv, i18n("Use Case View"), Uml::lvt_UseCase_View);
  lv = new MyListViewItem(rv, i18n("Logical View"), Uml::lvt_Logical_View);
  rv -> setOpen(true);
  ucv -> setOpen(true);
  lv -> setOpen(true);
  //setup misc.
  menu = 0;
  m_bStartedCut = false;
  loading = false;
  m_bIgnoreCancelRename = true;
  //setup slots/signals
  connect(this, SIGNAL(dropped(QDropEvent *, QListViewItem *, QListViewItem *)), this, SLOT(slotDropped(QDropEvent *, QListViewItem *, QListViewItem *)));
  connect( this, SIGNAL( collapsed( QListViewItem * ) ), this, SLOT( slotCollapsed( QListViewItem * ) ) );
  connect( this, SIGNAL( expanded( QListViewItem * ) ), this, SLOT( slotExpanded( QListViewItem * ) ) );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::setView(UMLView * v)
{
	if(!v)
		return;
	MyListViewItem * temp = findView(v);
	if(temp)
		setSelected(temp, true);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListView::contentsMouseDoubleClickEvent(QMouseEvent * me)
{
	MyListViewItem * item = dynamic_cast<MyListViewItem *>( currentItem() );
	if( !item || me -> button() != LeftButton )
		return;
	//see if on view
	Uml::ListView_Type lvType = item -> getType();
	if( lvType >= Uml::lvt_UseCase_Diagram && lvType <= Uml::lvt_Sequence_Diagram )
	{
		UMLView * pView = doc -> findView( item -> getID() );
		if( !pView )
			return;
		doc -> getDocWindow() -> updateDocumentation( false );
		pView -> showPropDialog();
		doc -> getDocWindow() -> showDocumentation( pView, true );
		item -> cancelRename( 0 );
		return;
	}
	//else see if an object
	UMLObject * object = item -> getUMLObject();
	//continue only if we are on a UMLObject
	if(!object)
		return;


	Uml::UMLObject_Type type = object -> getBaseType();
	int page = ClassPropDlg::page_gen;
	if(type == Uml::ot_Attribute || type == Uml::ot_Operation)
		object = (UMLObject *)object -> parent();
	//set what page to show
	if(type == Uml::ot_Attribute)
		page = ClassPropDlg::page_att;
	else if(type == Uml::ot_Operation)
		page = ClassPropDlg::page_op;

	if(object)
		doc -> showProperties(object, page);
   item -> cancelRename( 0 );//double click can cause it to go into rename mode.
}


bool MyListView::acceptDrag(QDropEvent * event) const
{

	QPoint pt =  ((MyListView*)this)->contentsToViewport(event -> pos());

	MyListViewItem * item = (MyListViewItem*)itemAt(pt);
	if(!item)
	{
		return false;
	}
	((QListView*)this)->setCurrentItem((QListViewItem*)item );
	CMyListViewItemDataList list;
	bool status = CUMLDrag::decode(event, list);

	if(!status)
	{
		return false;
	}
	CMyListViewItemDataListIt it(list);
	CMyListViewItemData* data = 0;
	int type = 0;
	bool accept = true;
	type = item -> getType();
    while(accept && ((data =  it.current()) != 0))
	{
    	++it;
		if(data->getType() == Uml::lvt_UseCase_Folder && type == Uml::lvt_UseCase_Folder)
    	{
    		continue;
    	}
    	if(data->getType() == Uml::lvt_Class && (type == Uml::lvt_Logical_Folder || type == Uml::lvt_Logical_View) )

    	{
			continue;
    	}
    	if( (data->getType() == Uml::lvt_Actor || data->getType() == Uml::lvt_UseCase || data->getType() == Uml::lvt_UseCase_Diagram) && (type == Uml::lvt_UseCase_Folder || type == Uml::lvt_UseCase_View) )
    	{
    		continue;
    	}
    	if((data->getType() >= Uml::lvt_Collaboration_Diagram && data->getType() <= Uml::lvt_Sequence_Diagram) && (type == Uml::lvt_Logical_Folder || type == Uml::lvt_Logical_View))
		{
    		continue;
		}
   		accept = accept && false;
	}

	return accept;
}


void MyListView::slotDropped(QDropEvent * de, QListViewItem * parent, QListViewItem * item)
{
   MyListViewItem * move;
   parent = 0;//done just to stop compiler warning
	if(!item)

		return;
	CMyListViewItemDataList list;
	bool status = CUMLDrag::decode(de, list);

	if(!status)
	{
		return;
	}
	CMyListViewItemDataListIt it(list);
	CMyListViewItemData* data = 0;
	Uml::ListView_Type type = Uml::lvt_Unknown;
	UMLView * v = 0;
	UMLObject * o =0;
	while((data =  it.current()) != 0)
	{
    	++it;
		if(data->getType() >= Uml::lvt_UseCase_Diagram && data->getType() <= Uml::lvt_Sequence_Diagram)
    	{
    		v = doc -> findView(data->getID());
    		move = findView(v);
    		if(!move)
    			continue;
    	}
    	else
    	{
    		o = doc -> findUMLObject(data->getID());
    		move = findUMLObject(o);
    		if(!move)
    			continue;
    	}
		type = ((MyListViewItem*)item)  -> getType();
    	//make sure trying to place in correct location
    	if( (data->getType() == Uml::lvt_UseCase_Folder || data->getType() == Uml::lvt_Actor || data->getType() == Uml::lvt_UseCase || data->getType() == Uml::lvt_UseCase_Diagram) && (type == Uml::lvt_UseCase_Folder || type == Uml::lvt_UseCase_View) )
    	{
    		moveItem(move, item, item);
    	}
    	if( ((data->getType() >= Uml::lvt_Collaboration_Diagram && data->getType() <= Uml::lvt_Sequence_Diagram) || data->getType() == Uml::lvt_Class) && (type == Uml::lvt_Logical_Folder || type == Uml::lvt_Logical_View))
		{
    		moveItem(move, item, item);
		}
	}
}

/** No descriptions */
void MyListView::setWidgetDefaults()
{
}


/* Reads a MyListView from a QDataStream, this implementation allows the posibiliy to have folders
inside folders and it is closely tied to the behavior of a QListViewItemIterator, in QT's documentation
it says this iterator performs a preorder reading of the tree and this function uses this fact */
bool MyListView::serialize(QDataStream *s, bool archive)
{
	if(archive)
	{
		MyListViewItem *temp;
		QListViewItemIterator it( this );
		//For the root item save only its child count.
		temp = (MyListViewItem*)it.current();
		*s << temp->childCount();
		++it;
		for( ; (temp = (MyListViewItem*)it.current()); ++it )
		{
			temp->getdata()->serialize(s, archive);
		}
	}
	else
	{
		/* Delete all default folder views because they will be readed from the file, this change is so in order to
			support folders inside folders*/
		delete ucv;
		ucv = 0;
		delete lv;
		lv = 0;
		int id, childs =0, rootchilds =0;
   	Uml::ListView_Type type = Uml::lvt_Unknown;
		QString text;
		int t, open;
			*s >> rootchilds;
		for(int i =0; i < rootchilds ; i++)
		{
				*s >> id
				>> t
				>> text
				>> childs
				>> open;
				type = (Uml::ListView_Type)t;
				switch( type)
				{
					case Uml::lvt_UseCase_View:
							ucv = new MyListViewItem(rv, i18n("Use Case View"), Uml::lvt_UseCase_View);
							ucv -> setOpen( (bool)open );
							for(int i =0; i < childs ; i++)
							{
								if( !ReadChilds( ucv, s ) )
								{
									return false;
								}
							}
					break;
					case Uml::lvt_Logical_View:

							lv = new MyListViewItem(rv, i18n("Logical View"), Uml::lvt_Logical_View);
							lv -> setOpen( (bool)open );
							for(int i =0; i < childs ; i++)
							{
								if( !ReadChilds( lv, s ) )
								{
									return false;
								}
							}
					break;
					default:
						return false;
					break;
				}//end switch
		}//end for
	}//end else
	return true;
}

bool MyListView::ReadChilds( MyListViewItem* parent, QDataStream *s)
{
	int id, childs;
 Uml::ListView_Type type = Uml::lvt_Unknown;
	int t, open;
	QString text;

	MyListViewItem * item;
	UMLObject * o;

	*s >> id
   	 >> t
		 >> text
		 >> childs
		>> open;
	type = (Uml::ListView_Type)t;
	if(type == Uml::lvt_Actor || type == Uml::lvt_UseCase || type == Uml::lvt_Class)
	{
		o = doc -> findUMLObject(id);
		item = new MyListViewItem(parent, text, type, o);
	}
	else if(type == Uml::lvt_Attribute || type == Uml::lvt_Operation)
	{
		item = new MyListViewItem(parent, text, type, ((UMLConcept*)parent->getUMLObject())->findChildObject(id));
	}
	else
	{
		item = new MyListViewItem(parent, text, type, id);
	}
   item -> setOpen( (bool)open );
	for(int i = 0; i < childs ; i++)
	{
		if(!ReadChilds(item, s))
		{
			return false;
		}
	}

	return true;
}
/** No descriptions */
bool MyListView::GetSelectedItems(CMyListViewItemList &ItemList)
{
	ItemList.setAutoDelete( FALSE );
  QListViewItemIterator it(this);
  // iterate through all items of the list view
  for ( ; it.current(); ++it )
	{
  		if ( it.current()->isSelected() )
		{
			ItemList.append((MyListViewItem*)it.current());
		}
  }

	return true;
}
/** Returns the amount of bytes needed to serialize Item to the clipboard*/
long MyListView::getClipSizeOfItemList(CMyListViewItemList* Items)

{
	if(!Items)
	{
		return 0;
	}
	long l_size = sizeof(Items->count());
	CMyListViewItemListIt item_it( *Items );
	MyListViewItem * item;
	QString name;

	while ( ( item=item_it.current()) != 0 )
	{
		++item_it;
		l_size += item->getdata()->getClipSizeOf();
	}

	return l_size;
}

/** Serialize an instance of this class to a QDataStream that will be put in the clipboard */
bool MyListView::clipSerializeItemList(CMyListViewItemList* Items, QDataStream *s, const bool toClip)
{
	MyListViewItem * item = 0;
	if(toClip)
	{
		if(!Items)
		{
			return false;
		}
		*s << Items->count();
		CMyListViewItemListIt item_it( *Items );
		while ( ( item=item_it.current()) != 0 )
		{
			++item_it;
			item->getdata()->clipSerialize(s, toClip);
  		}
	}
	else
	{
		uint count =0;
		UMLObject * o;
		*s >> count;
		CMyListViewItemData data;
		for(uint i=0; i < count; i++)
		{
   		data.clipSerialize(s, toClip);
   		switch( data.getType() )
   		{
				case Uml::lvt_Actor:
				case Uml::lvt_UseCase:
					o = doc -> findUMLObject(data.getID());
					item = new MyListViewItem(ucv, data.text(0), data.getType(), o);
				break;
				case Uml::lvt_Class:
					o = doc -> findUMLObject(data.getID());
					item = new MyListViewItem(lv, data.text(0), data.getType(), o);
				break;
				/*case Uml::ot_Attribute:
				case Uml::ot_Operation:
					item = new MyListViewItem(parent, text, type, ((UMLConcept*)parent->getUMLObject())->findChildObject(id));
				break;*/
				case Uml::lvt_Logical_Folder:
					item = new MyListViewItem(lv, data.text(0), data.getType());
				break;
				case Uml::lvt_UseCase_Folder:
					item = new MyListViewItem(ucv, data.text(0), data.getType());
				break;
				default:
				break;
			}
			Items->append(item);
			/*for(uint j = 0; j < childs; j++)
			{
				if(!ReadClipChildren(item, s, i, Items))
				{
					return false;
				}
			}*/
		}
	}

	return true;
}
/** Reads the children of a MyListViewItem from a QDataStream
representing the clipboard */
/*bool MyListView::ReadClipChildren(MyListViewItem* parent, QDataStream* s, uint& Count,
																																			CMyListViewItemList* Items)
{
	MyListViewItem * item;
	CMyListViewItemData data;
	UMLObject* o;

	data.clipSerialize(s, false);
	switch(data.getType())
	{
 		case Uml::ot_Actor:
 		case Uml::ot_UseCase:
		case Uml::ot_Concept:
 			o = doc -> findUMLObject(data.getID());
 			item = new MyListViewItem(parent, data.text(0), data.getType(), o);
 		break;
 		//case Uml::ot_Attribute:
 		//case Uml::ot_Operation:
 		//	item = new MyListViewItem(parent, text, type, ((UMLConcept*)parent->getUMLObject())->findChildObject(id));
 		//break;
 		default:
 			item = new MyListViewItem(parent, data.text(0), data.getType());
 		break;
 	}
	Items->append(item);
	Count++; //Increases the number of readed items from the QDataStream
 	for(int j = 0; j < data.childCount(); j++)
 	{
 		if(!ReadClipChildren(item, s, Count, Items))
 		{
 			return false;
 		}
 	}

	return true;
}*/

/** Creates a new MyListViewItem from a CMyListViewItemData,
if parent is null the ListView Decides who is going to be
the parent */
MyListViewItem* MyListView::CreateItem(CMyListViewItemData& Data, CIDChangeLog& IDChanges,
																	MyListViewItem* parent /*= 0*/)
{
	UMLObject* o = 0;
	UMLView *v = 0;
	MyListViewItem* item = 0;
	if(!parent)
	{
		if(Data.getType() == Uml::lvt_Actor || Data.getType() == Uml::lvt_UseCase || Data.getType() == Uml::lvt_UseCase_Folder || Data.getType() == Uml::lvt_UseCase_Diagram)
		{
			parent = ucv;
		}
		else if(Data.getType() > Uml::lvt_UseCase_Diagram && Data.getType() <= Uml::lvt_Sequence_Diagram)
		{
			parent = lv;
		}
		else if(Data.getType() != Uml::lvt_Attribute && Data.getType() != Uml::lvt_Operation)
		{
			parent = lv;
		}
		else
		{
			return 0;
		}
	}
	int newID;
	UMLObject * pObject = 0;
	UMLConcept * pConcept = 0;

	switch(Data.getType())
 	{
		case Uml::lvt_Actor:
		case Uml::lvt_UseCase:
		case Uml::lvt_Class:
			newID = IDChanges.FindNewID(Data.getID());
			//if there is no ListViewItem associated with the new ID,
			//it could exist an Item already asocciated if the user chose to reuse an uml object
			if(!(item = findItem(newID)))
			{
		     	o = doc -> findUMLObject(IDChanges.FindNewID(Data.getID()));
				item = new MyListViewItem(parent, Data.getLabel(), Data.getType(), o);
			}
		break;
		case Uml::lvt_Logical_Folder:
		case Uml::lvt_UseCase_Folder:
			item = new MyListViewItem(parent, Data.getLabel(), Data.getType());
		break;
		case Uml::lvt_Attribute:
			pConcept =  (UMLConcept *)parent -> getUMLObject();
        newID = IDChanges.FindNewID( Data.getID() );
			pObject = pConcept  -> findChildObject( newID );
			if( pObject )
				item = new MyListViewItem( parent, Data.getLabel(), Data.getType(), pObject );
			else
				item = 0;
		break;

		case Uml::lvt_Operation:
			pConcept =  (UMLConcept *)parent -> getUMLObject();
			pObject = pConcept  -> findChildObject( IDChanges.FindNewID( Data.getID() ) );
			if( pObject )
				item = new MyListViewItem( parent, Data.getLabel(), Data.getType(), pObject );
			else
				item = 0;
		break;
		case Uml::lvt_UseCase_Diagram:
		case Uml::lvt_Sequence_Diagram:
		case Uml::lvt_Collaboration_Diagram:
		case Uml::lvt_Class_Diagram:
		case Uml::lvt_State_Diagram:
		case Uml::lvt_Activity_Diagram:
			v = doc->findView(IDChanges.FindNewID(Data.getID()));
			if(!v)
			{
				return 0;
			}
			item = new MyListViewItem(parent, v->GetName(), convert_DT_LVT( v->getType() ),  IDChanges.FindNewID(Data.getID()));
		break;
		default:
		break;
	}
	return item;
}

int MyListView::getSelectedCount()
{
  	QListViewItemIterator it(this);
	int count = 0;
  	// iterate through all items of the list view
  	for ( ; it.current(); ++it )
	{
  		if ( it.current()->isSelected() )
		{
			count++;
		}
  	}

	return count;
}

void MyListView::focusOutEvent ( QFocusEvent * fe)
{
	clearSelection();
	triggerUpdate();
	//repaint();

	QListView::focusOutEvent(fe);
}

Uml::ListView_Type MyListView::convert_DT_LVT(Uml::Diagram_Type dt)
{
  Uml::ListView_Type type =  Uml::lvt_Unknown;
  switch(dt)
  {
    case Uml::dt_UseCase:
      type = Uml::lvt_UseCase_Diagram;
    break;

    case Uml::dt_Class:
      type = Uml::lvt_Class_Diagram;
    break;

    case Uml::dt_Sequence:
      type = Uml::lvt_Sequence_Diagram;
    break;

    case Uml::dt_Collaboration:
      type = Uml::lvt_Collaboration_Diagram;
    break;

    case Uml::dt_State:
      type = Uml::lvt_State_Diagram;
    break;

		case Uml::dt_Activity:
			type = Uml::lvt_Activity_Diagram;
		default:
		break;
  }
  return type;
}

Uml::ListView_Type MyListView::convert_OT_LVT(Uml::UMLObject_Type ot)
{
  Uml::ListView_Type type =  Uml::lvt_Unknown;
  switch(ot)
  {
    case Uml::ot_UseCase:
      type = Uml::lvt_UseCase;
    break;

    case Uml::ot_Actor:
      type = Uml::lvt_Actor;
    break;

    case Uml::ot_Concept:
      type = Uml::lvt_Class;
    break;

    case Uml::ot_Attribute:
      type = Uml::lvt_Attribute;
    break;

    case Uml::ot_Operation:
      type = Uml::lvt_Operation;
    break;
		default:
		break;
  }
  return type;
}

QPixmap & MyListView::getPixmap( Icon_Type type )
{
  switch( type )
  {
    case it_Home:
      return m_Pixmaps.Home;
    break;

    case it_Folder_Green:
      return m_Pixmaps.Folder_Green;
    break;

    case it_Folder_Green_Open:
      return m_Pixmaps.Folder_Green_Open;
    break;

    case it_Folder_Grey:
      return m_Pixmaps.Folder_Grey;
    break;

    case it_Folder_Grey_Open:
      return m_Pixmaps.Folder_Grey_Open;
    break;

    case it_Diagram:
      return m_Pixmaps.Diagram;
    break;

    case it_Class:
      return m_Pixmaps.Class;
    break;

    case it_Actor:
      return m_Pixmaps.Actor;
    break;

    case it_UseCase:
      return m_Pixmaps.UseCase;
    break;

    case it_Public_Method:
      return m_Pixmaps.Public_Method;
    break;

    case it_Private_Method:
      return m_Pixmaps.Private_Method;
    break;

    case it_Protected_Method:
      return m_Pixmaps.Protected_Method;
    break;

    case it_Public_Attribute:
      return m_Pixmaps.Public_Attribute;
    break;

    case it_Private_Attribute:
      return m_Pixmaps.Private_Attribute;
    break;

    case it_Protected_Attribute:
      return m_Pixmaps.Protected_Attribute;
    break;
  }//end switch
  return m_Pixmaps.Home;//you'll know you have a problem if this shows up in the list!!
}

void MyListView::loadPixmaps()
{
  KStandardDirs * dirs = KGlobal::dirs();
  QString dataDir = dirs -> findResourceDir("data", "uml/pics/object.xpm");
  dataDir += "/uml/pics/";

  m_Pixmaps.Home.load( dataDir + "folder_home.png" );
  m_Pixmaps.Folder_Green.load( dataDir + "folder_green.png" );
  m_Pixmaps.Folder_Green_Open.load( dataDir + "folder_green_open.png" );
  m_Pixmaps.Folder_Grey.load( dataDir + "folder_grey.png" );
  m_Pixmaps.Folder_Grey_Open.load( dataDir + "folder_grey_open.png" );
  m_Pixmaps.Diagram.load( dataDir + "CVnamespace.png" ); //change to have different one for each type of diagram
  m_Pixmaps.Class.load( dataDir + "umlclass.xpm" );
  m_Pixmaps.Actor.load( dataDir + "actor.xpm" );
  m_Pixmaps.UseCase.load( dataDir + "case.xpm" );
  m_Pixmaps.Public_Method.load( dataDir + "CVpublic_meth.png" );
  m_Pixmaps.Private_Method.load( dataDir + "CVprivate_meth.png" );
  m_Pixmaps.Protected_Method.load( dataDir + "CVprotected_meth.png" );
  m_Pixmaps.Public_Attribute.load( dataDir + "CVpublic_var.png" );
  m_Pixmaps.Private_Attribute.load( dataDir + "CVprivate_var.png" );
  m_Pixmaps.Protected_Attribute.load( dataDir + "CVprotected_var.png" );
}

void MyListView::slotExpanded( QListViewItem * item )
{
  MyListViewItem * myItem= (MyListViewItem *)item;
  switch( myItem -> getType() )
  {
    case Uml::lvt_Logical_View:
    case Uml::lvt_UseCase_View:
    case Uml::lvt_Logical_Folder:
    case Uml::lvt_UseCase_Folder:
      myItem -> updateFolder();
    break;
		default:
		break;
  }//end switch
}

void MyListView::slotCollapsed( QListViewItem * item )
{
  MyListViewItem * myItem = (MyListViewItem *)item;
  switch( myItem -> getType() )
  {
    case Uml::lvt_Logical_View:
    case Uml::lvt_UseCase_View:
    case Uml::lvt_Logical_Folder:
    case Uml::lvt_UseCase_Folder:
      myItem -> updateFolder();
    break;
		default:
		break;
  }//end switch
}

void MyListView::slotCutSuccessful()
{
	if( m_bStartedCut )
	{
		//deletion code here
		m_bStartedCut = false;
	}
}

void MyListView::addNewItem( QListViewItem * parent, Uml::ListView_Type type )
{
  QString name = "folder";
  MyListViewItem * newItem = dynamic_cast<MyListViewItem *>( parent );//used for att/ops - rest overwrite
  UMLConcept * childParent = 0;
  parent -> setOpen( true );
  switch( type )
  {
    case Uml::lvt_UseCase_Folder:
    case Uml::lvt_Logical_Folder:
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;

    case Uml::lvt_Actor:
      name = getUniqueUMLObjectName( Uml::ot_Actor );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, (UMLObject *)0 );
      newItem -> setPixmap( 0, getPixmap( it_Actor ) );
    break;

    case Uml::lvt_Class:
      name = getUniqueUMLObjectName( Uml::ot_Concept );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, (UMLObject *)0 );
      newItem -> setPixmap( 0, getPixmap( it_Class ) );
    break;

    case Uml::lvt_Attribute:
      childParent = dynamic_cast<UMLConcept *>( newItem -> getdata() -> getUMLObject() );
      name = getUniqueChildUMLObjectName( childParent, Uml::ot_Attribute );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, (UMLObject *)0 );
      newItem -> setPixmap( 0, getPixmap( it_Public_Attribute ) );
    break;

    case Uml::lvt_Operation:
      childParent = dynamic_cast<UMLConcept *>( newItem -> getdata() -> getUMLObject() );
      name = getUniqueChildUMLObjectName( childParent, Uml::ot_Operation );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, (UMLObject *)0 );
      newItem -> setPixmap( 0, getPixmap( it_Public_Method ) );
    break;

    case Uml::lvt_UseCase:
      name = getUniqueUMLObjectName( Uml::ot_UseCase );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, (UMLObject *)0 );
      newItem -> setPixmap( 0, getPixmap( it_UseCase ) );
    break;

    case Uml::lvt_Class_Diagram:
      name = getUniqueDiagramName( Uml::dt_Class );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;

    case Uml::lvt_UseCase_Diagram:
      name = getUniqueDiagramName( Uml::dt_UseCase );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;

    case Uml::lvt_Sequence_Diagram:
      name = getUniqueDiagramName( Uml::dt_Sequence );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;

    case Uml::lvt_Collaboration_Diagram:
      name = getUniqueDiagramName( Uml::dt_Collaboration );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;

    case Uml::lvt_State_Diagram:
      name = getUniqueDiagramName( Uml::dt_State );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;
    case Uml::lvt_Activity_Diagram:
      name = getUniqueDiagramName( Uml::dt_Activity );
      newItem = new MyListViewItem( dynamic_cast<MyListViewItem *>( parent ), name, type, -1 );
    break;
		default:
		break;
  }
  m_bIgnoreCancelRename = false;
  newItem -> setOpen( true );
  newItem -> setCreating( true );
  newItem -> startRename( 0 );
}

void MyListView::slotItemRenamed( QListViewItem * item , int /*col*/ )
{
  //if true the item was cancel before this message
  if( m_bIgnoreCancelRename )
    return;
  m_bIgnoreCancelRename = true;
   MyListViewItem * renamedItem = dynamic_cast< MyListViewItem *>( item ) ;
  Uml::ListView_Type type = renamedItem -> getType();
  QString newText = renamedItem -> text( 0 );
  renamedItem -> setCreating( false );

  //if the length of any type then delete it.
  if( newText.length() == 0 )
  {
    KMessageBox::error( kapp -> mainWidget() , i18n( "The name you entered was invalid!\nCreation process has been cancelled" ),
        i18n( "Name not valid!" ) );
    delete item;
    return;
  }
  // No need to do anything for folders if the string length was > 0
  if( type == Uml::lvt_Logical_Folder || type == Uml::lvt_UseCase_Folder )
    return;

  if( !isUnique( renamedItem, newText ) )
  {
		//if operation ask if ok not to be unique i.e overloading
		if( type == Uml::lvt_Operation )
		{
			if( KMessageBox::warningYesNo( kapp -> mainWidget() , i18n( "The name you entered was not unique!\nIs this what you wanted?" ),
				i18n( "Name not unique!" ) ) == KMessageBox::No )
			{
				delete item;
				return;
			}
		}
		else
		{
			KMessageBox::error( kapp -> mainWidget() , i18n( "The name you entered was not unique!\nCreation process has been cancelled" ),
			i18n( "Name not unique!" ) );
			delete item;
			return;
		}
  }

  switch( type )
  {
    case Uml::lvt_Actor:
      createUMLObject( renamedItem, Uml::ot_Actor );
    break;

    case Uml::lvt_Class:
      createUMLObject( renamedItem, Uml::ot_Concept );
    break;

    case Uml::lvt_UseCase:
      createUMLObject( renamedItem, Uml::ot_UseCase );
    break;

    case Uml::lvt_Attribute:
      createChildUMLObject( renamedItem, Uml::ot_Attribute );
    break;

    case Uml::lvt_Operation:
      createChildUMLObject( renamedItem, Uml::ot_Operation );
    break;

    case Uml::lvt_Class_Diagram:
      createDiagram( renamedItem, Uml::dt_Class );
    break;

    case Uml::lvt_UseCase_Diagram:
      createDiagram( renamedItem, Uml::dt_UseCase );
    break;

    case Uml::lvt_Sequence_Diagram:
      createDiagram( renamedItem, Uml::dt_Sequence );
    break;

    case Uml::lvt_Collaboration_Diagram:
      createDiagram( renamedItem, Uml::dt_Collaboration );
    break;

   case Uml::lvt_State_Diagram:
      createDiagram( renamedItem, Uml::dt_State );
    break;

   case Uml::lvt_Activity_Diagram:
      createDiagram( renamedItem, Uml::dt_Activity );
    break;

		default:
		break;

  }
}

void MyListView::createUMLObject( MyListViewItem * item, Uml::UMLObject_Type type )
{
  QString name = item -> text( 0 );
  UMLObject * object = doc -> findUMLObject( type, name );
  if( object )
  {
    delete item;
    return;
  }
  switch( type )
  {
    case Uml::ot_UseCase:
      object = new UMLUseCase( doc, name, doc -> getUniqueID() );
    break;

    case Uml::ot_Actor:
      object = new UMLActor( doc, name, doc -> getUniqueID() );
    break;

    case Uml::ot_Concept:
      object = new UMLConcept( doc, name, doc -> getUniqueID() );
    break;
		default:
		break;
  }
  doc -> addUMLObject( object );
  CMyListViewItemData * pData = item -> getdata();
  pData -> setUMLObject( object );
  pData -> setID( object -> getID() );
  pData -> setLabel( name );
}

void MyListView::createChildUMLObject( MyListViewItem * item, Uml::UMLObject_Type type )
{
  QString name = item -> text( 0 );
  UMLObject * object = dynamic_cast<MyListViewItem *>( item -> parent() ) -> getUMLObject();
  if( !object )
  {
    delete item;
    return;
  }
  UMLConcept * parent = dynamic_cast<UMLConcept *>( object );

  if( type == Uml::ot_Attribute )
    object = parent -> addAttribute( name, doc -> getUniqueID() );
  else
    object = parent -> addOperation( name, doc -> getUniqueID() );

  CMyListViewItemData * pData = item -> getdata();
  pData -> setUMLObject( object );
  pData -> setID( object -> getID() );
  pData -> setLabel( name );
  loading = true;
  doc -> signalChildUMLObjectCreated( object );
  loading = false;
}

void MyListView::createDiagram( MyListViewItem * item, Uml::Diagram_Type type )
{
  QString name = item -> text( 0 );
  UMLView * view = doc -> findView( type, name );
  if( view )
  {
    delete item;
    return;
  }
  UMLViewData * pData = new UMLViewData();
  pData -> setName( name );
  pData -> setType( type );
  pData -> setID( doc -> getUniqueID() );
  view =  new UMLView( doc -> getSplitter(), pData );
  doc -> addView( view );
  view  -> setOptionState( ((UMLApp *) doc -> parent()) -> getOptionState() );
  CMyListViewItemData * data = item -> getdata();
  data -> setID( view -> getID() );
  data -> setLabel( name );
  view -> Activate();
  doc -> changeCurrentView( view -> getID() );
}

QString MyListView::getUniqueUMLObjectName( Uml::UMLObject_Type type )
{
  QString temp = "",
      name = "",
      newClass = "new class",
      newActor = "new actor",
      newUseCase ="new usecase";

  UMLObject * object = 0;
  int count = 0;

  if( type == Uml::ot_Actor )
    name = newActor;
  else if( type == Uml::ot_UseCase )
    name = newUseCase;
  else
    name = newClass;

  do
  {
    temp = name;
    if( count > 0 )
      temp.append("_").append( QString::number( count ) );
    object = doc -> findUMLObject( type, temp );
    count++;
  }while( object );

  return temp;
}

QString MyListView::getUniqueChildUMLObjectName( UMLConcept * parent, Uml::UMLObject_Type type )
{
  QString name = "",
    temp = "",
    newAtt = "new attribute",
    newOp = "new operation";

  bool cont = true;
  int count = 0;

  if( type == Uml::ot_Attribute )
    name = newAtt;
  else
    name = newOp;

  do
  {
    temp = name;
    if( count > 0 )
      temp.append("_").append( QString::number( count ) );
    cont = parent -> findChildObject( type, temp ).count() != 0;
    count++;
  }while( cont );
  return temp;
}

QString MyListView::getUniqueDiagramName( Uml::Diagram_Type type )
{
  QString name = "",
      temp = "",
      classDiagram = "class diagram",
      seqDiagram = "sequence diagram",
      collDiagram = "collaboration diagram",
      ucDiagram = "usecase diagram",
      stateDiagram = "state diagram",
			activityDiagram = "activity diagram";

  UMLView * view = 0;
  int count = 0;

  if( type == Uml::dt_Class )
    name = classDiagram;
  else if( type == Uml::dt_Sequence )
    name = seqDiagram;
  else if( type == Uml::dt_Collaboration )
    name =collDiagram;
  else if( type == Uml::dt_UseCase )
    name = ucDiagram;
  else if( type == Uml::dt_State )
		name = stateDiagram;
	else
		name = activityDiagram;

  do
  {
    temp = name;
    if( count > 0 )
      temp.append("_").append( QString::number( count ) );
    view = doc -> findView( type, temp );
    count++;
  }while( view );
  return temp;
}

bool MyListView::isUnique( MyListViewItem * item, QString name )
{
  MyListViewItem * parentItem = dynamic_cast<MyListViewItem *>( item -> parent() );
  UMLConcept * parent = dynamic_cast<UMLConcept *>( parentItem  -> getUMLObject() );
  Uml::ListView_Type type = item -> getType();
  switch( type )
  {
    case Uml::lvt_Class_Diagram:
      return !doc -> findView( Uml::dt_Class, name );
    break;

    case Uml::lvt_Sequence_Diagram:
      return !doc -> findView( Uml::dt_Sequence, name );
    break;

    case Uml::lvt_UseCase_Diagram:
      return !doc -> findView( Uml::dt_UseCase, name );
    break;

    case Uml::lvt_Collaboration_Diagram:
      return !doc -> findView( Uml::dt_Collaboration, name );
    break;

    case Uml::lvt_State_Diagram:
      return !doc -> findView( Uml::dt_State, name );
    break;

    case Uml::lvt_Activity_Diagram:
      return !doc -> findView( Uml::dt_Activity, name );
    break;

    case Uml::lvt_Actor:
      return !doc -> findUMLObject( Uml::ot_Actor, name );
    break;

    case Uml::lvt_UseCase:
      return !doc -> findUMLObject( Uml::ot_UseCase, name );
    break;

    case Uml::lvt_Class:
      return !doc -> findUMLObject( Uml::ot_Concept, name );
    break;

    case Uml::lvt_Attribute:
      return ( parent -> findChildObject( Uml::ot_Attribute, name ).count() == 0 );
    break;

    case Uml::lvt_Operation:
      return ( parent -> findChildObject( Uml::ot_Operation, name ).count() == 0 );
    break;
    default:
		break;
  }
  return false;
}

void MyListView::cancelRename( QListViewItem * item )
{
  if( !m_bIgnoreCancelRename )
  {
    delete item;
    m_bIgnoreCancelRename = true;
  }
}

bool MyListView::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement listElement = qDoc.createElement( "listview" );
  bool status = rv -> getdata() -> saveToXMI( qDoc, listElement);
  qElement.appendChild( listElement );
  return status;
}

bool MyListView::loadFromXMI( QDomElement & element )
{
  delete ucv;
  ucv = 0;
  delete lv;
  lv = 0;
  QDomNode node = element.firstChild();
  QDomElement domElement = node.toElement();
  while( !domElement.isNull() )
  {
    if( domElement.tagName() == "listitem" )
    {
      QString type = domElement.attribute( "type", "-1" );
      if( type == "-1" )
        return false;
      Uml::ListView_Type lvType = (Uml::ListView_Type)type.toInt();
      if( lvType == Uml::lvt_View )
      {
        if( !loadChildrenFromXMI( rv, domElement ) )
          return false;
      }
      else
        return false;
    }
    node = node.nextSibling();
    domElement = node.toElement();

  }//end while
  return true;
}

bool MyListView::loadChildrenFromXMI( MyListViewItem * parent, QDomElement & element )
{
  QDomNode node = element.firstChild();
  QDomElement domElement = node.toElement();
  while( !domElement.isNull() )
  {
    if( domElement.tagName() == "listitem" )
    {
      QString id = domElement.attribute( "id", "-1" );
      QString type = domElement.attribute( "type", "-1" );
      QString label = domElement.attribute( "label", "" );
      QString open = domElement.attribute( "open", "1" );
      if( type == "-1" )
        return false;
      Uml::ListView_Type lvType = (Uml::ListView_Type)type.toInt();
      bool bOpen = (bool)open.toInt();
      int nID = id.toInt();
      UMLObject * pObject = 0;
      MyListViewItem * item = 0;
      switch( lvType )
      {
        case Uml::lvt_Actor:
        case Uml::lvt_UseCase:
        case Uml::lvt_Class:
          pObject = doc -> findUMLObject( nID );
          item = new MyListViewItem( parent, label, lvType, pObject );
        break;
        case Uml::lvt_Attribute:
        case Uml::lvt_Operation:
          item = new MyListViewItem( parent, label, lvType, dynamic_cast<UMLConcept *>( parent->getUMLObject() ) -> findChildObject( nID ) );
        break;

        case Uml::lvt_Logical_View:
          item = new MyListViewItem( parent, label, lvType, nID );
          lv = item;
        break;

        case Uml::lvt_UseCase_View:
          item = new MyListViewItem( parent, label, lvType, nID );
          ucv = item;
        break;

        default:
          item = new MyListViewItem( parent, label, lvType, nID );
        break;
      }//end switch
      item -> setOpen( (bool)bOpen );
      if( !loadChildrenFromXMI( item, domElement ) )
        return false;
    }//end if
    node = domElement.nextSibling();
    domElement = node.toElement();
  }//end while
  return true;
}









