#include <qpixmap.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kapplication.h>
#include <kdebug.h>
#include "mylistview.h"
#include "umlview.h"
#include "umldoc.h"
#include "mylistviewitem.h"

MyListViewItem::MyListViewItem(MyListView * parent, QString name, Uml::ListView_Type t,UMLObject*o) : QListViewItem(parent, name)
{
  m_bCreating = false;
  m_pListView = parent;
  m_Data.setType( t );
  m_Data.setUMLObject( o );
  if( !o )
    m_Data.setID( -1 );
  else
    m_Data.setID( o -> getID() );
  setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Home ) );
  m_Data.setListViewItem( this );
  setText( name );
  setRenameEnabled( 0, false );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem::MyListViewItem(MyListViewItem * parent, QString name, Uml::ListView_Type t,UMLObject*o) : QListViewItem(parent, name)
{
  m_bCreating = false;
  m_Data.setType( t );
  m_Data.setUMLObject( o );
  if( !o )
  {
    m_Data.setID( -1 );
    updateFolder();
  }
  else
  {
    updateObject();
    m_Data.setID( o -> getID() );
  }
  if( t == Uml::lvt_Logical_View || t == Uml::lvt_UseCase_View )
    setRenameEnabled( 0, false );
  else
    setRenameEnabled( 0, true );
  m_Data.setListViewItem(this);
  setText( name );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem::MyListViewItem(MyListViewItem * parent, QString name, Uml::ListView_Type t,int id) : QListViewItem(parent, name)
{
  m_bCreating = false;
  m_Data.setType( t );
  m_Data.setUMLObject( 0 );
  m_Data.setID( id );
  setPixmap(0, m_pListView -> getPixmap( MyListView::it_Diagram ) );
  m_Data.setListViewItem( this );
	/*
		Constructor also used by folder so just make sure we don't need to 
		to set pixmap to folder.  doesn't hurt diagrams.
	*/
	updateFolder();
  setText( name );
  setRenameEnabled( 0, true );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListViewItem::~MyListViewItem()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
Uml::ListView_Type MyListViewItem::getType()
{
	return m_Data.getType();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
int MyListViewItem::getID()
{
		return m_Data.getID();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void MyListViewItem::updateObject()
{
  if( !m_Data.getUMLObject() )
    return;

  Uml::Scope scope = m_Data.getUMLObject() -> getScope();
  setText( m_Data.getUMLObject() -> GetName());

  switch(m_Data.getUMLObject() -> getBaseType())
  {
    case Uml::ot_Actor:
      setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Actor ) );
    break;

    case Uml::ot_UseCase:
      setPixmap( 0, m_pListView -> getPixmap( MyListView::it_UseCase ) );
    break;

    case Uml::ot_Concept:
      setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Class ) );
    break;

    case Uml::ot_Operation:
      if( scope == Uml::Public )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Public_Method ) );
      else if( scope == Uml::Private )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Private_Method ) );
      else
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Protected_Method ) );
    break;

    case Uml::ot_Attribute:
      if( scope == Uml::Public )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Public_Attribute ) );
      else if( scope == Uml::Private )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Private_Attribute ) );
      else
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Protected_Attribute ) );
    break;
		default:
		break;
  }//end switch	
}

CMyListViewItemData* MyListViewItem::getdata()
{
  return &m_Data;

}

void MyListViewItem::setdata(CMyListViewItemData& NewData)
{
  m_Data = NewData;
}

void MyListViewItem::updateFolder()
{
  switch( m_Data.getType() )
  {
    case Uml::lvt_UseCase_View:
    case Uml::lvt_UseCase_Folder:
      if( isOpen() )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Folder_Grey_Open ) );
      else
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Folder_Grey ) );
    break;

    case Uml::lvt_Logical_View:
    case Uml::lvt_Logical_Folder:
      if( isOpen() )
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Folder_Green_Open ) );
      else
        setPixmap( 0, m_pListView -> getPixmap( MyListView::it_Folder_Green ) );
    break;
		default:
		break;
  }
}

void MyListViewItem::setOpen( bool open )
{
  QListViewItem::setOpen( open );
  updateFolder();
}

void MyListViewItem::setText( QString text )
{
  m_Data.setLabel( text );
  QListViewItem::setText( 0, text );
  //method was going to have tooltip functions but maybe not needed.
}

void MyListViewItem::okRename( int col )
{
	if( m_bCreating )
	{
		m_bCreating = false;
		QListViewItem::okRename( col );
		m_pListView -> slotItemRenamed( this, 0 );
		return;
	}
	QListViewItem::okRename( col );
	QString newText = text( col );
	UMLObject * object = 0;
	UMLConcept * parent = 0;
	UMLView * view = 0, * anotherView;
	if( newText.length() == 0 )
	{
		KMessageBox::error( kapp -> mainWidget() , i18n( "The name you entered was invalid!\nRenaming process has been cancelled" ),
			i18n( "Name not valid!" ) );
		setText( m_Data.getLabel() );
		return;
	}
	switch( m_Data.getType( ) )
 	{
		case Uml::lvt_UseCase:
		case Uml::lvt_Actor:
		case Uml::lvt_Class:
			object = m_Data.getUMLObject();
			if( object )
			{
        			object = m_pListView -> getDocument() -> findUMLObject( object -> getBaseType(), newText );
				if( object && object == m_Data.getUMLObject() )
					object = 0;
				if( !object )
				{
					m_Data.getUMLObject() -> SetName( newText );
					m_Data.setLabel( newText );
					m_pListView -> getDocument() -> signalUMLObjectChanged( m_Data.getUMLObject() );
					return;
				}
			}
			break;

	case Uml::lvt_Operation:
			object = m_Data.getUMLObject();
			if( object )
			{
				parent = dynamic_cast<UMLConcept *>( object -> parent() );
				//see if op already has that name and not the op/att we are renaming
				//then give a warning about the name being the same
				QPtrList<UMLObject> list = parent -> findChildObject( object -> getBaseType(), newText ); 
				if(list.isEmpty() || (!list.isEmpty() && KMessageBox::warningYesNo( kapp -> mainWidget() , 
					i18n( "The name you entered was not unique!\nIs this what you wanted?" ),
					i18n( "Name not unique!" ) ) == KMessageBox::Yes ))
				{
       					object -> SetName( newText );
       					m_Data.setLabel( newText );
       					m_pListView -> getDocument() -> signalChildUMLObjectUpdate( m_Data.getUMLObject() );
       					return;
				}	
				setText( m_Data.getLabel() );
				return;
			}
			break;

	case Uml::lvt_Attribute:
			object = m_Data.getUMLObject();
			if( object )
			{
				parent = dynamic_cast<UMLConcept *>( object -> parent() );
				QPtrList<UMLObject> list = parent -> findChildObject( object -> getBaseType(), newText ); 
				if (list.isEmpty()) 
				{
					object -> SetName( newText );
					m_Data.setLabel( newText );
					m_pListView -> getDocument() -> signalChildUMLObjectUpdate(object);
					return;
				}
			}
			break;

	case Uml::lvt_UseCase_Diagram:
	case Uml::lvt_Class_Diagram:
	case Uml::lvt_Sequence_Diagram:
	case Uml::lvt_Collaboration_Diagram:
	case Uml::lvt_State_Diagram:
	case Uml::lvt_Activity_Diagram:
			view = m_pListView -> getDocument() -> findView( m_Data.getID() );
			if( view )
			{
				anotherView = m_pListView -> getDocument() -> findView( view -> getType(), newText );
				if( anotherView && anotherView -> getID() == m_Data.getID() )
					anotherView = 0;
				if( !anotherView )
				{
					view -> SetName( newText );
					m_Data.setLabel( newText );
					m_pListView -> getDocument() -> signalDiagramRenamed( view );
       					return;
				}
			}
			break;
	case Uml::lvt_UseCase_Folder:
	case Uml::lvt_Logical_Folder:
			m_Data.setLabel( newText );
			return;
			break;
	default:
			break;
	}
	KMessageBox::error( kapp -> mainWidget() , i18n( "The name you entered was invalid!\nRenaming process has been cancelled" ),
	i18n( "Name not valid!" ) );
	setText( m_Data.getLabel() );
	return;
}

void MyListViewItem::cancelRename( int col )
{
  QListViewItem::cancelRename( col );
  if( m_bCreating )
    m_pListView -> cancelRename( this );
}

MyListView * MyListViewItem::m_pListView = 0;




