#include <qdatastream.h>
#include "mylistviewitemdata.h"
#include "mylistviewitem.h"
#include "umlobject.h"
#include <kdebug.h>

CMyListViewItemData::CMyListViewItemData()
{
	m_pItem = 0;
	object = 0;
	m_nChildren = 0;
}

CMyListViewItemData::CMyListViewItemData(CMyListViewItemData& Other)
{
	*this = Other;
}

CMyListViewItemData::~CMyListViewItemData()
{
}

void CMyListViewItemData::setListViewItem(MyListViewItem* Item)
{
	m_pItem = Item;
}

MyListViewItem* CMyListViewItemData::getListViewItem()
{
	return m_pItem;
}

int CMyListViewItemData::childCount()
{
	if(m_pItem)
	{
		m_nChildren = m_pItem->childCount();

		return m_nChildren;
	}
	
	return m_nChildren;
}

long CMyListViewItemData::getClipSizeOf()
{
 	long l_size = sizeof(m_nId) + sizeof(int) + sizeof(int);
 	Q_UINT32 tmp; //tmp is used to calculate the size of each serialized null string	
 	QString name = (QString)text(0);

 	if (!name.length()) //We assume we are working with QT 2.1.x or superior, that means
 																//if unicode returns a null pointer then the serialization process of the QString object
 															 	//will write a null marker 0xffffff, see QString::operator<< implementation
  	{
  		l_size += sizeof(tmp);
  	}
  	else
  	{
  		l_size += (name.length()*sizeof(QChar));
  	}
	l_size += sizeof(Q_INT32);//open status

	return l_size;
}

bool CMyListViewItemData::clipSerialize(QDataStream *s, const bool toClip)
{
	int open = 1;
	if( m_pItem )
		open = (int)m_pItem -> isOpen();
	if(toClip)
	{
		int count = childCount();
		int t = m_Type;
		*s << m_nId
    	  	 << t
    	 	 << text(0)
    	 	 << count	
			<<  open;
	}
	else
	{
			int t;
	  *s >> m_nId
   		 >> t
   		 >> label
   		 >> m_nChildren
			>> open;
			m_Type = (Uml::ListView_Type)t;
			if( m_pItem )
				m_pItem -> setOpen( (bool) open );
	}

	return true;
}

bool CMyListViewItemData::serialize(QDataStream *s, bool archive)
{
	int open = 1;
	if( m_pItem )
		open = (int)m_pItem -> isOpen();
	if(archive)
	{
		int t = m_Type;
		int count = childCount();
		*s << m_nId
				<< t
				<< text(0)
				<< count
				<< open;
	}
	else
	{
			
		  int t;
		*s >> m_nId
			 >> t
			 >> label
			 >> m_nChildren
			 >> open;
			m_Type = (Uml::ListView_Type)t;
			if( m_pItem )
				m_pItem -> setOpen( (bool)open );
	}

	return false;
}

QString CMyListViewItemData::text(int index)
{
	if(m_pItem)
	{
		label = m_pItem->text(index);
		return label;
	}
	return label;
}

CMyListViewItemData& CMyListViewItemData::operator=(CMyListViewItemData & Other)
{
	m_pItem = Other.m_pItem;
	object = Other.object;
	m_nId = Other.m_nId;
	m_Type = Other.m_Type;
	m_nChildren = Other.childCount();
	label = Other.text(0);
	
	return *this;
}

bool CMyListViewItemData::operator==(const CMyListViewItemData& Other)
{
	if(m_pItem != Other.m_pItem)
	{
		return false;
	}
	if(!(object == Other.object)) /*It should be compared using UMLObject::operator==*/
	{
		return false;
	}
	if(m_nId != Other.m_nId)
	{
		return false;
	}
	if(m_Type != Other.m_Type)
	{
		return false;
	}
	if(m_nChildren != Other.m_nChildren)
	{
		return false;
	}
	if(label != Other.label)
	{
		return false;
	}

	return true;
}

bool CMyListViewItemData::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement itemElement = qDoc.createElement( "listitem" );
  itemElement.setAttribute( "id", m_nId );
  itemElement.setAttribute( "type", m_Type );
  itemElement.setAttribute( "label", label );
  itemElement.setAttribute( "open", m_pItem -> isOpen() );
  MyListViewItem * childItem = dynamic_cast<MyListViewItem *> ( m_pItem -> firstChild() );
  while( childItem )
  {
    bool status = childItem -> getdata() -> saveToXMI( qDoc, itemElement );
    if( !status )
      return false;
    childItem = dynamic_cast<MyListViewItem *> ( childItem -> nextSibling() );
  }
  qElement.appendChild( itemElement );
  return true;
}


