#include <qdatastream.h>
#include <kdebug.h>

#include "notewidgetdata.h"

CNoteWidgetData::CNoteWidgetData() : CUMLWidgetData()
{
	m_bLinkDocumentation = false;
	m_Text  = "";
  m_Type = Uml::wt_Note;
}

CNoteWidgetData::CNoteWidgetData(CNoteWidgetData & Other) : CUMLWidgetData(Other)
{
     *this = Other;
}

CNoteWidgetData::~CNoteWidgetData()
{

}

CNoteWidgetData & CNoteWidgetData::operator=(CNoteWidgetData & Other)
{
	*((CUMLWidgetData*)this) = (CUMLWidgetData)Other;
	m_bLinkDocumentation = Other.m_bLinkDocumentation;
	m_Text  = Other.m_Text;
	return *this;
}

bool CNoteWidgetData::operator==(CNoteWidgetData & Other)
{
    if(!(*((CUMLWidgetData*)this) == (CUMLWidgetData)Other))
	{
		return false;
	}
	if(m_bLinkDocumentation != Other.m_bLinkDocumentation)
	{
		return false;
	}
	if(m_Text  != Other.m_Text)
	{
		return false;
	}
	return true;
}

QString CNoteWidgetData::getText()
{
	return m_Text;
}

void CNoteWidgetData::setText( QString Text)
{
	m_Text = Text;
}

long CNoteWidgetData::getClipSizeOf()
{
	long l_size = CUMLWidgetData::getClipSizeOf();
	Q_UINT32 tmp; //tmp is used to calculate the size of each serialized null string
	
	QString name;
	if(m_bLinkDocumentation == true)
	{
    	name = "@LINKDOCS@";
	}
	else
	{
		name = m_Text;
	}

	if ( !name.length() ) 
	{
		l_size += sizeof(tmp);
	}
	else
	{
		l_size += (name.length()*sizeof(QChar));
	}

	return l_size;
}

bool CNoteWidgetData::getLinkDocumentation()
{
	return m_bLinkDocumentation;
}

void CNoteWidgetData::setLinkDocumentation( bool LinkDocumentation)
{
	m_bLinkDocumentation = LinkDocumentation;
}

bool CNoteWidgetData::serialize(QDataStream *s, bool archive)
{
	if(!CUMLWidgetData::serialize(s, archive))
	{
		return false;
	}
	if(archive)
	{	
		QString saveString;
		if(m_bLinkDocumentation == true)
		{
      		saveString = "@LINKDOCS@";
		}
      	else
		{
      		saveString = m_Text;
		}

  		*s << saveString;
	}
	else
	{
		//Temporary store the text
		*s	>> m_Text;
	}

	return true;
}

bool CNoteWidgetData::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement noteElement = qDoc.createElement( "UML:NoteWidget" );
  bool status = CUMLWidgetData::saveToXMI( qDoc, noteElement );
  noteElement.setAttribute( "text", m_Text );
  qElement.appendChild( noteElement );
  return status;
}

bool CNoteWidgetData::loadFromXMI( QDomElement & qElement )
{
  if( !CUMLWidgetData::loadFromXMI( qElement ) )
    return false;
  m_Text = qElement.attribute( "text", "" );
  return true;
}




