#include <qpainter.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include "seqlinewidget.h"
#include "umlview.h"
#include "objectwidget.h"

ObjectWidget::ObjectWidget(UMLView * view,  UMLObject *o, CUMLWidgetData *pData) : UMLWidget(view, o, pData)
{
	m_pLine = 0;
	init();
	if( m_pLine )
		m_pLine -> setLineLength( ((CObjectWidgetData*)m_pData) -> getLineLength() );
}

ObjectWidget::ObjectWidget(UMLView * view, UMLObject *o, int lid) : UMLWidget(view, o, new CObjectWidgetData())
{
	m_pLine = 0;
	((CObjectWidgetData*)m_pData)->m_nLocalID = lid;
	((CObjectWidgetData*)m_pData)->m_InstanceName = "";
	((CObjectWidgetData*)m_pData)->m_bMultipleInstance = false;
	m_pData->setType(wt_Object);
	calculateSize();
  init();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ObjectWidget::ObjectWidget(UMLView * view) : UMLWidget(view, new CObjectWidgetData())
{
	m_pLine = 0;
	((CObjectWidgetData*)m_pData)->m_nLocalID = -1;
	((CObjectWidgetData*)m_pData)->m_InstanceName = "";
	((CObjectWidgetData*)m_pData)->m_bMultipleInstance = false;
  init();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectWidget::init()
{            
	m_Doc = "";
	m_nOldID = 0;
  if( m_pView -> getType() == dt_Sequence )
  {
    m_pLine = new SeqLineWidget( m_pView, this );
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////
ObjectWidget::~ObjectWidget()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectWidget::draw(QPainter & p , int offsetX, int offsetY)
{
	if( ( (CObjectWidgetData *) m_pData ) -> getDrawAsActor() )
		drawActor( p, offsetX, offsetY );
	else
		drawObject( p, offsetX, offsetY );

	p.setPen(m_LineColor);
	if(m_bSelected)
		drawSelected(&p, offsetX, offsetY);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectWidget::slotMenuSelection(int sel)
{
	QString name = "";
	switch(sel)
	{
		case ListPopupMenu::mt_Rename_Object:

			bool ok;
			name = KLineEditDlg::getText(i18n("Enter object name"), ((CObjectWidgetData*)m_pData)->m_InstanceName, &ok, m_pView);
			if(ok)
			{
				((CObjectWidgetData*)m_pData)->m_InstanceName = name;
				calculateSize();
				moveEvent( 0 );
				update();
			}
		break;
		
		case ListPopupMenu::mt_Properties:
			m_pView->getDocument() -> showProperties(this);
			calculateSize();
			moveEvent( 0 );
			update();
		break;
		
		case ListPopupMenu::mt_Up:
			tabUp();
		break;
		
		case ListPopupMenu::mt_Down:
			tabDown();
		break;

		default:
			UMLWidget::slotMenuSelection(sel);
		break;
	}
}	
////////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectWidget::calculateSize()
{
	int width, height, textWidth;
	QFont font = m_pData -> getFont();
	font.setUnderline( true );
	QFontMetrics fm = QFontMetrics( font );
	int fontHeight  = fm.lineSpacing();
	int oldHeight = this -> height();
	if( ( (CObjectWidgetData *)m_pData ) -> getDrawAsActor() )
	{
		QString t = ((CObjectWidgetData*)m_pData)->m_InstanceName + " : " + m_pObject -> GetName();
		textWidth = fm.width( t );
		width = textWidth > A_WIDTH?textWidth:A_WIDTH;
		height = A_HEIGHT + fontHeight + A_MARGIN;
		width += A_MARGIN * 2;
	}
	else
	{ 
		QString t = ((CObjectWidgetData*)m_pData)->m_InstanceName + " : " + m_pObject -> GetName();
		textWidth = fm.width(t);
		width = textWidth > O_WIDTH?textWidth:O_WIDTH;
		height = fontHeight + O_MARGIN * 2;
		width += O_MARGIN * 2;
		if(((CObjectWidgetData*)m_pData)->m_bMultipleInstance)
		{
			width += 10;
			height += 10;
		}
	}//end else drawasactor
	setSize(width, height);
	if( m_pView -> getType() == dt_Sequence )
		setY( (int)y() + (oldHeight - height ) );
	emit sigWidgetMoved(((CObjectWidgetData*)m_pData)->m_nLocalID);//makes any message widgets connected resize themselves
 	AdjustAssocs( (int)x(), (int)y() );//adjust assoc lines
	moveEvent( 0 );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void ObjectWidget::setMultipleInstance(bool multiple)
{
	//make sure only calling this in relation to an object on a collab. diagram
	if(m_pView -> getType() != dt_Collaboration)
		return;
	((CObjectWidgetData*)m_pData)->m_bMultipleInstance = multiple;
	calculateSize();
	update();
}

/** Activate the object after serializing it from a QDataStream */

bool ObjectWidget::Activate(CIDChangeLog* ChangeLog /*= 0*/)
{
	bool status = UMLWidget::Activate(ChangeLog);
	if(status)
	{
		calculateSize();
	}
	moveEvent( 0 );
	return status;
}

void ObjectWidget::SynchronizeData()
{
	UMLWidget::SynchronizeData();
	if( m_pLine )
		((CObjectWidgetData*)m_pData)-> setLineLength( m_pLine -> getLineLength() );
}

void ObjectWidget::moveEvent(QMoveEvent */*m*/)
{
  emit sigWidgetMoved( m_pData->getId() );
  if( m_pLine )
    m_pLine -> setStartPoint( (int)x() + width() / 2, (int)y() + height() );
}

void ObjectWidget::slotColorChanged(int /*viewID*/)
{
  m_FillColor = m_pView -> getBGColor();
  m_LineColor = m_pView -> getLineColor();

  if( m_pLine)
    m_pLine -> setPen( QPen( m_LineColor, 0, DashLine ) );
}

void ObjectWidget::slotLineLengthChanged()
{
  moveEvent( 0 );
}

void ObjectWidget::cleanup()
{

  UMLWidget::cleanup();
  if( m_pLine )
	{
		m_pLine -> cleanup();
    delete m_pLine;
	}
}

void ObjectWidget::drawObject(QPainter & p, int offsetX, int offsetY)
{

	p.setPen(m_LineColor);
	if(m_pData->getUseFillColor())
		p.setBrush(m_FillColor);
	else
		p.setBrush(m_pView -> viewport() -> backgroundColor());
	int w = width();
	int h= height();
	QFont font = m_pData -> getFont();


	font.setUnderline( true );
	p.setFont( font );
	QString t = ((CObjectWidgetData*)m_pData)->m_InstanceName + " : " + m_pObject -> GetName();
	if(((CObjectWidgetData*)m_pData)->m_bMultipleInstance)
	{
		p.drawRect(offsetX + 10, offsetY + 10, w - 10, h - 10);
		p.drawRect(offsetX + 5, offsetY + 5, w - 10, h - 10);
		p.drawRect(offsetX, offsetY, w - 10, h - 10);
		p.setPen(QPen(black));

		p.drawText(offsetX + O_MARGIN, offsetY + O_MARGIN, w - O_MARGIN * 2 - 10, h - O_MARGIN * 2 - 10, AlignCenter, t);    
	}
	else
	{
		p.drawRect(offsetX, offsetY, w, h);
		p.setPen(QPen(black));
		p.drawText(offsetX + O_MARGIN, offsetY + O_MARGIN, w - O_MARGIN * 2, h - O_MARGIN * 2, AlignCenter, t);
	}
}

void ObjectWidget::drawActor(QPainter & p, int offsetX, int offsetY)
{
	p.setPen(m_LineColor);
	if(m_pData->getUseFillColor())
		p.setBrush(m_FillColor);
	int w = width();
	int textStartY = A_HEIGHT + A_MARGIN;
	QFont font = m_pData -> getFont();
	font.setUnderline( true );
	p.setFont( font );
	QFontMetrics fm = QFontMetrics( m_pData -> getFont() );
	int fontHeight  = fm.lineSpacing();
	
	int middleX = w / 2;
	int thirdY = A_HEIGHT / 3;
	
	//draw actor
	p.drawEllipse(offsetX + middleX - A_WIDTH / 2, offsetY,  A_WIDTH, thirdY);//head
	p.drawLine(offsetX + middleX, offsetY + thirdY, offsetX + middleX, offsetY + thirdY * 2);//body
	p.drawLine(offsetX + middleX, offsetY + 2 * thirdY, offsetX + middleX - A_WIDTH / 2, offsetY + A_HEIGHT);//left leg

	p.drawLine(offsetX + middleX, offsetY +  2 * thirdY, offsetX + middleX + A_WIDTH / 2, offsetY + A_HEIGHT);//right leg
	p.drawLine(offsetX + middleX - A_WIDTH / 2, offsetY + thirdY + thirdY / 2, offsetX + middleX + A_WIDTH / 2, offsetY + thirdY + thirdY / 2);//arms
	//draw text
	p.setPen(QPen(black));
	QString t = ((CObjectWidgetData*)m_pData)->m_InstanceName + " : " + m_pObject -> GetName();
	p.drawText(offsetX + A_MARGIN, offsetY + textStartY, w - A_MARGIN * 2, fontHeight, AlignCenter, t);  
}

void ObjectWidget::mouseMoveEvent(QMouseEvent *me)
{
  int newX = 0, newY = 0, count;
  int moveX, moveY;
	if( !m_bSelected )
		m_pView -> setSelected( this, me );
  m_bSelected = true;
  count = m_pView -> getSelectCount();

	if( !m_bMouseDown )
	  if( me -> button() != LeftButton )
   	 return;
  //If not m_bStartMove means moving as part of selection
  //me->pos() will have the amount we need to move.
  if(!m_bStartMove)
  {
    moveX = (int)me -> x();
    moveY = (int)me -> y();
  }
  else
  {
    //we started the move so..
    //move any others we are selected
    moveX = (int)me -> x() - m_nOldX - m_nPressOffsetX;  
    moveY = (int)me -> y() - m_nOldY - m_nPressOffsetY;
	 if( ((int)x() + moveX) < 0 )
		moveX = 0;
	 if( ( (int)y() + moveY) < 0 )
		moveY = 0;
    if( count > 1 )
		if( m_pView -> getType() == dt_Sequence )
      	m_pView -> moveSelected( this, moveX, 0 ); 
    	else
      	m_pView -> moveSelected( this, moveX, moveY );
  }
  newX = (int)x() + moveX;
  newY = (int)y() + moveY;

  newX = newX < 0?0:newX;
  newY = newY < 0?0:newY;
  //implement rule for sequence diagram
  if( m_pView -> getType() == dt_Sequence )
      newY = (int)this -> y();
  m_nOldX = newX;
  m_nOldY = newY;
  setX( newX );
  setY( newY );  
  AdjustAssocs(newX, newY);
	moveEvent(0);
}

void ObjectWidget::tabUp()
{
	int newY = (int)y() - height();
	newY = newY + height() < 80?80 - height():newY;
	setY( newY );
	moveEvent( 0 );
	AdjustAssocs( (int)x(), newY);
}

void ObjectWidget::tabDown()
{
	int newY = (int)y() + height();
	setY( newY );
	moveEvent( 0 );
	AdjustAssocs( (int)x(), newY);
}

void ObjectWidget::setShowDeconstruction( bool bShow )
{
	( ( CObjectWidgetData *)m_pData ) -> setShowDeconstruction( bShow ); 
	if( m_pLine )
	m_pLine -> setupDeconBox();
}

int ObjectWidget::getEndLineY()
{
	int y = (int)this -> y() + height();
	if( m_pLine)
		y += m_pLine -> getLineLength();
	if( ( ( CObjectWidgetData *)m_pData ) -> getShowDeconstruction() )
		y += 10;
	return y;
}













