#include <qdatastream.h>
#include <kdebug.h>

#include "objectwidgetdata.h"

CObjectWidgetData::CObjectWidgetData(): CUMLWidgetData()
{
  m_Type = Uml::wt_Object;
	m_InstanceName = "";
	m_bMultipleInstance= false;
	m_nLocalID = -1;
	m_bDrawAsActor = false;
	m_bShowDeconstruction = false;
	m_nLineLength = 0;
}

CObjectWidgetData::CObjectWidgetData(CObjectWidgetData & Other) : CUMLWidgetData(Other)
{
    *this = Other;
}

CObjectWidgetData::~CObjectWidgetData()
{

}

CObjectWidgetData & CObjectWidgetData::operator=(CObjectWidgetData & Other)
{
	*((CUMLWidgetData*)this) = (CUMLWidgetData)Other;
	m_InstanceName = Other.m_InstanceName;
	m_bMultipleInstance= Other.m_bMultipleInstance;
	m_nLocalID = Other.m_nLocalID;
	m_bDrawAsActor = Other.m_bDrawAsActor;
	m_bShowDeconstruction = Other.m_bShowDeconstruction;
	return *this;
}

bool CObjectWidgetData::operator==(CObjectWidgetData & Other)
{
	if(!(*((CUMLWidgetData*)this) == (CUMLWidgetData)Other))
	{
		return false;
	}
	if(m_InstanceName != Other.m_InstanceName)
	{
		return false;
	}
	if(m_bMultipleInstance != Other.m_bMultipleInstance)
	{
		return false;
	}
	if(m_nLocalID != Other.m_nLocalID)
	{
		return false;
	}
	if( m_bDrawAsActor != Other.m_bDrawAsActor )
		return false;
	if( m_bShowDeconstruction != Other.m_bShowDeconstruction )
		return false;

	return true;
}

QString CObjectWidgetData::getInstanceName()
{
	return m_InstanceName;
}

void CObjectWidgetData::setInstanceName( QString InstanceName)
{
	m_InstanceName = InstanceName;
}

int CObjectWidgetData::getLocalID()
{
	return m_nLocalID;
}

void CObjectWidgetData::setLocalID( int LocalID)
{
	m_nLocalID = LocalID;
}

bool CObjectWidgetData::getMultipleInstance()
{
	return m_bMultipleInstance;
}

void CObjectWidgetData::setMultipleInstance( bool MultipleInstance)
{
	m_bMultipleInstance = MultipleInstance;
}

long CObjectWidgetData::getClipSizeOf()
{

	long l_size = CUMLWidgetData::getClipSizeOf();
	l_size += sizeof(m_nLocalID);
	l_size += sizeof( int ); //m_bMultipleInstance
	l_size += sizeof( int ); //m_bDrawAsActor
	l_size += sizeof( int ); //m_bShowDeconstruction
	l_size += sizeof( int ); //m_nLineLength
	Q_UINT32 tmp; //tmp is used to calculate the size of each serialized null string

	if ( !m_InstanceName.length() ) //We assume we are working with QT 2.1.x or superior, that means
																	//if unicode returns a null pointer then the serialization process of the QString object
																 	//will write a null marker 0xffffff, see QString::operator<< implementation
	{

		l_size += sizeof(tmp);
	}
	else
	{
		l_size += (m_InstanceName.length()*sizeof(QChar));
	}

	return l_size;
}

bool CObjectWidgetData::serialize(QDataStream *s, bool archive)
{
	if(!CUMLWidgetData::serialize(s, archive))
	{
		return false;
	}
	if(archive)
	{
		*s << m_nLocalID
				<< m_InstanceName
				<< (int)m_bMultipleInstance
				<< (int)m_bDrawAsActor
				<< (int)m_bShowDeconstruction
				<< m_nLineLength;
	}
	else
	{
		int multi, actor, decon;
		*s >> m_nLocalID
			>> m_InstanceName
			>> multi
			>> actor
			>> decon
			>> m_nLineLength;
		m_bMultipleInstance = multi;
		m_bDrawAsActor = actor;
		m_bShowDeconstruction = decon;
	}
	
	return true;
}

void CObjectWidgetData::print2cerr()
{
	CUMLWidgetData::print2cerr();
	kdDebug() << "m_InstanceName = " << m_InstanceName << endl;
	if(m_bMultipleInstance)
	{
		kdDebug() << "m_bMultipleInstance = " << "true" << endl;
	}
	else
	{
		kdDebug() << "m_bMultipleInstance = " << "false" << endl;
	}
	kdDebug() << "m_nLocalID = " << m_nLocalID << endl;
}

bool CObjectWidgetData::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement objectElement = qDoc.createElement( "UML:ObjectWidget" );
  bool status = CUMLWidgetData::saveToXMI( qDoc, objectElement );
  objectElement.setAttribute( "instancename", m_InstanceName );
  objectElement.setAttribute( "drawasactor", m_bDrawAsActor );
  objectElement.setAttribute( "multipleinstance", m_bMultipleInstance );
  objectElement.setAttribute( "localid", m_nLocalID );
  objectElement.setAttribute( "decon", m_bShowDeconstruction );
  objectElement.setAttribute( "length", m_nLineLength );
  qElement.appendChild( objectElement );
  return status;
}

bool CObjectWidgetData::loadFromXMI( QDomElement & qElement )
{
  if( !CUMLWidgetData::loadFromXMI( qElement ) )
    return false;
  m_InstanceName = qElement.attribute( "instancename", "" );
  QString draw = qElement.attribute( "drawasactor", "0" );
  QString multi = qElement.attribute( "multipleinstance", "0" );
  QString localid = qElement.attribute( "localid", "0" );
  QString decon = qElement.attribute( "decon", "0" );
  QString length = qElement.attribute( "length", "0" );

  m_bDrawAsActor = (bool)draw.toInt();
  m_bMultipleInstance = (bool)multi.toInt();
  m_nLocalID = localid.toInt();
	m_bShowDeconstruction = (bool)decon.toInt();
	m_nLineLength = length.toInt();
  return true;

}




