#include <klineeditdlg.h>
#include <klocale.h>
#include "statewidgetdata.h"
#include "umlwidget.h"
#include "umlview.h"
#include "dialogs/statedialog.h"
#include "statewidget.h"

StateWidget::StateWidget(UMLView * view, CUMLWidgetData* pData) : UMLWidget(view, pData)
{	
}

StateWidget::StateWidget(UMLView * view, StateType stateType ) : UMLWidget(view, new CStateWidgetData())
{
  dynamic_cast<CStateWidgetData *>( m_pData ) -> setStateType( stateType );
	m_pData->setType(wt_State);
	calculateSize();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
StateWidget::StateWidget(UMLView * view) : UMLWidget(view, new CStateWidgetData())
{
	m_pData->setType(wt_State);
	calculateSize();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
StateWidget::~StateWidget()
{
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void StateWidget::draw(QPainter & p, int offsetX, int offsetY)
{
  int w = width();
  int h = height();
  StateType type = dynamic_cast<CStateWidgetData *>( m_pData ) -> getStateType();
  if( type == Normal )
  {
    p.setPen(m_LineColor);
    if(m_pData->getUseFillColor())
      p.setBrush(m_FillColor);
    QFontMetrics fm = QFontMetrics( m_pData -> getFont() );
    int fontHeight  = fm.lineSpacing();
    //int middleX = w / 2;
    int textStartY = (h / 2) - (fontHeight / 2);
		QStringList list = dynamic_cast<CStateWidgetData *>( m_pData ) -> getActivityList(); 
		int count = list.count();
		if( count == 0 )
		{
			p.drawRoundRect(offsetX, offsetY, w, h, 75, 75);
    	p.setPen(black);
			QFont font = m_pData -> getFont();
			p.setFont( font );
    	p.drawText(offsetX + STATE_MARGIN, offsetY + textStartY, w - STATE_MARGIN * 2, fontHeight, AlignCenter, GetName());
			p.setPen(m_LineColor);
    }
		else
		{
			p.drawRoundRect(offsetX, offsetY, w, h, 35, 35);
			textStartY = offsetY + STATE_MARGIN;
			p.setPen(black);   
			QFont font = m_pData -> getFont();
			font.setBold( true );
			p.setFont( font );
			p.drawText(offsetX + STATE_MARGIN, textStartY, w - STATE_MARGIN * 2, fontHeight, AlignCenter, GetName());
			font.setBold( false );
			p.setFont( font );
			p.setPen(m_LineColor);
			int linePosY = textStartY + fontHeight;
			for( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
			{
				textStartY += fontHeight;
				p.drawLine( offsetX, linePosY, offsetX + w, linePosY );
				p.setPen(black);
				p.drawText(offsetX + STATE_MARGIN, textStartY, w - STATE_MARGIN * 2 - 1, fontHeight, AlignCenter, *it);
				p.setPen(m_LineColor);
				linePosY += fontHeight;
			}//end for
		}//end else
  }
  else if( type == Initial )
  {
    p.setPen(m_LineColor);
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX, offsetY, w, h );
  }
  else
  {
    p.setPen(m_LineColor);
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX, offsetY, w, h );
    p.setBrush( white );
    p.drawEllipse( offsetX + 1, offsetY + 1, w - 2, h - 2 );
    p.setBrush( m_LineColor );
    p.drawEllipse( offsetX + 3, offsetY + 3, w - 6, h - 6 );
  }
  if(m_bSelected)
    drawSelected(&p, offsetX, offsetY);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void StateWidget::calculateSize()
{
	int width = 10, height = 10;
  if( dynamic_cast<CStateWidgetData *>( m_pData ) -> getStateType() == Normal )
  {
		QFont font = m_pData -> getFont();
		font.setBold( true );//use bold for all calculations
    QFontMetrics fm = QFontMetrics( font );
    int fontHeight  = fm.lineSpacing();
    int textWidth = fm.width(GetName());
		QStringList list = dynamic_cast<CStateWidgetData *>( m_pData ) -> getActivityList(); 
		int count = list.count();
		height = fontHeight;
		if( count > 0 )
		{
			height = fontHeight * ( count + 1);
			for( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
			{
				int w = fm.width( *it );
				if( w > textWidth )
					textWidth = w; 
			}//end for
		}//end if
	  width = textWidth > STATE_WIDTH?textWidth:STATE_WIDTH;
		height = height > STATE_HEIGHT?height:STATE_HEIGHT;
    width += STATE_MARGIN * 2;
		height += STATE_MARGIN * 2;
  }
  setSize(width, height);
}

void StateWidget::SynchronizeData()
{


	//Nothing to synchronize
	UMLWidget::SynchronizeData();
}

void StateWidget::SetName(QString strName)
{
  dynamic_cast<CStateWidgetData *>( m_pData ) -> setName( strName );
  calculateSize();
  AdjustAssocs( (int)x(), (int)y() );
}

QString StateWidget::GetName()
{
  return dynamic_cast<CStateWidgetData *>( m_pData ) -> getName();
}

QString StateWidget::getDoc()
{
  return dynamic_cast<CStateWidgetData *>( m_pData ) -> getDoc();
}

void StateWidget::setDoc( QString doc )
{
  dynamic_cast<CStateWidgetData *>( m_pData ) -> setDoc( doc );
}

StateWidget::StateType StateWidget::getStateType()
{
  return dynamic_cast<CStateWidgetData *>( m_pData ) -> getStateType();
}

void StateWidget::setStateType( StateType stateType )
{
  dynamic_cast<CStateWidgetData *>( m_pData ) -> setStateType( stateType );
}

void StateWidget::slotMenuSelection(int sel)
{
  bool done = false;
  bool ok = false;
  QString name = GetName();

  switch( sel )
  {
    case ListPopupMenu::mt_Rename:
      name = KLineEditDlg::getText( i18n("Enter State name."), i18n("Enter the name of the new State."), GetName(), &ok );
      if( ok && name.length() > 0 )
        SetName( name );
      done = true;
    break;

    case ListPopupMenu::mt_Properties:
      mouseDoubleClickEvent( 0 );
      done = true;
    break;
		case ListPopupMenu::mt_New_Activity:
			name = KLineEditDlg::getText( i18n("Enter Activity."), i18n("Enter the name of the new Activity."), i18n("new activity"), &ok );
			if( ok && name.length() > 0 )
				addActivity( name );
			done = true;
		break;
  }

  if( !done )
    UMLWidget::slotMenuSelection( sel );
}

void StateWidget::mouseDoubleClickEvent(QMouseEvent * /*me*/)
{
  m_pView -> updateDocumentation( false );
  StateDialog dialog( m_pView, this );
  if( dialog.exec() && dialog.getChangesMade() )
  {
    //put here anything that needs to be done for changes
    //nothing at the moment.
  }
  m_pView -> showDocumentation( this, true );
}

bool StateWidget::addActivity( QString activity )
{
	bool result = dynamic_cast<CStateWidgetData *>( m_pData ) -> addActivity( activity );
	if( result )
	{
		calculateSize();
	}
	return result;
}

bool StateWidget::removeActivity( QString activity )
{
	bool result = dynamic_cast<CStateWidgetData *>( m_pData ) -> removeActivity( activity );
	if( result )
	{
		calculateSize();
	}
	return result;
}

void StateWidget::setActivities( QStringList & list )
{
	dynamic_cast<CStateWidgetData *>( m_pData ) -> setActivities( list );
	calculateSize();
}

QStringList & StateWidget::getActivityList()
{
	return dynamic_cast<CStateWidgetData *>( m_pData ) -> getActivityList();
}

bool StateWidget::renameActivity( QString activity, QString newName )
{
	return dynamic_cast<CStateWidgetData *>( m_pData ) -> renameActivity( activity, newName );
}





