// include files for QT
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qclipboard.h>
#include <qtimer.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <klocale.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kprinter.h>
#include <kcursor.h>
#include <kstatusbar.h>
#include <kdebug.h>
#include <kstandarddirs.h>

// application specific includes
#include "codegenerator.h"
#include "dialogs/configcodegenerators.h"
#include "dialogs/selectlanguagesdlg.h"
#include "dialogs/codegenerationwizard.h"
#include "umlview.h"
#include "dialogs/diagramprintpage.h"
#include "dialogs/classwizard.h"
#include "clipboard/umlclipboard.h"
#include "clipboard/cumldrag.h"
#include "umldoc.h"
#include "cclassimport.h"
#include "umlnamespace.h"
#include "uml.h"
#include <stdlib.h> //neded for setting up LD_LIBRARY_PATH

UMLApp::UMLApp(QWidget* , const char* name):KMainWindow(0, name)
{
	m_pDocWindow = 0;
	config=kapp->config();
	listView = 0;
	ldict.setAutoDelete(true);
	langSelect = new QPopupMenu(this);
	loading = false;
	m_clipTimer = 0;
	m_copyTimer = 0;
  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
	readOptionState();
  initActions();
  initDocument();
  initView();
  initClip();
  initStatusBar();
	readOptions();
  if( optionState.uiState.showDocWindow )
		m_pDocWindow -> show();
	else
		m_pDocWindow -> hide();
  ///////////////////////////////////////////////////////////////////
  // disable actions at startup
  fileSave->setEnabled(true);
  fileSaveAs->setEnabled(true);
  filePrint->setEnabled(false);
  editCut->setEnabled(false);
  editCopy->setEnabled(false);
  editPaste->setEnabled(false);

  if(  optionState.generalState.lastFile != "" && optionState.generalState.diagram != SettingsDlg::diagram_no )
  {
    switch( optionState.generalState.diagram )
   {
     case SettingsDlg::diagram_usecase:
       doc -> createDiagram( Uml::dt_UseCase, false);
     break;

     case SettingsDlg::diagram_class:
       doc -> createDiagram( Uml::dt_Class, false );
     break;

     case SettingsDlg::diagram_sequence:
       doc -> createDiagram( Uml::dt_Sequence, false );
     break;

      case SettingsDlg::diagram_collaboration:
       doc -> createDiagram( Uml::dt_Collaboration, false );
      break;

      case SettingsDlg::diagram_state:
       doc -> createDiagram( Uml::dt_State, false );
      break;

      case SettingsDlg::diagram_activity:
       doc -> createDiagram( Uml::dt_Activity, false );
      break;

			default:
			break;
   }//end switch
  }//end if new diagram
	else
		doc -> newDocument();
	doc -> setModified( false );
	slotUpdateViews();


  //insert Language Selection menu in "Code Generation" in the Menubar
  // --> the name "Code Generation" as seen in the Menu is taken from .ui file..
  // and depends on the language we are using (english, spanish, etc) so we need to
  // look for the object name instead.
  int t = menuBar() -> count();
  int id=-1;
  for( int i =0; i<t;i++)
    {
      id = menuBar()->idAt(i);
      if(id == -1)break;
      if(menuBar()->findItem(id)->popup()->name() == QString("code"))
        break;
    }
   if(id!=-1)
    {
      QPopupMenu *m = menuBar()->findItem(id)->popup();
      if(m)
        m->insertItem(i18n("Active Language"),langSelect,-1,m->count()-1);
    }

  initLibraries();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
UMLApp::~UMLApp()
{
     if(m_clipTimer)
	 {
			delete m_clipTimer;
	 }
	 if(m_copyTimer)
	 {
			delete m_copyTimer;
	 }

		delete statProg;
		delete m_statusLabel;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::initActions()
{
  fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
  fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
  fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
  fileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection());
  fileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection());
  fileClose = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
  filePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection());
  fileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection());
  editCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection());
  editCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection());
  editPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection());
  viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
  viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());

	classWizard = new KAction(i18n("Class Wizard..."),0,this,SLOT(slotClassWizard()),
			actionCollection(),"class_wizard");

	showDocumentation = new KToggleAction( i18n("Show Documentation") , 0 , this, SLOT( slotShowDocWindow() ),
														actionCollection(), "Show_Documentation" );
	preferences = new KAction( i18n("Configure UML...") , 0 , this, SLOT( slotPrefs() ),
														actionCollection(), "Configure_UML" );

	genWizard = new KAction(i18n("Code Generation Wizard..."),0,this,SLOT(generationWizard()),
			actionCollection(),"generation_wizard");
  genAll = new KAction(i18n("Generate All Code"),0,this,SLOT(generateAllCode()),
			actionCollection(),"generate_all");

	importClasses = new KAction(i18n("Import Classes..."),0,this,SLOT(slotImportClasses()),
			actionCollection(),"import_class");

  confLanguages = new KAction(i18n("Add / Remove Languages"),0,this,
                        SLOT(configureLanguages()),actionCollection(),"configure_languages");

  fileNew->setStatusText(i18n("Creates a new document"));
  fileOpen->setStatusText(i18n("Opens an existing document"));
  fileOpenRecent->setStatusText(i18n("Opens a recently used file"));
  fileSave->setStatusText(i18n("Saves the actual document"));
  fileSaveAs->setStatusText(i18n("Saves the actual document as..."));
  fileClose->setStatusText(i18n("Closes the actual document"));
  filePrint ->setStatusText(i18n("Prints out the actual document"));
  fileQuit->setStatusText(i18n("Quits the application"));
  editCut->setStatusText(i18n("Cuts the selected section and puts it to the clipboard"));
  editCopy->setStatusText(i18n("Copies the selected section to the clipboard"));
  editPaste->setStatusText(i18n("Pastes the clipboard contents to actual position"));
  viewToolBar->setStatusText(i18n("Enables/disables the toolbar"));
  viewStatusBar->setStatusText(i18n("Enables/disables the statusbar"));
	preferences -> setStatusText( i18n( "Set the default program preferences") );
    // The different views
	classDiagram = new KAction( i18n( "&Class Diagram" ), 0,
		this, SLOT( slotClassDiagram() ), actionCollection(), "new_class_diagram" );

	sequenceDiagram= new KAction( i18n( "&Sequence Diagram" ), 0,
		this, SLOT( slotSequenceDiagram() ), actionCollection(), "new_sequence_diagram" );

	collaborationDiagram = new KAction( i18n( "&Collaboration Diagram" ), 0,
		this, SLOT( slotCollaborationDiagram() ), actionCollection(), "new_collaboration_diagram" );

	useCaseDiagram= new KAction( i18n( "&Use Case Diagram" ), 0,
		this, SLOT( slotUseCaseDiagram() ), actionCollection(), "new_use_case_diagram" );

	stateDiagram= new KAction( i18n( "S&tate Diagram" ), 0,
		this, SLOT( slotStateDiagram() ), actionCollection(), "new_state_diagram" );

	activityDiagram= new KAction( i18n( "&Activity Diagram" ), 0,
		this, SLOT( slotActivityDiagram() ), actionCollection(), "new_activity_diagram" );

	viewProp = new KAction( i18n( "&Properties" ), 0,
		this, SLOT( slotViewProp() ), actionCollection(), "view_prop" );

	viewProp -> setEnabled( false );

	showDocumentation -> setChecked( optionState.uiState.showDocWindow );
	showDocumentation -> setStatusText( i18n( "Enables/disables the documentation window" ) );
  // use the absolute path to your umlui.rc file for testing purpose in createGUI();
  createGUI();

}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
  statProg = new QProgressBar(statusBar(),"Progressbar");
  statProg->setFixedWidth( 100 );             // arbitrary width
  statProg->setCenterIndicator(true);
  statProg->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
  statProg->setMargin( 0 );
  statProg->setLineWidth(0);
  statProg->setBackgroundMode( QWidget::PaletteBackground );
  statProg->setFixedHeight( statProg->sizeHint().height() - 8 );

  m_statusLabel = new KStatusBarLabel( i18n("Ready."), 0, statusBar() );
  m_statusLabel->setFixedHeight( m_statusLabel->sizeHint().height() );

  m_statusLabel->setFrameStyle( QFrame::NoFrame | QFrame::Plain );
  m_statusLabel->setMargin( 0 );
  m_statusLabel->setLineWidth(0);

  statusBar()->addWidget( m_statusLabel, 1, false );
  statusBar()->addWidget(statProg, 0,  true);

  m_statusLabel->setAlignment(AlignLeft|AlignVCenter);

  connect(doc,SIGNAL(sigSetStatusbarProgressSteps(int)),statProg,SLOT(setTotalSteps(int)));

  connect(doc,SIGNAL(sigSetStatusbarProgress(int)),statProg,SLOT(setProgress(int)));
  connect(doc,SIGNAL(sigResetStatusbarProgress()),statProg,SLOT(reset()));

  connect(listView,SIGNAL(sigSetStatusbarProgressSteps(int)),statProg,SLOT(setTotalSteps(int)));
  connect(listView,SIGNAL(sigSetStatusbarProgress(int)),statProg,SLOT(setProgress(int)));
  connect(listView,SIGNAL(sigResetStatusbarProgress()),statProg,SLOT(reset()));

	connect(doc, SIGNAL( sigWriteToStatusBar(const QString &) ), this, SLOT( slotStatusMsg(const QString &) ));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::initDocument()
{
  doc = new UMLDoc(this);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::initView()
{
	splitter = new QSplitter( this,"SPLITTER" );
	m_pDocSplitter = new QSplitter( splitter ,"DOC SPLITTER" );
	m_pDocSplitter -> setOrientation( Vertical );

	listView = new MyListView( m_pDocSplitter ,"LISTVIEW" );
	m_pDocSplitter -> setResizeMode( listView, QSplitter::Stretch );
	m_pDocWindow = new DocWindow( doc, m_pDocSplitter, "DOCWINDOW" );
	m_pDocWindow -> setMaximumSize( this -> width(), this -> height() / 2 );
	if( optionState.uiState.showDocWindow )
		m_pDocWindow -> show();
	else
		m_pDocWindow -> hide();
	m_pDocSplitter -> setResizeMode( m_pDocWindow, QSplitter::Stretch );
	splitter -> setResizeMode( m_pDocSplitter, QSplitter::KeepSize );
	toolsbar = new WorkToolBar(this,"WORKTOOLBAR");
	addToolBar(toolsbar,Right,true);
	doc -> setupListView(listView);//make sure has a link to list view and add info widget
  doc -> setupSignals();//make sure gets signal from list view
  listView -> setDocument(doc);
  setCentralWidget( splitter );
  setCaption(doc->URL().fileName(),false);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::openDocumentFile(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));

  doc->openDocument( url);
  fileOpenRecent->addURL( url );
  slotStatusMsg(i18n("Ready."));
  setCaption(url.fileName(), false);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
UMLDoc *UMLApp::getDocument() const
{
  return doc;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void UMLApp::saveOptions()
{
  toolBar("mainToolBar")->saveSettings(config, "toolbar");
  toolsbar->saveSettings(config, "workbar");
  fileOpenRecent->saveEntries(config,"Recent Files");
  config -> setGroup( "General Options" );
	config->writeEntry("Geometry", size());
  config -> writeEntry( "autosave", optionState.generalState.autosave );
  config -> writeEntry( "time", optionState.generalState.time );
  config -> writeEntry( "logo", optionState.generalState.logo );
  config -> writeEntry( "loadlast", optionState.generalState.loadlast );

  config -> writeEntry( "diagram", optionState.generalState.diagram );
	if( doc->URL().fileName() == i18n( "Untitled" ) )
		config -> writeEntry( "lastFile", "" );

	else
		config -> writeEntry( "lastFile", doc -> URL().prettyURL() );
  config -> setGroup( "TipOfDay");
  optionState.generalState.tip = config -> readBoolEntry( "RunOnStart", true );
  config -> writeEntry( "RunOnStart", optionState.generalState.tip );

  config -> setGroup( "UI Options" );
  config -> writeEntry( "useFillColor", optionState.uiState.useFillColor );
  config -> writeEntry( "fillColor", optionState.uiState.fillColor );
  config -> writeEntry( "lineColor", optionState.uiState.lineColor );
	config -> writeEntry( "showDocWindow", optionState.uiState.showDocWindow );
	config -> writeEntry( "font", optionState.uiState.font );


  config -> setGroup( "Class Options" );
  config -> writeEntry( "showVisibility", optionState.classState.showScope );
  config -> writeEntry( "showAtts", optionState.classState.showAtts);
  config -> writeEntry( "showOps", optionState.classState.showOps );
  config -> writeEntry( "showStereoType", optionState.classState.showStereoType );
  config -> writeEntry( "showAttSig", optionState.classState.showAttSig );
  config -> writeEntry( "ShowOpSig", optionState.classState.showOpSig );
  config -> writeEntry( "showPackage", optionState.classState.showPackage );

  config -> setGroup("Code Generation");
  config -> writeEntry("forceDoc",optionState.codegenState.forceDoc);
  config -> writeEntry("forceSections",optionState.codegenState.forceSections);
  config -> writeEntry("outputDirectory",optionState.codegenState.outputDir);
  config -> writeEntry("headingsDirectory",optionState.codegenState.headingsDir);
  config -> writeEntry("includeHeadings",optionState.codegenState.includeHeadings);
  config -> writeEntry("overwritePolicy",optionState.codegenState.overwritePolicy);
  config -> writeEntry("modnamePolicy",  optionState.codegenState.modname);
  config -> writeEntry("activeLanguage",activeLanguage);

  //save library information
  QStringList libsknown;
  QDict<QStringList> llist;
  llist.setAutoDelete(true);
  QDictIterator<GeneratorInfo> it( ldict );
  QStringList templ;
  for(it.toFirst() ; it.current(); ++it )
       {
        if(!libsknown.contains(it.current()->library))
          libsknown.append(it.current()->library);
        if(!llist.find(it.current()->library))
          llist.insert(it.current()->library,new QStringList());

          llist[it.current()->library]->append(it.current()->language);
          llist[it.current()->library]->append(it.current()->object);
       }

  config -> writeEntry("libsKnown",libsknown);
  QDictIterator<QStringList> sit(llist);
  for(sit.toFirst();sit.current();++sit)
    config -> writeEntry(sit.currentKey(),**sit);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::readOptions()
{
  // bar status settings
  toolBar("mainToolBar")->applySettings(config, "toolbar");
  //do config for work toolbar
  toolsbar->applySettings(config, "workbar");
  fileOpenRecent->loadEntries(config,"Recent Files");
  config->setGroup("General Options");
  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////

void UMLApp::saveProperties(KConfig *_config)
{
  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
  {
    // saving to tempfile not necessary

  }
  else
  {
    KURL url=doc->URL();
    _config->writeEntry("filename", url.url());
    _config->writeEntry("modified", doc->isModified());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);

    KURL _url(tempurl);
    doc->saveDocument(_url);
  }
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::readProperties(KConfig* _config)
{
  QString filename = _config->readEntry("filename", "");
  KURL url(filename);
  bool modified = _config->readBoolEntry("modified", false);
  if(modified)
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL _url(tempname);


    if(canRecover)
    {
      doc->openDocument(_url);
      doc->setModified();
      setCaption(_url.fileName(),true);
      QFile::remove(tempname);
    }
  }
  else
  {
    if(!filename.isEmpty())
    {
      doc->openDocument(url);
      setCaption(url.fileName(),false);

    }
  }

}
////////////////////////////////////////////////////////////////////////////////////////////////////

bool UMLApp::queryClose()
{
  return doc->saveModified();


}
////////////////////////////////////////////////////////////////////////////////////////////////////
bool UMLApp::queryExit()
{
  saveOptions();
  doc -> deleteContents();
  return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileNew()
{
	slotStatusMsg(i18n("Creating new document..."));
	if(!doc->saveModified())
	{
	}
	else
	{
		doc->newDocument();
		setCaption(doc->URL().fileName(), false);
		fileOpenRecent->setCurrentItem( -1 );
		setModified( false );
	}
	viewProp -> setEnabled( false );
	slotUpdateViews();
	slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileOpen()
{
  slotStatusMsg(i18n("Opening file..."));
	loading = true;

  if(!doc->saveModified())
  {

     // here saving wasn't successful

  }
  else

  {
    KURL url=KFileDialog::getOpenURL(QString::null, i18n("*.xmi|XMI Files\n*.*|All Files"), this, i18n("Open File..."));
    if(!url.isEmpty())
    {
      if(doc->openDocument(url))
      	fileOpenRecent->addURL( url );
      setCaption(doc->URL().fileName(), false);
    }

  }
	slotUpdateViews();
	loading = false;
  slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileOpenRecent(const KURL& url)
{
  slotStatusMsg(i18n("Opening file..."));
	loading = true;





	KURL oldURL = doc->URL();

  if(!doc->saveModified())
  {
     // here saving wasn't successful
  }
  else
  {
    if(!doc->openDocument(url))
  	{
     	fileOpenRecent->removeURL(url);
     	fileOpenRecent->setCurrentItem( -1 );
   }
    setCaption(doc->URL().fileName(), false);
  }

	loading = false;
	slotUpdateViews();
  slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));
	if(doc->URL().fileName() == i18n("Untitled"))
		slotFileSaveAs();
	else
	  doc->saveDocument(doc -> URL());

 	slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
bool UMLApp::slotFileSaveAs()

{
    slotStatusMsg(i18n("Saving file with a new filename..."));
    bool cont = true;
    KURL url;
    QString ext;
    while(cont)
    {
        url=KFileDialog::getSaveURL(QDir::currentDirPath(), i18n("*.xmi|XMI Files\n*.*|All Files"), this, i18n("Save as..."));

        if(url.isEmpty())
            cont = false;
        else
        {

            QString file = url.path(-1);
            QFileInfo info(file);
            ext = info.extension(false);
            if(ext != "xmi")
                url.setFileName(url.fileName() + ".xmi");
            QDir d = url.path(-1);

            if(QFile::exists(d.path()))
            {
                int want_save = KMessageBox::warningYesNo(this, i18n("The file %1 exists.\nDo you wish to overwrite it?").arg(url.path()), i18n("Warning"));
                if(want_save == KMessageBox::Yes)
                    cont = false;
            }
            else cont = false;

        }
    }
    if(!url.isEmpty()){
        doc->saveDocument(url);
        fileOpenRecent->addURL(url);
   	setCaption(url.fileName(),doc->isModified());
        slotStatusMsg(i18n("Ready."));
        return true;

    } else {
        slotStatusMsg(i18n("Ready."));
        return false;
    }
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));

  slotFileNew();

}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFilePrint()

{
  	slotStatusMsg(i18n("Printing..."));

  	KPrinter printer;
  	printer.setFullPage(true);
  	DiagramPrintPage * selectPage = new DiagramPrintPage(0, doc);
	printer.addDialogPage(selectPage);
	QString msg;
 	if (printer.setup(this))
	{

		 	doc -> print(&printer);
  	}
  	slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotFileQuit()
{
	slotStatusMsg(i18n("Exiting..."));
  	if(!doc->saveModified())
  	{

  	}
  	else
  	{
  		saveOptions();
		kapp -> quit();
	}
  	slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotEditCut()
{
	//Up to user to delete the selection.
  slotStatusMsg(i18n("Cutting selection..."));
  	bool  fromview = (doc->getCurrentView() && doc->getCurrentView()->getSelectCount());
	if( EditCutCopy( fromview ) )
		emit sigCutSuccessful();
  slotStatusMsg(i18n("Ready."));
	doc -> setModified( true );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotEditCopy()
{
  	slotStatusMsg(i18n("Copying selection to clipboard..."));
	bool  fromview = (doc->getCurrentView() && doc->getCurrentView()->getSelectCount());
	EditCutCopy( fromview );
	slotStatusMsg(i18n("Ready."));
	doc -> setModified( true );
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotEditPaste()
{
   slotStatusMsg(i18n("Inserting clipboard contents..."));
	QMimeSource * data = QApplication::clipboard()->data();
	CUMLClipboard clipboard;
	setCursor(KCursor::waitCursor());
   if(!clipboard.paste(doc, data))
	{
		slotStatusMsg(i18n("paste error"));
	}
	else
	{
  		slotStatusMsg(i18n("Ready."));

	}
	setCursor(KCursor::arrowCursor());
   editPaste->setEnabled(false);
	doc -> setModified( true );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggling toolbar..."));

  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off

  if(!viewToolBar->isChecked())
  {
    toolBar("mainToolBar")->hide();
  }
  else
  {
    toolBar("mainToolBar")->show();
  }


  slotStatusMsg(i18n("Ready."));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if(!viewStatusBar->isChecked())
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  slotStatusMsg(i18n("Ready."));
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
	m_statusLabel->setText( text );

	if ( loading )
		m_statusLabel->repaint();
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::slotClassDiagram()
{
    getDocument() -> createDiagram( Uml::dt_Class ) ;
}
void UMLApp::slotSequenceDiagram()
{
    getDocument() -> createDiagram( Uml::dt_Sequence );
}
void UMLApp::slotCollaborationDiagram()
{
    getDocument() -> createDiagram( Uml::dt_Collaboration );
}
void UMLApp::slotUseCaseDiagram()
{
    getDocument() -> createDiagram( Uml::dt_UseCase );
}

void UMLApp::slotStateDiagram()

{
    getDocument() -> createDiagram( Uml::dt_State );
}

void UMLApp::slotActivityDiagram()
{
    getDocument() -> createDiagram( Uml::dt_Activity );
}


WorkToolBar* UMLApp::getWorkToolBar()
{
	return toolsbar;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::setModified(bool _m)
{
	//fileSave -> setEnabled(_m);

	//if anything else needs to be done on a mofication, put it here


	//if a file is modified, allow printing
	//we don't stop printing just because a file is modified, if a file has content
	//then we allow printing
	if(_m)
		enablePrint(_m);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLApp::enablePrint(bool _p)
{
	filePrint -> setEnabled(_p);
}
/** initialize the QT's global clipboard support for the application */
void UMLApp::initClip()
{
	QClipboard* clip = QApplication::clipboard();
	connect(clip, SIGNAL(dataChanged()), this, SLOT(slotClipDataChanged()));


	m_clipTimer = new QTimer(this, "timer");
    m_clipTimer->start(1000, FALSE);
    connect(m_clipTimer, SIGNAL(timeout()), this, SLOT(slotClipDataChanged()));

	m_copyTimer = new QTimer(this, "copytimer");
	m_copyTimer->start(500, FALSE);
	connect(m_copyTimer, SIGNAL(timeout()), this, SLOT(slotCopyChanged()));
}


void UMLApp::slotClipDataChanged()
{
	QMimeSource * data = QApplication::clipboard()->data();

	//Pass the MimeSource to the Doc
	editPaste->setEnabled( data && CUMLDrag::canDecode(data) );
}

void UMLApp::slotCopyChanged()
{
	if(listView->getSelectedCount() || (doc->getCurrentView() && doc->getCurrentView()->getSelectCount()))
	{
		editCopy->setEnabled(true);
		editCut->setEnabled(true);
	}
	else
	{
		editCopy->setEnabled(false);
		editCut->setEnabled(false);
	}
}




void UMLApp::slotPrefs() {
	/* the KTipDialog may have changed the value */
	config -> setGroup( "TipOfDay");
	optionState.generalState.tip = config -> readBoolEntry( "RunOnStart", true );

	dlg = new SettingsDlg( this, optionState );
        connect(dlg, SIGNAL( applyClicked() ), this, SLOT( slotApplyPrefs() ) );

	if( dlg->exec() == QDialog::Accepted && dlg->getChangesApplied() ) {
		slotApplyPrefs();
	}

        disconnect(dlg, SIGNAL( applyClicked() ), this, SLOT( slotShowDocWindow() ) );
	delete dlg; dlg = NULL;
}

void UMLApp::slotApplyPrefs() {
	if (dlg) {
		//changes made so let the document tell the views
		optionState = dlg->getOptionState();

		/* we need this to sync both values */
		config -> setGroup( "TipOfDay");
		config -> writeEntry( "RunOnStart", optionState.generalState.tip );

		if( optionState.uiState.showDocWindow != showDocumentation->isChecked() ) {
			slotShowDocWindow();
		}
		doc -> settingsChanged( optionState );
	}
}

bool UMLApp::getPasteState()
{
	return editPaste -> isEnabled();
}

bool UMLApp::getCutCopyState()
{
	return editCopy -> isEnabled();
}

bool UMLApp::EditCutCopy( bool bFromView )
{
	CUMLClipboard clipboard;
	QMimeSource * clipdata = 0;
	if((clipdata = clipboard.copy(doc, bFromView)) != 0)
	{
		QClipboard* clip = QApplication::clipboard();
		clip->setData(clipdata);//the global clipboard takes ownership of the clipdata memory
	 	connect(clip, SIGNAL(dataChanged()), this, SLOT(slotClipDataChanged()));
		return true;
	}
	return false;
}

void UMLApp::slotShowDocWindow() {
	if(! m_pDocWindow -> isVisible() ) {
		m_pDocWindow -> show();
	} else {
		m_pDocWindow -> hide();
	}
	showDocumentation -> setChecked( m_pDocWindow -> isVisible() );
}

void UMLApp::readOptionState()
{
  config -> setGroup( "General Options" );
  optionState.generalState.autosave = config -> readBoolEntry( "autosave", false );
  optionState.generalState.time = config -> readNumEntry( "time", 0 );
  optionState.generalState.logo = config -> readBoolEntry( "logo", true );
  optionState.generalState.loadlast = config -> readBoolEntry( "loadlast", true );

  optionState.generalState.diagram  = ( SettingsDlg::Diagram ) config -> readNumEntry( "diagram", 1 );
  config -> setGroup( "TipOfDay");

  optionState.generalState.tip = config -> readBoolEntry( "RunOnStart", true );

  config -> setGroup( "UI Options" );
  optionState.uiState.useFillColor = config -> readBoolEntry( "useFillColor", true );
  QColor defaultYellow = QColor( 255, 255, 192 );

  optionState.uiState.fillColor = config -> readColorEntry( "fillColor", &defaultYellow );
  optionState.uiState.lineColor = config -> readColorEntry( "lineColor", &red );
	optionState.uiState.showDocWindow = config -> readBoolEntry( "showDocWindow", true );
	QFont font = ((QWidget *) this)->font() ;
	optionState.uiState.font = config -> readFontEntry("font", &font );

  config -> setGroup( "Class Options" );

  optionState.classState.showScope = config -> readBoolEntry( "showVisibility", false );
  optionState.classState.showAtts = config -> readBoolEntry( "showAtts", true );
  optionState.classState.showOps = config -> readBoolEntry( "showOps", true );
  optionState.classState.showStereoType = config -> readBoolEntry( "showStereoType", false );
  optionState.classState.showAttSig = config -> readBoolEntry( "showAttSig", false );
  optionState.classState.showOpSig = config -> readBoolEntry( "ShowOpSig", false );
  optionState.classState.showPackage = config -> readBoolEntry( "showPackage", false );

  config -> setGroup("Code Generation");
  optionState.codegenState.forceDoc = config -> readBoolEntry("forceDoc",true);
  optionState.codegenState.forceSections = config -> readBoolEntry("forceSections",false);
  QString temp = config -> readEntry("outputDirectory","");
  if(temp.isEmpty())
    temp = QDir::homeDirPath() + "/uml-generated-code/";
  optionState.codegenState.outputDir = temp;
  optionState.codegenState.includeHeadings = config->readBoolEntry("includeHeadings",true);
  temp = "";
  temp = config -> readEntry("headingsDirectory","");
  if(temp.isEmpty())
    {
      KStandardDirs stddirs;
      temp =  stddirs.findDirs("data","uml/headings").first();
    }
  optionState.codegenState.headingsDir = temp;
	optionState.codegenState.overwritePolicy = (CodeGenerator::OverwritePolicy)config -> readNumEntry("overwritePolicy",CodeGenerator::Ask);
  optionState.codegenState.modname = (CodeGenerator::ModifyNamePolicy)config -> readNumEntry("modnamePolicy",CodeGenerator::Capitalise);
}

CodeGenerator* UMLApp::generator()
{
  GeneratorInfo *info;
  if(activeLanguage.isEmpty())
    {
      KMessageBox::sorry(this,i18n("There is no Active Language defined.\nPlease select\
one of the installed languages to generate the code in"),i18n("No Language selected"));
      return 0;
    }
  info = ldict.find(activeLanguage);
  if(!info) {kdDebug()<<"error looking up library information (dictionary)"<<endl;return 0;}

  KLibLoader *loader = KLibLoader::self();
  if(!loader) {kdDebug()<<"error getting KLibLoader!"<<endl;return 0;}

  KLibFactory *fact = loader->factory(info->library);
  if(!fact) {kdDebug()<<"error getting the Factory"<<endl;return 0;}

  QObject *o=fact->create(0,0,info->object);
  if(!o) {kdDebug()<<"could not create object"<<endl;return 0;}

  CodeGenerator *g = (CodeGenerator*)o;
  g->setDocument(doc);
  return g;
}

void UMLApp::configureLanguages()
{
  ConfigCodeGenerators d(ldict,this);
  if(d.exec())
  {
    ldict.clear();
    ldict = d.configDictionary();
    updateLangSelectMenu();
  }
}

void UMLApp::generateAllCode()
{
  CodeGenerator *gen = generator();
  gen->setForceDoc(optionState.codegenState.forceDoc);
  gen->setForceSections(optionState.codegenState.forceSections);
  gen->setIncludeHeadings(optionState.codegenState.includeHeadings);
  gen->setHeadingFileDir(optionState.codegenState.headingsDir);
  gen->setModifyNamePolicy(optionState.codegenState.modname);
  gen->setOutputDirectory(optionState.codegenState.outputDir);
  gen->setOverwritePolicy(optionState.codegenState.overwritePolicy);
  gen->generateAllClasses();
  delete gen;
}

void UMLApp::generationWizard()
{
  CodeGenerator *gen = generator();
  CodeGenerationWizard wizard(gen,doc,0,optionState.codegenState,this);
  wizard.exec();
  delete gen;
}



void UMLApp::setActiveLanguage(int id)
{
  uint i;
  for(i=0; i <langSelect->count(); i++)
    langSelect->setItemChecked(langSelect->idAt(i),false);  //uncheck everything
  langSelect->setItemChecked(id,true);
  activeLanguage = langSelect->text(id);
}

void UMLApp::slotViewProp()
{
	doc -> getCurrentView() -> showPropDialog();
}

void UMLApp::setViewPropState( bool bState )
{
	viewProp -> setEnabled( bState );
}

void UMLApp::slotUpdateViews()
{
	int count = menuBar() -> count();
	int id=-1;
	for( int i =0; i<count;i++)
	{
		id = menuBar()->idAt(i);
		if(id == -1)
			return;
		if(menuBar()->findItem(id)->popup()->name() == QString("views"))
			break;
	}
	if(id == -1)
		return;
	QPopupMenu *menu = menuBar()->findItem(id)->popup();
	if( !menu )
		return;
	count = menu -> count();
	id = -1;
	for( int i =0; i<count;i++)
	{
		id = menu -> idAt( i );
		QString text = menu -> findItem( id ) -> text();
		if( text == i18n("Show") )
			break;
	}
	if( id == -1 )
		return;
	menu = menu -> findItem( id ) -> popup();
	if( !menu )
		return;
//	count = menu -> count();
//	for( int i =0; i<count;i++)

//		menu -> removeItemAt( i );
	menu -> clear();
	for(QListIterator<UMLView> views = getDocument()->getViewIterator();views.current(); ++views)
		menu -> insertItem( views.current()->GetName(), views.current(), SLOT( slotShowView() ) );
}

void UMLApp::slotImportClasses()
{
	QStringList fileList = KFileDialog::getOpenFileNames(QString::null,
		i18n("*.h|Header Files (*.h)\n*.*|All Files"), this, i18n("Select Classes to Import...") );
	((CClassImport*)doc)->importCPP( fileList );
}

void UMLApp::slotClassWizard()
{
	ClassWizard dlg( doc );
	dlg.exec();
}
void UMLApp::initLibraries()
{
  KStandardDirs stdDirs;
  QStringList libsKnown, libsFound, libsMissing,strlist;
  QString str;
  config -> setGroup("Code Generation");
  activeLanguage = config->readEntry("activeLanguage","");
  //first search for libraries in all "codegenerators" directoris, and then
  //see which of them are already known and which are new
  libsFound = stdDirs.findAllResources("data","uml/codegenerators/*.la",false,true);

  QFileInfo fi;
  for ( QStringList::Iterator it = libsFound.begin(); it != libsFound.end(); ++it )
    {
      fi.setFile(*it);        //leav only filename (no path, no extension)
      *it = fi.fileName().left(fi.fileName().length()-3);
     }


  libsKnown = config->readListEntry("libsKnown",':');

  for ( QStringList::Iterator it = libsKnown.begin(); it != libsKnown.end(); ++it )
    if(libsFound.contains(*it))
        libsFound.remove(*it);
    else
        libsMissing.append(*it); //it was registered and now is gone,
                                  // add to missing libs.


  for ( QStringList::Iterator it = libsMissing.begin(); it != libsMissing.end(); ++it )
           libsKnown.remove(*it);


//debug output..
      for ( QStringList::Iterator it = libsKnown.begin(); it != libsKnown.end(); ++it )
          kdDebug()<<"Known lib: "<<*it<<endl;
      for ( QStringList::Iterator it = libsFound.begin(); it != libsFound.end(); ++it )
          kdDebug()<<"New lib: "<<*it<<endl;
      for ( QStringList::Iterator it = libsMissing.begin(); it != libsMissing.end(); ++it )
          kdDebug()<<"Missing lib: "<<*it<<endl;
///////////////

  //add paths for the library loader.
  QString ld_path = getenv("LD_LIBRARY_PATH");
  if(!ld_path.isEmpty()) ld_path +=":";

  QStringList dirs = stdDirs.findDirs("data","uml/codegenerators/");
  for ( QStringList::Iterator it = dirs.begin(); it != dirs.end(); ++it )
      ld_path+=*it+":";

  setenv("LD_LIBRARY_PATH",ld_path,1);



  QStringList languages;
  GeneratorInfo *info;

//////// Load informatin for already registered libraries.
  for ( QStringList::Iterator it = libsKnown.begin(); it != libsKnown.end(); ++it )
   {
      languages = config->readListEntry(*it);
      for ( QStringList::Iterator lit = languages.begin(); lit != languages.end(); ++lit )
        {
          info = new GeneratorInfo;
          info -> language = *lit;  //language name
          info->library = *it ;
             ++lit;                 //go to next entry list (object)
          info->object = *lit;
          ldict.insert(info->language,info);
        }
  }



//////// Register new libraries
  if(!libsFound.isEmpty())
  {

  KMessageBox::information(this,i18n("UML Modeller found new Code Generation libraries.\nPlease select the languages you want to use"),
                                i18n("New Libraries found"));

    SelectLanguagesDlg d(this);
    d.offerLanguages(libsFound);
    if(d.exec())
      {
        GeneratorInfoDict tempdict;
        tempdict = d.selectedLanguages();
        tempdict.setAutoDelete(false);
        QDictIterator<GeneratorInfo> dictit( tempdict );
        for( dictit.toFirst(); dictit.current(); ++dictit )
          { //check if there is another language with the same name in other lib...
            info = dictit.current();
            int suffix=2;
            if(ldict.find(info->language))
              {
                while(ldict.find(info->language+":"+QString::number(suffix)))
                      suffix++;
                info->language+=":" + QString::number(suffix);
               }
            ldict.insert(info->language,info);
           }
       }

  } //endif (!libFound.isEmpty())

////////// Discard deleted libraries
  if(!libsMissing.isEmpty())
  {
    for ( QStringList::Iterator it = libsMissing.begin(); it != libsMissing.end(); ++it )
      str += *it + "\n";


  KMessageBox::information(this,i18n("The following libraries are registered in UML Modeller\
 for Code Generation but can not be found. They will be removed from the configuration\n")+ str);
        //maybe show the path were the libs were looked for?

    }


///////// Finaly done... just update the language selection menu.
updateLangSelectMenu();

}


void UMLApp::updateLangSelectMenu()
{
    langSelect->clear();
    langSelect->setCheckable(true);
    int id;
    bool foundActive = false;
    QDictIterator<GeneratorInfo> it( ldict );
    for(it.toFirst() ; it.current(); ++it )
      {
        id = langSelect->insertItem(it.current()->language,this,SLOT(setActiveLanguage(int)));
        if(activeLanguage == it.current()->language)
          {
            langSelect->setItemChecked(id,true);
            foundActive = true;
          }
        else
            langSelect->setItemChecked(id,false);
      }

  //if we could not find the active language, we try to fall back to C++
  if(!foundActive)
    {
      for(uint index=0; index <langSelect->count(); index++)
        if (langSelect->text(langSelect->idAt(index)) =="Cpp" )
          {
            langSelect->setItemChecked(langSelect->idAt(index),true);
            activeLanguage = langSelect->text(langSelect->idAt(index));
            break;
          }
    }
  //last try... if we dont have a activeLanguage and we have no Cpp installed we just
  //take the first language we find as "active"
  if(activeLanguage.isEmpty() && langSelect->count() > 0)
    {
      langSelect -> setItemChecked(langSelect->idAt(0),true);
      activeLanguage = langSelect -> text(langSelect->idAt(0));
    }

}



void UMLApp::keyPressEvent(QKeyEvent *e)
{
  switch(e->key())
   {
      case Qt::Key_Shift:
        //toolsbar->setOldTool();
        e->accept();
        break;

      default:
        e->ignore();
   }

}

void UMLApp::keyReleaseEvent(QKeyEvent *e)
{
  switch(e->key())
   {
      case Qt::Key_Shift:
        toolsbar->setOldTool();
        e->accept();
        break;
      case Qt::Key_Escape:
        toolsbar->setDefaultTool();
        e->accept();
        break;
      default:
        e->ignore();
   }

}














