// include files for Qt
#include <qpixmap.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qstring.h>
#include <qobjcoll.h>
#include <qdragobject.h>
#include <qpaintdevicemetrics.h>
#include <qfileinfo.h>
#include <qlist.h>
#include <qpixmap.h>

#include <math.h>
//kde include files
#include <ktempfile.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>
#include <kprinter.h>
#include <kcursor.h>
#include <kfiledialog.h>
#include <klineeditdlg.h>
#include <klocale.h>
#include <kdebug.h>

// application specific includes
#include "umlview.h"
#include "umldoc.h"
#include "uml.h"
#include "worktoolbar.h"
#include "docwindow.h"
#include "assocrules.h"
#include "umlviewcanvas.h"
#include "dialogs/classoptionspage.h"
#include "dialogs/umlviewdialog.h"

#include "clipboard/idchangelog.h"
#include "clipboard/cumldrag.h"

#include "conceptwidget.h"
#include "actorwidget.h"
#include "usecasewidget.h"
#include "notewidget.h"
#include "associationwidget.h"
#include "objectwidget.h"
#include "floatingtext.h"
#include "messagewidget.h"
#include "statewidget.h"
#include "activitywidget.h"
#include "seqlinewidget.h"

#include "mylistviewitemdatalist.h"
#include "associationwidgetdatalist.h"
#include "umlwidgetlist.h"

#include "umlwidgetdata.h"
#include "floatingtextdata.h"



UMLView::UMLView( QWidget * parent, UMLViewData * pData ) : QCanvasView(  parent,  "AnUMLView" )
{
	m_pData = pData;
	init();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::init()
{
  //Setup up booleans
  m_bPaste = false;
  m_bDrawRect = false;
  m_bActivated = false;
  m_bCreateObject = false;
  m_bDrawSelectedOnly = false;
  m_bPopupShowing = false;
  m_bStartedCut = false;
  //clear pointers
  m_pMoveAssoc = 0;
  m_pOnWidget = 0;
  m_pAssocLine = 0;
  m_pIDChangesLog = 0;
  m_pFirstSelectedWidget = 0;
  m_pMenu = 0;
	m_pSeqLine = 0;
  //setpu graphical items
  viewport() -> setBackgroundMode( NoBackground );
  setCanvas( new UMLViewCanvas( this ) );
  canvas() -> setUpdatePeriod( 20 );
  resizeContents(1300, 1300);
  canvas() -> resize( 1300, 1300 );
  setAcceptDrops(TRUE);
  viewport() -> setAcceptDrops(TRUE);
  setDragAutoScroll(false);
  viewport() -> setMouseTracking(false);
  m_SelectionRect.setAutoDelete( true );
  m_CurrentCursor = WorkToolBar::tbb_Arrow;
  //setup signals
  connect( this, SIGNAL(sigRemovePopupMenu()), this, SLOT(slotRemovePopupMenu() ) );
  connect( (UMLApp *)getDocument() -> parent() , SIGNAL( sigCutSuccessful() ),
		this, SLOT( slotCutSuccessful() ) );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
UMLView::~UMLView()
{
	if( m_pData )
		delete m_pData;
	if(m_pIDChangesLog)
	{
		delete    m_pIDChangesLog;
		m_pIDChangesLog = 0;
	}
  if( m_pAssocLine )
    delete m_pAssocLine;
  m_SelectionRect.clear();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
UMLDoc *UMLView::getDocument() const
{
	UMLApp *theApp=(UMLApp *) (parentWidget()->parentWidget());

  return theApp->getDocument();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::print(KPrinter *pPrinter, QPainter & pPainter)
{
	int height, width;
	int offsetX = 0, offsetY = 0, widthX = 0, heightY = 0;
	//get the size of the page
	QPaintDeviceMetrics metrics(pPrinter);
	QFontMetrics fm = fontMetrics();
	int fontHeight  = fm.lineSpacing();
	int marginX = pPrinter -> margins().width();
	int marginY = pPrinter -> margins().height();
	//double margin at botton of page as it doesn't print down there
	//on my printer, so play safe as default.
	if(pPrinter -> orientation() == KPrinter::Portrait)
	{
		width = metrics.width() - marginX * 2;
		height = metrics.height() - fontHeight - 4 - marginY * 3;
	}
	else
	{
		marginX *= 2;
		width = metrics.width() - marginX * 2;
		height = metrics.height() - fontHeight - 4 - marginY * 2;
	}
	//get the smallest rect holding the diagram
	QRect rect = getDiagramRect();
	QPixmap diagram(rect.width() - rect.x(), rect.height() - rect.y());
	getDiagram(diagram);
	//now draw to printer

	//loop until all of the picture is printed
	bool finishX = false, finishY = false;
	int page = 0, pageY = 1;
	do
	{
		if(!finishX)
			offsetX = page * width;
		else if(!finishY)
		{
			offsetY = pageY * height;
			offsetX = 0;
			pageY++;
		}
		if((width + offsetX) > diagram.width()  )
		{
			finishX = true;
			widthX = diagram.width() - offsetX;
		}
		else widthX = width;
		if(diagram.width() < width)
		{
			widthX = diagram.width();
			finishX = true;
			offsetX = 0;
		}
		if((height + offsetY) > diagram.height() )
		{
			finishY = true;
			heightY = diagram.height() - offsetY;
		}
		else heightY = height;
		if(diagram.height() < height)
		{
			finishY = true;
			heightY = diagram.height();
			offsetY = 0;


		}
		pPainter.drawPixmap(marginX , marginY , diagram, offsetX, offsetY, widthX, heightY);
		//draw foot note
		QString string = i18n("Diagram: ") + GetName() +i18n(" Page %1").arg(page + 1);
		QColor textColor(50, 50, 50);
		pPainter.setPen(textColor);
		pPainter.drawLine(marginX,  marginY + height + 2, marginX + width, marginY + height + 2);
		pPainter.drawText(marginX, marginY + height + 4, width, fontHeight, AlignLeft, string);
		//painter.drawText(marginX, height + 4, string);
		if(!finishX || !finishY)
		{
			pPrinter -> newPage();
			page++;
		}
	}while(!finishX || !finishY);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportMouseReleaseEvent(QMouseEvent *me)
{
  QPoint p = viewportToContents( me -> pos() );
  if(m_bDrawRect)
  {
    viewport() -> setMouseTracking( false );
    m_bDrawRect = false;
    m_SelectionRect.clear();
  }
  if( m_pAssocLine )
  {
    delete m_pAssocLine;
    m_pAssocLine = 0;
  }
  m_Pos.setX(p.x());
  m_Pos.setY(p.y());
  //make a mouse event with the co-ords change to contents
  //save everyone else doing it over and over
  QMouseEvent m( QEvent::MouseButtonRelease, p, me -> button(), me -> state() );
  if( allocateMouseReleaseEvent( &m ) )
    return;

  if( m_CurrentCursor == WorkToolBar::tbb_Arrow  || me -> state() != LeftButton )
  {
    viewport() -> setMouseTracking( false );
		if( me -> state() == RightButton )
			setMenu();
    return;
  }
  //create an initial activity widget
  if( m_CurrentCursor == WorkToolBar::tbb_Initial_Activity )
  {
    ActivityWidget * temp = new ActivityWidget( this , ActivityWidget::Initial );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //create end activity
  if( m_CurrentCursor == WorkToolBar::tbb_End_Activity )
  {
    ActivityWidget * temp = new ActivityWidget( this , ActivityWidget::End );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //create branch/merge activity
  if( m_CurrentCursor == WorkToolBar::tbb_Branch )
  {
    ActivityWidget * temp = new ActivityWidget( this , ActivityWidget::Branch );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
		//create fork/join activity
  if( m_CurrentCursor == WorkToolBar::tbb_Fork )
  {
    ActivityWidget * temp = new ActivityWidget( this , ActivityWidget::Fork );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //create a activity widget

  if( m_CurrentCursor == WorkToolBar::tbb_Activity )

  {
    bool ok = false;
    QString name = KLineEditDlg::getText( i18n("Enter Activity name."), i18n("Enter the name of the new Activity."), i18n("new activity"), &ok );
    if( ok )
    {
      ActivityWidget * temp = new ActivityWidget( this , ActivityWidget::Normal );
      temp -> SetName( name );
      temp -> setID( getDocument() -> getUniqueID() );//needed for associations
      temp ->setX( m_Pos.x() );
      temp -> setY ( m_Pos.y() );
      temp -> setVisible( true );
      temp->SetActivated();
   	 temp -> setFont( m_pData -> getFont() );
  	  temp -> setUseFillColor( m_pData -> getUseFillColor() );
    }
    return;
  }
  //create a state widget
  if( m_CurrentCursor == WorkToolBar::tbb_State )
  {
    bool ok = false;
    QString name = KLineEditDlg::getText( i18n("Enter State name."), i18n("Enter the name of the new State."), i18n("new state"), &ok );
    if( ok )
    {
      StateWidget * temp = new StateWidget( this , StateWidget::Normal );
      temp -> SetName( name );
      temp -> setID( getDocument() -> getUniqueID() );//needed for associations
      temp ->setX( m_Pos.x() );
      temp -> setY ( m_Pos.y() );
      temp -> setVisible( true );
      temp->SetActivated();
   	 temp -> setFont( m_pData -> getFont() );
  	  temp -> setUseFillColor( m_pData -> getUseFillColor() );
    }
    return;
  }
  //create an initial state widget
  if( m_CurrentCursor == WorkToolBar::tbb_Initial_State )
  {
    StateWidget * temp = new StateWidget( this , StateWidget::Initial );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //create end state
  if( m_CurrentCursor == WorkToolBar::tbb_End_State )
  {
    StateWidget * temp = new StateWidget( this , StateWidget::End );
    temp -> setID( getDocument() -> getUniqueID() );//needed for associations
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //Create a NoteBox widget
  if(m_CurrentCursor == WorkToolBar::tbb_Note)
  {
    //no need to register with document but get a id from it
    //id used when checking to delete object and assocs
    NoteWidget *temp= new NoteWidget(this, getDocument()->getUniqueID());
    temp ->setX( m_Pos.x() );
    temp -> setY ( m_Pos.y() );
    temp -> setVisible( true );
    temp->SetActivated();
    temp -> setFont( m_pData -> getFont() );
    temp -> setUseFillColor( m_pData -> getUseFillColor() );
    return;
  }
  //Create a Floating Text widget
  if(m_CurrentCursor == WorkToolBar::tbb_Text)
  {
    FloatingText * ft = new FloatingText(this, tr_Floating, "");
    ft -> changeTextDlg();
    //if no text entered delete
    if(!FloatingText::isTextValid(ft -> getText()))
      delete ft;
    else
    {
      ft -> setX( m_Pos.x() );
      ft -> setY( m_Pos.y() );
      ft -> setVisible( true );
      ft -> setID(getDocument() -> getUniqueID());
      ft->SetActivated();
      ft -> setFont( m_pData -> getFont() );
    	ft -> setUseFillColor( m_pData -> getUseFillColor() );
    }
    return;
  }
  //Create a Message on a Sequence diagram
  if(m_CurrentCursor == WorkToolBar::tbb_Seq_Message)
  {
    UMLWidget * w = onWidgetLine( m.pos() );
    if(w)
    {
      if(!m_pFirstSelectedWidget)
      {
        m_pFirstSelectedWidget = w;
        viewport() -> setMouseTracking( true );
        if( m_pAssocLine )
         delete m_pAssocLine;
        m_pAssocLine = new QCanvasLine( canvas() );
        m_pAssocLine -> setPoints( p.x(), p.y(), p.x(), p.y() );
        m_pAssocLine -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );
        m_pAssocLine -> setVisible( true );
        return;
      }
      else
      {
        FloatingText * t = new FloatingText(this, tr_Seq_Message, "");
        t -> setFont( m_pData -> getFont() );
        t -> setID(getDocument() -> getUniqueID());
        MessageWidget *temp = new MessageWidget(this, m_pFirstSelectedWidget, w, t, getDocument() -> getUniqueID(), me -> y());
        connect(this, SIGNAL(sigColorChanged(int)), temp, SLOT(slotColorChanged(int)));
        t->SetActivated();
        temp->SetActivated();
			m_pFirstSelectedWidget = 0;
        if( m_pAssocLine )
        {
          delete m_pAssocLine;
          m_pAssocLine = 0;
        }
        return;
      }
    }//end if w
    else
      return;
  }//end if

  else if( m_CurrentCursor < WorkToolBar::tbb_Actor || m_CurrentCursor > WorkToolBar::tbb_State )
  {
    m_pFirstSelectedWidget = 0;
    return;
  }

  //if we are creating an object, we really create a class
  if(m_CurrentCursor == WorkToolBar::tbb_Object)
    m_CurrentCursor = WorkToolBar::tbb_Concept;
  m_bCreateObject = true;
  getDocument()->createUMLObject(convert_TBB_OT(m_CurrentCursor));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::slotToolBarChanged(int c)
{
	m_CurrentCursor = (WorkToolBar::ToolBar_Buttons)c;
	m_pFirstSelectedWidget = 0;
	m_bPaste = false;
  m_bDrawRect = false;
	if( m_pAssocLine )
	{
		delete m_pAssocLine;
		m_pAssocLine = 0;
	}
	//setCursor(KCursor::arrowCursor());
  viewport() -> setMouseTracking( false );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::showEvent(QShowEvent */*se*/)
{
	UMLApp *theApp=(UMLApp *) (parentWidget()->parentWidget());
	WorkToolBar *tb = theApp -> getWorkToolBar();
	connect(tb,SIGNAL(sigButtonChanged(int)), this, SLOT(slotToolBarChanged(int)));
	connect(this,SIGNAL(sigResetToolBar()), tb, SLOT(slotResetToolBar()));
	connect(getDocument(), SIGNAL(sigObjectCreated(UMLObject *)), this, SLOT(slotObjectCreated(UMLObject *)));
	resetToolbar();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::hideEvent(QHideEvent */*he*/)
{
	UMLApp *theApp=(UMLApp *) (parentWidget()->parentWidget());
	WorkToolBar *tb = theApp -> getWorkToolBar();
	disconnect(tb,SIGNAL(sigButtonChanged(int)), this, SLOT(slotToolBarChanged(int)));
	disconnect(this,SIGNAL(sigResetToolBar()), tb, SLOT(slotResetToolBar()));
	disconnect(getDocument(), SIGNAL(sigObjectCreated(UMLObject *)), this, SLOT(slotObjectCreated(UMLObject *)));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
MyListView * UMLView::getListView()
{
  return getDocument()->listView;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::slotObjectCreated(UMLObject * o)
{
	m_bPaste = false;
	//setCursor(KCursor::arrowCursor()); why???
	int type  = o->getBaseType();

	//check to see if we want the message
	//may be wanted by someone else e.g. list view
	if(m_bCreateObject)
	{
      UMLWidget *temp = 0;
		if(type == ot_Actor)
		{
			ActorWidget * a = new ActorWidget(this, o);
			temp = (UMLWidget*)a;
		}
		else if(type == ot_UseCase)
		{
			UseCaseWidget * u = new UseCaseWidget(this, o);
			temp = (UMLWidget*)u;
		}
		else if(type == ot_Concept)
		{
			//see if we really want an object widget or concept widget
			if(m_pData -> m_Type == dt_Class)
			{
				ConceptWidget * c = new ConceptWidget(this, o);
				temp = (UMLWidget*)c;
			}
			else
			{
				ObjectWidget * ow = new ObjectWidget(this, o, m_pData -> getUniqueID() );
				temp = (UMLWidget *)ow;
			}
		} else
			kdDebug() << "ERROR: trying to create an invalid widget" << endl;
		int y=m_Pos.y();
		if(temp -> getBaseType() == wt_Object && this -> m_pData -> m_Type == dt_Sequence)
		{
			y = 80 - temp -> height();
		}
		temp -> setX( m_Pos.x() );
		temp -> setY( y );
		temp -> moveEvent( 0 );//needed for ObjectWidget to set seq. line properly
		temp -> setVisible( true );
		temp -> SetActivated();
		temp -> setFont( m_pData -> getFont() );
		temp -> setUseFillColor( m_pData -> getUseFillColor() );
		m_bCreateObject = false;
		switch( type )
		{
			case ot_Actor:
			case ot_UseCase:
			case ot_Concept:
				createAutoAssociations( temp );
			break;
		}
	}
}




////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::slotObjectRemoved(UMLObject * o)
{
	m_bPaste = false;
	//setCursor(KCursor::arrowCursor()); why??
	int id = o -> getID();
	QObjectList * l = queryList( "UMLWidget");
  	QObjectListIt it( *l );
	UMLWidget *obj;

  	while ((obj=(UMLWidget*)it.current()) != 0 )
	{
  		++it;
    	if(obj -> getID() == id)

   		{
    		removeWidget(obj);
    		delete l;

    		//keep calling self while a widget representing o exits as there may be others
		  	slotObjectRemoved(o);
    		return;
	    }
  	}
 	delete l;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportDragEnterEvent(QDragEnterEvent *e)
{
	CMyListViewItemDataList list;
	bool status = CUMLDrag::decode(e, list);
	if(!status)
	{
		return;
	}
	CMyListViewItemDataListIt it(list);

	CMyListViewItemData* data = it.current();

	ListView_Type lvtype = data -> getType();

	UMLObject* temp = 0;
   //if dragging diagram - set false
  switch( lvtype )
  {
    case lvt_UseCase_Diagram:
    case lvt_Sequence_Diagram:
    case lvt_Class_Diagram:
    case lvt_Collaboration_Diagram:
    case lvt_State_Diagram:
		case lvt_Activity_Diagram:
      status = false;
    break;
		default:
		break;
  }
  //can't drag anything onto state/activity diagrams
  if( m_pData -> m_Type == dt_State || m_pData -> m_Type == dt_Activity)
    status = false;
   //make sure can find UMLObject
	if(status)
		if( !(temp = getDocument()->findUMLObject( data -> getID() ) ) )
		{
			kdDebug() << "object not found" << endl;
			status = false;
		}
	//make sure dragging item onto correct diagram
	// concept - class,seq,coll diagram
	// actor,usecase - usecase diagram
	if(status)
	{
		UMLObject_Type ot = temp -> getBaseType();
		if(m_pData -> m_Type == dt_UseCase)
			if(ot < ot_Actor || ot > ot_UseCase)
			{
				status = false;
			}
		if(m_pData -> m_Type == dt_Sequence || m_pData -> m_Type == dt_Class || m_pData -> m_Type == dt_Collaboration)
			if(ot != ot_Concept)
				status = false;
		if(m_pData -> m_Type == dt_UseCase || m_pData -> m_Type == dt_Class)
		{

			if( widgetOnDiagram( data -> getID() ) )
				status = false;
		}
	}
	e->accept(status);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportDropEvent(QDropEvent *e)
{
	CMyListViewItemDataList list;
	bool status = CUMLDrag::decode(e, list);
	if(!status)
	{
		return;

	}

	CMyListViewItemDataListIt it(list);
	CMyListViewItemData* data = it.current();
	ListView_Type lvtype = data -> getType();
	UMLObject* o = 0;
	if(lvtype >= lvt_UseCase_Diagram && lvtype <= lvt_Sequence_Diagram)
   		status = false;
	if(status)
		if( !( o = getDocument()->findUMLObject( data->getID() ) ) )
		{
			kdDebug() << "object not found" << endl;
			status = false;
		}
	if(status)
	{
		m_bCreateObject = true;
		m_Pos = e->pos();

		slotObjectCreated(o);
		getDocument() -> setModified(true);
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////
UMLWidget * UMLView::onWidgetLine( QPoint point )
{
	SeqLineWidget * pLine = 0;
	for( pLine = m_SeqLineList.first(); pLine; pLine = m_SeqLineList.next() )
		if( pLine -> onWidget( point ) )
			return pLine -> getObjectWidget();
	return 0;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::checkMessages(UMLWidget * w)
{
	if(m_pData -> m_Type != dt_Sequence)
		return;


	QObjectList * l = queryList( "UMLWidget");
   QObjectListIt it( *l );
   MessageWidget *obj;
   while ( (obj=(MessageWidget*)it.current()) != 0 )
	{
   		++it;
   		if(obj -> getBaseType() == wt_Message)
   		{
   			if(obj -> contains(w))
   			{
   				//make sure message doesn't have any associations
   				RemoveAssociations(obj);
              obj -> cleanup();
   				//make sure not in selected list
   				m_SelectedList.remove(obj);

   				delete obj;
   				delete l;
   				checkMessages(w);
    				return;
   			}
   		}
   }
   delete l;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
bool UMLView::widgetOnDiagram(int id)
{
	QObjectList * l = queryList( "UMLWidget");
   QObjectListIt it( *l );
   UMLWidget *obj;

   while ( (obj=(UMLWidget*)it.current()) != 0 )
	{
   	++it;
   	if(id == obj -> getID())
   	{
   		delete l;
   		return true;
   	}
   }
   delete l;
   return false;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportMouseMoveEvent(QMouseEvent *me)
{
  QPoint p = viewportToContents( me -> pos() );
	m_LineToPos =p;
  if( m_pFirstSelectedWidget )
  {
    if( m_pAssocLine )
    {
      QPoint sp = m_pAssocLine -> startPoint();
      m_pAssocLine -> setPoints( sp.x(), sp.y(), p.x(), p.y() );
    }
    return;
  }
  if(m_bDrawRect)
  {

    if( m_SelectionRect.count() == 4)
    {

      QCanvasLine * line = m_SelectionRect.at( 0 );
      line -> setPoints( m_Pos.x(), m_Pos.y(), p.x(), m_Pos.y() );

      line = m_SelectionRect.at( 1 );
      line -> setPoints( p.x(), m_Pos.y(), p.x(), p.y() );

      line = m_SelectionRect.at( 2 );
      line -> setPoints( p.x(), p.y(), m_Pos.x(), p.y() );

      line = m_SelectionRect.at( 3 );
      line -> setPoints( m_Pos.x(), p.y(), m_Pos.x(), m_Pos.y() );

      selectWidgets();
    }
  }
  //make a mouse event with the co-ords change to contents


  //save everyone else doing it over and over
  QMouseEvent m( QEvent::MouseMove, p, me -> button(), me -> state() );
  if( allocateMouseMoveEvent( &m ) )
    return;
}

bool UMLView::serialize(QDataStream *s, bool archive)
{
	if( archive )
		SynchronizeData();
	return m_pData -> serialize( s, archive);
}

long UMLView::getClipSizeOf()
{
	SynchronizeData();
	return m_pData -> getClipSizeOf();
}

UMLWidget * UMLView::findWidget( int id )
{
	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );
   UMLWidget *obj;
	while ( (obj=(UMLWidget*)it.current()) != 0 )
	{
   	++it;
		if( obj -> getBaseType() == wt_Object )
		{
			if( dynamic_cast<ObjectWidget *>( obj ) -> getLocalID() == id )
			{

				delete l;
				return obj;
			}
		}
		else if( obj -> getID() == id )
		{
			delete l;
			return obj;
		}
	}
	delete l;
	return 0;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::removeWidget(UMLWidget * o)
{
	if(!o)
   	return;
	RemoveAssociations(o);

	UMLWidget_Type t = o->getBaseType();
	if(m_pData -> m_Type == dt_Sequence && t == wt_Object)
   	checkMessages(o);
	if( m_pOnWidget == o )
	{
	   getDocument() -> getDocWindow() -> updateDocumentation( true );
		m_pOnWidget = 0;
	}
   o -> cleanup();
	m_SelectedList.remove(o);
  disconnect( this, SIGNAL( sigRemovePopupMenu() ), o, SLOT( slotRemovePopupMenu() ) );
  disconnect( this, SIGNAL( sigClearAllSelected() ), o, SLOT( slotClearAllSelected() ) );
  disconnect( this, SIGNAL(sigColorChanged(int)), o, SLOT(slotColorChanged(int)));
  disconnect( getDocument(), SIGNAL(sigWidgetUpdated(UMLObject *)), o, SLOT(slotChangeWidget(UMLObject *)));
	o -> deleteLater();
  delete o;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::setBGColor(QColor color)
{
	m_pData -> setFillColor( color );
	emit sigColorChanged(m_pData -> m_nID);
	canvas() -> setAllChanged();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::setLineColor(QColor color)
{
	m_pData -> setLineColor(color );
	emit sigColorChanged(m_pData -> m_nID);
  emit sigLineColorChanged( color );
	canvas() -> setAllChanged();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportMouseDoubleClickEvent(QMouseEvent * me)
{
  //make a mouse event with the co-ords change to contents
  //save everyone else doing it over and over
  QMouseEvent m( QEvent::MouseButtonDblClick, viewportToContents( me -> pos() ), me -> button(), me -> state() );
  if( allocateMouseDoubleClickEvent( &m ) )
    return;
  clearSelected();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
QRect UMLView::getDiagramRect()
{
	int startx, starty, endx, endy;
	startx = starty = 0;
	endx = 0;
	endy = 0;
	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );
 	UMLWidget *obj;
 	while ( (obj=(UMLWidget*)it.current()) != 0 )
 	{
 		int objEndX = (int)obj -> x() + obj -> width();
 		int objEndY = (int)obj -> y() + obj -> height();
 		if(endx == 0)
 			endx = objEndX;
		else if(endx < objEndX)
 			endx = objEndX;
 		if(endy == 0)
 			endy = objEndY;
 		else if(endy < objEndY)
 			endy = objEndY;
 		++it;
 	}
 	//if seq. diagram, make sure print all of the lines
 	if(m_pData -> m_Type == dt_Sequence )
	{
		SeqLineWidget * pLine = 0;
		for( pLine = m_SeqLineList.first(); pLine; pLine = m_SeqLineList.next() )
		{
			int y = pLine -> getObjectWidget() -> getEndLineY();
			endy = endy < y?y:endy;
		}

	}
 	delete l;
 	return QRect(startx, starty,  endx, endy );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::setSelected(UMLWidget * w, QMouseEvent * /*me*/)
{
	//only add if wasn't in list

	if(!m_SelectedList.remove(w))
		m_SelectedList.append(w);
	int count = m_SelectedList.count();
	//only call once - if we select more, no need to keep clearing  window

	// if count == 1, widget will update the doc window with their data when selected
	if( count == 2 )
		updateDocumentation( true );//clear doc window
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::clearSelected()
{
  m_SelectedList.clear();
  emit sigClearAllSelected();
  //getDocument() -> enableCutCopy(false);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::moveSelected(UMLWidget * w, int x, int y)
{
	QMouseEvent me(QMouseEvent::MouseMove, QPoint(x,y), LeftButton, ShiftButton);
	UMLWidget * temp = 0;
	//loop through list and move all widgets
	//don't move the widget that started call
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
		if(temp != w)
			temp -> mouseMoveEvent(&me);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::selectionUseFillColor(bool useFC)
{
	UMLWidget * temp = 0;
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
		temp -> setUseFillColor(useFC);
}

void UMLView::selectionSetFont( QFont font )

{
	UMLWidget * temp = 0;
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
		temp -> setFont( font );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::deleteSelection()
{
	/*
		Don't delete text widget that are connect to associations as these will be cleaned up
		by the associations.
	*/
	UMLWidget * temp = 0;
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
	{
		if( temp -> getBaseType() == wt_Text && ((FloatingText *)temp) -> getRole() != tr_Floating )
			temp -> hide();
		else
		{
			removeWidget(temp);
			deleteSelection();
			return;
		}
	}
	//now delete any selected associations
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;
		if( assocwidget-> getSelected() )
			RemoveAssoc(assocwidget);
	}
	//make sure list empty - it should be anyway, just a check.
	m_SelectedList.clear();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::viewportMousePressEvent(QMouseEvent * me)
{
	getDocument() -> setModified(true);
  int x, y;
  if( m_pAssocLine )
  {
    delete m_pAssocLine;
    m_pAssocLine = 0;
  }
  viewport() -> setMouseTracking( true );
  emit sigRemovePopupMenu();
  //make a mouse event with the co-ords change to contents

  //save everyone else doing it over and over
  QMouseEvent m( QEvent::MouseButtonPress, viewportToContents( me -> pos() ), me -> button(), me -> state() );
  if( allocateMousePressEvent( &m ) )
    return;

  x = m.x();
  y = m.y();
  m_Pos.setX( x );
  m_Pos.setY( y );
  m_LineToPos.setX( x );
  m_LineToPos.setY( y );
  if(m_bPaste)
  {
    m_bPaste = false;
    //Execute m_bPaste action when pasting widget from another diagram
    //This needs to be changed to use CUMLClipboard
    //clipboard -> m_bPaste(this, pos);
    //setCursor(KCursor::arrowCursor()); why??
  }
	clearSelected();

  if(m_CurrentCursor == WorkToolBar::tbb_Arrow)

  {
    QCanvasLine * line = new QCanvasLine( canvas() );
    line -> setPoints( x, y, x, y);
    line -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );
    line -> setVisible( true );
    m_SelectionRect.append( line );//four lines needed for rect.

    line = new QCanvasLine( canvas() );
    line -> setPoints( x, y, x, y);

    line -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );
    line -> setVisible( true );

    m_SelectionRect.append( line );


    line = new QCanvasLine( canvas() );
    line -> setPoints( x, y, x, y);
    line -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );
    line -> setVisible( true );
    m_SelectionRect.append( line );


    line = new QCanvasLine( canvas() );
    line -> setPoints( x, y, x, y);
    line -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );
    line -> setVisible( true );
    m_SelectionRect.append( line );
    m_bDrawRect = true;
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::selectWidgets()
{
	clearSelected();
	UMLWidget * temp;
	int x,y,w,h;
	int px = 0, qx = 0, py = 0, qy = 0;

	QRect rect;
	px = m_Pos.x();
	py = m_Pos.y();
	qx = m_LineToPos.x();
	qy = m_LineToPos.y();
	if(px <= qx)
	{
		rect.setLeft(px);
		rect.setRight(qx);
	}
	else
	{
		rect.setLeft(qx);
		rect.setRight(px);
	}
	if(py <= qy)
	{
		rect.setTop(py);
		rect.setBottom(qy);

	}
	else
	{
		rect.setTop(qy);
		rect.setBottom(py);
	}
	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );
  	while ( (temp=(UMLWidget*)it.current()) != 0 )
	{
		x = (int)temp -> x();
		y =  (int)temp -> y();
		w = temp -> width();
		h = temp -> height();
		QRect rect2(x, y, w, h);
 		++it;
		//see if any part of widget is in the rectangle
		//made of points pos and m_LineToPos
		if(rect.intersects(rect2))
		{
			//if it is text that is part of an association then select the association
 			//and the objects that are connected to it.
 			if(temp -> getBaseType() == wt_Text && ((FloatingText *)temp) -> getRole() != tr_Floating )
 			{


 				int t = ((FloatingText *)temp) -> getRole();
 				if( t == tr_Seq_Message )
 				{
 					MessageWidget * mw = (MessageWidget *)((FloatingText *)temp) -> getMessage();
 					mw -> setSelected(true);
 					m_SelectedList.remove(mw);//make sure not in there
 					m_SelectedList.append(mw);
 					UMLWidget * ow = mw -> getWidgetA();
 					ow -> setSelected(true);
 					m_SelectedList.remove(ow);//make sure not in there
 					m_SelectedList.append(ow);
					ow = mw -> getWidgetB();
 					ow -> setSelected(true);
 					m_SelectedList.remove(ow);//make sure not in there
 					m_SelectedList.append(ow);
 				}
 				else
 				{
 					CAssociationWidget * a = dynamic_cast<CAssociationWidget *>( (dynamic_cast<FloatingText *>(temp)) -> getAssoc() );

 					a -> setSelected(true);
					//select the two widgets
					UMLWidget * uw = a->getWidgetA();
 					uw -> setSelected(true);
 					m_SelectedList.remove(uw);//make sure not in there
 					m_SelectedList.append(uw);
 					uw = a->getWidgetB();
 					uw -> setSelected(true);
 					m_SelectedList.remove(uw);//make sure not in there
 					m_SelectedList.append(uw);
					//select all the text
					FloatingText * text = a -> getMultiAWidget();
					if( text )
					{
						text  -> setSelected( true );
 						m_SelectedList.remove( text );
 						m_SelectedList.append( text );
					}
					text = a -> getMultiBWidget();
					if( text )
					{
						text  -> setSelected( true );
 						m_SelectedList.remove( text );
 						m_SelectedList.append( text );
					}
					text = a -> getRoleWidget();
					if( text )
					{
						text  -> setSelected( true );
 						m_SelectedList.remove( text );
 						m_SelectedList.append( text );
					}
 				}//end else
 			}//end if text
 			else if(temp -> getBaseType() == wt_Message)
 			{
 				UMLWidget * ow = ((MessageWidget *)temp) -> getWidgetA();
 				ow -> setSelected(true);
 				m_SelectedList.remove(ow);//make sure not in there
 				m_SelectedList.append(ow);
 				ow = ((MessageWidget *)temp) -> getWidgetB();
 				ow -> setSelected(true);
 				m_SelectedList.remove(ow);//make sure not in there
 				m_SelectedList.append(ow);
 			}
 			 if(temp -> isVisible())
 			 {
 				m_SelectedList.remove(temp);//just make sure hasn't already been added automatically
				m_SelectedList.append(temp);
				temp -> setSelected(true);
			}
		}
	}
	delete l;
	SelectAssociations( true );

	//now do the same for the messagewidgets
	QObjectList * lw = queryList( "UMLWidget");
	QObjectListIt itw( *lw );
	temp = 0;
  	while ( (temp=(UMLWidget*)itw.current()) != 0 )
	{
		++itw;
		if( temp -> getBaseType() == wt_Message )
		{
			MessageWidget * w = (MessageWidget *)temp;
			if( w -> getWidgetA() -> getSelected() &&
				w -> getWidgetB() -> getSelected() )
			{
				w -> setSelected( true );
				m_SelectedList.remove( w );
				m_SelectedList.append( w );
			}//end if
		}
	}//end while
	delete lw;
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void  UMLView::getDiagram(QPixmap & diagram)
{
  //unselect all before grab
  UMLWidget * temp = 0;
  for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
    temp -> setSelected(false);
  if( m_pMoveAssoc )
    m_pMoveAssoc -> setSelected( false );


  QPainter painter;
  painter.begin(&diagram);
  canvas() -> drawArea( QRect( 0, 0, diagram.width(), diagram.height() ), &painter );
  painter.end();

  //select again
  for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
    temp -> setSelected( true );
  if( m_pMoveAssoc )
    m_pMoveAssoc -> setSelected( true );

}

void UMLView::updateNoteWidgets()
{
	CAssociationWidget * a = 0;

	CAssociationWidgetListIt assoc_it(m_Associations);
	while((a = assoc_it.current()))
	{
		++assoc_it;
		if(a->getAssocType() == at_Anchor)
		{
			UMLWidget *wa = 0,*wb = 0, *destination= 0, *source = 0;

			wa = a->getWidgetA();

  			wb = a-> getWidgetB();
			bool copyText = false;
			if(wa->getBaseType() == wt_Note)
			{
        		kdDebug() << "A note" << endl;
				source = wb;
				destination = wa;
				copyText = true;
			}

			else if(wb->getBaseType() == wt_Note)
			{
      	  		kdDebug() << "B note" << endl;
				source = wa;
				destination = wb;
				copyText = true;

			}

			if(copyText == true && ((NoteWidget*)destination)->getLinkState() == true
													&& source->getUMLObject()->getDoc() != NULL)
			{
				((NoteWidget*)destination)->setDoc(source->getUMLObject()->getDoc());

			}
		}
	}
}

void UMLView::exportImage(const KURL& _url) {
    KURL url = _url;
    if(!url.isEmpty()) {
        QString s;
        KTempFile tmpfile;

        if (url.isLocalFile()) {
            QString file = url.path(-1);
            QFileInfo info(file);
            QString ext = info.extension(false);
            if(ext != "png") {
                url.setFileName(url.fileName() + ".png");
            }
            s = url.path();
        }
				else {
            s = tmpfile.name();

        }

        QRect rect = getDiagramRect();
        if (rect.isEmpty()) {
            KMessageBox::sorry(0, i18n("Can not save an empty diagram"),

                               i18n("Diagram save error!"));
        } else {
            QPixmap diagram(rect.width() - rect.x(), rect.height() - rect.y());

            getDiagram(diagram);
            diagram.save(s, "PNG");


            if (!url.isLocalFile()) {
                if(!KIO::NetAccess::upload(tmpfile.name(), url)) {


                    KMessageBox::error(0,

                                       i18n("There was a problem saving file: %1").arg(url.path()),



                  						                     i18n("Save error!"));
              }
              tmpfile.unlink();
           } //!isLocalFile
        } //rect.isEmpty()

    } //!url.isEmpty()
}//exportImage()

void UMLView::slotActivate()
{
    getDocument()->changeCurrentView(getID());
}

CUMLObjectList* UMLView::getUMLObjects()
{
	int type;
	CUMLObjectList* list = new CUMLObjectList;
	list->setAutoDelete(FALSE);

	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );

	UMLWidget *obj;
   while ( (obj=(UMLWidget*)it.current()) != 0 )
   {
      ++it;
 		type = obj -> getBaseType();
		switch( type ) //use switch for easy future exspansion
 		{
			case wt_Actor:
			case wt_Class:
			case wt_UseCase:
			case wt_Object:
 				list->append( obj->getUMLObject() );
			break;
 		}
 	}
  delete l;
	return list;
}

bool UMLView::Activate()
{
	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );
	UMLWidget *obj;

	//Activate Regular widgets then activate  messages
  while ( (obj=(UMLWidget*)it.current()) != 0 )
  {
  		++it;
		//If this UMLWidget is already activated or is a MessageWidget then skip it

 		if(obj->IsActivated() || obj->getBaseType() == wt_Message)
 		{
 			continue;

		}
		if(!obj->Activate())
		{
			continue;
		}
  		obj -> setVisible( true );
	}//end while

	QObjectListIt it2( *l );
	//Activate Message widgets
  while ( (obj=(UMLWidget*)it2.current()) != 0 )
  {
  		++it2;
		//If this UMLWidget is already activated or isnt a MessageWidget then skip it
 		if(obj->IsActivated() || obj->getBaseType() != wt_Message)
 			continue;

		if(!obj->Activate(getDocument()->getChangeLog()))
		{
			kdDebug() << "Couldn't activate message widget" << endl;
			continue;
		}
  		obj -> setVisible( true );

	}//end while
	delete l;

	//Activate All associationswidgets
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget *assocwidget;
  //first get total count
  while((assocwidget = assoc_it.current()))
  {
		++assoc_it;
    if( !assocwidget -> IsActivated() )
    {
      assocwidget->Activate();
      if( m_PastePoint.x() != 0 )
      {
        int x = m_PastePoint.x() - m_Pos.x();
        int y = m_PastePoint.y() - m_Pos.y();
        assocwidget -> moveEntireAssoc( x, y );
      }
    }//end if
	}//end while
	return true;
}


bool UMLView::GetSelectedWidgets(CUMLWidgetList&WidgetList)

{
	UMLWidget * temp = 0;
	int type;
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
	{
		type = temp->getBaseType();
		switch( type )
		{
			case wt_Actor:
			case wt_UseCase:
			case wt_Class:
			case wt_Object:
			case wt_Note:
			case wt_Message:
			case wt_State:
			case wt_Activity:
         	WidgetList.append(temp);
			break;

			case wt_Text:
				if( ((FloatingText *) temp ) -> getRole() == tr_Floating )
					WidgetList.append(temp);
			break;
		}//end switch
	}//end for
	return true;
}

bool UMLView::GetSelectedWidgetDatas(CUMLWidgetDataList&WidgetDataList)
{
	UMLWidget * temp = 0;
	UMLWidget_Type type;
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
	{
		type = temp->getBaseType();

		switch( type )
		{
			case wt_Actor:
			case wt_UseCase:
			case wt_Class:
			case wt_Object:
			case wt_Note:
			case wt_State:
			case wt_Activity:
				WidgetDataList.append(temp->getData());
			break;

			case wt_Text:
				if( ((FloatingText *) temp ) -> getRole() == tr_Floating )
					WidgetDataList.append(temp->getData());
			break;
			default:
			break;
		}//end switch
	}
	//Insert WidgetData after regular widgets because when they are created
	//in a m_bPaste operation they requiere the widgets it associates to be created first
	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
	{
		type = temp->getBaseType();
		if(type == wt_Message)
			WidgetDataList.append(temp->getData());
	}
	return true;
}

bool UMLView::GetSelectedAssocDatas(CAssociationWidgetDataList & AssociationWidgetDataList)
{
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;
		if( assocwidget -> getSelected() )
			AssociationWidgetDataList.append(assocwidget->getData());
	}
	return true;
}

bool UMLView::CreateWidget(CUMLWidgetData* WidgetData)
{
	if( !WidgetData )
		return false;
	UMLWidget * widget = 0;
  UMLObject * object = getDocument() -> findUMLObject( WidgetData -> getId() );
	switch( WidgetData->getType() )
	{
		case wt_Actor:
			widget = new ActorWidget(this , object, WidgetData);
		break;

		case wt_UseCase:
			widget = new UseCaseWidget(this, object, WidgetData);
		break;

		case wt_Class:
			widget = new ConceptWidget(this, object, WidgetData);
		break;

		case wt_Text:
			widget = new FloatingText(this, WidgetData);
		break;

		case wt_Note:
			widget = new NoteWidget(this, WidgetData);
		break;

		case wt_Object:
			widget = new ObjectWidget(this, object, WidgetData);
		break;

		case wt_Message:
			widget = new MessageWidget(this, WidgetData);
		break;

		case wt_State:
			widget = new StateWidget( this, WidgetData );
		break;

		case wt_Activity:
			widget = new ActivityWidget( this, WidgetData );
		break;

		default:
			return false;
		break;
	}
	widget -> setX( WidgetData -> getX() );
	widget -> setY( WidgetData -> getY() );
	return true;
}


bool UMLView::addWidget( CUMLWidgetData * pWidgetData )
{
	if( !pWidgetData )
		return false;

	CIDChangeLog * log = getDocument() -> getChangeLog();
	if( !log || !m_pIDChangesLog)
	{

		kdDebug()<<"A log is not open"<<endl;
		return false;

	}
   if( pWidgetData -> getX() < m_Pos.x() )
		m_Pos.setX( pWidgetData -> getX() );
	if( pWidgetData -> getY() < m_Pos.y() )
		m_Pos.setY( pWidgetData -> getY() );
	//see if we need a new id to match object
	int newID = -1;
	CObjectWidgetData* pObjectData = 0;
	CMessageWidgetData * pMessageData = 0;
	UMLObject * pObject = 0;
	int waID = -1, wbID = -1, newWAID = -1, newWBID = -1, newTextID = - 1;
	switch( pWidgetData -> getType() )
	{

		case wt_Class:
		case wt_Actor:
		case wt_UseCase:
			newID = log->FindNewID( pWidgetData -> getId() );
			if( newID == -1 )
				return false;
			pWidgetData -> setId( newID );
			pObject = getDocument() -> findUMLObject( newID );
         if( !pObject )
			{
				kdDebug() << "addWidget::Can't find UMLObject" << endl;
				return false;
			}
      //make sure it doesn't already exist.
      if( findWidget( newID ) )
        return true;//don't stop paste just because widget found.
		break;

		case wt_Message:
			newID = getDocument() -> AssignNewID( pWidgetData -> getId() );

			pWidgetData -> setId( newID );
			pMessageData = dynamic_cast<CMessageWidgetData *>( pWidgetData );
			waID = pMessageData -> getWidgetAID();
			wbID = pMessageData -> getWidgetBID();
			newWAID = m_pIDChangesLog ->FindNewID( waID );
			newWBID = m_pIDChangesLog ->FindNewID( wbID );

			if( newWAID == -1 || newWBID == -1 )
			{
				kdDebug()<<"Error with ids : "<<newWAID<<" "<<newWBID<<endl;
				return false;
			}
			pMessageData -> setWidgetAID( newWAID );
			pMessageData -> setWidgetBID( newWBID );
			if( pMessageData -> getTextID() != - 1 )
			{
				newTextID = getDocument() -> AssignNewID( pMessageData -> getTextID() );
				pMessageData -> setTextID( newTextID );
			}
			else
				pMessageData -> setTextID( getDocument() -> getUniqueID() );

		break;

		case wt_Object:
			if((pObjectData = dynamic_cast<CObjectWidgetData*>(pWidgetData)))
			{
				newID = log->FindNewID( pWidgetData -> getId() );
				if( newID == -1 )
					return false;
           	pObjectData -> setId( newID );
				int nNewLocalID = m_pData -> getUniqueID();
				int nOldLocalID = pObjectData -> getLocalID();
				m_pIDChangesLog -> AddIDChange( nOldLocalID, nNewLocalID );
				pObjectData -> setLocalID( nNewLocalID );
				pObject = getDocument() -> findUMLObject( newID );
         	if( !pObject )
				{
					kdDebug() << "addWidget::Can't find UMLObject" << endl;
					return false;
				}
			}
		break;

		case wt_Note:
			newID = getDocument() -> AssignNewID( pWidgetData  -> getId() );
			pWidgetData -> setId( newID );
		break;

		case wt_Text:
			newID = getDocument() -> AssignNewID( pWidgetData  -> getId() );
			pWidgetData -> setId( newID );
		break;

		case wt_State:
		case wt_Activity:
			newID = getDocument() -> AssignNewID( pWidgetData  -> getId() );
			pWidgetData -> setId( newID );
		break;

		default:
			kdDebug() << "Trying to add an invalid widget type" << endl;
			return false;
		break;
	}
	return CreateWidget( pWidgetData );
}

bool UMLView::addAssociation( CAssociationWidgetData* AssocData )
{
	if(!AssocData)
		return false;

	CIDChangeLog * log = getDocument() -> getChangeLog();

	if( !log )
		return false;

	int ida = -1, idb = -1;
	Association_Type type = AssocData -> getAssocType();
	CIDChangeLog* localLog = GetLocalIDChangeLog();
	if( getType() == dt_Collaboration || getType() == dt_Sequence )
	{
		//check local log first
		ida = localLog->FindNewID( AssocData->getWidgetAID() );

		idb = localLog->FindNewID( AssocData->getWidgetBID() );
		//if either is still not found and assoc type is anchor
		//we are probably linking to a notewidet - else an error
		if( ida == -1 && type == at_Anchor )
			ida = log->FindNewID(AssocData->getWidgetAID());
		if( idb == -1 && type == at_Anchor )
			idb = log->FindNewID(AssocData->getWidgetBID());
	}
	else
	{
		ida = log->FindNewID(AssocData->getWidgetAID());
		idb = log->FindNewID(AssocData->getWidgetBID());
	}
	if(ida == -1 || idb == -1)
  	{
  		return false;
  	}
	AssocData->setWidgetAID(ida);
	AssocData->setWidgetBID(idb);
	UMLWidget * m_pWidgetA = findWidget(AssocData->getWidgetAID());
 	UMLWidget * m_pWidgetB = findWidget(AssocData->getWidgetBID());
   //make sure valid widget ids

	if(!m_pWidgetA || !m_pWidgetB)
 		return false;

	//make sure valid
	if( !AssocRules::allowAssociation( AssocData -> getAssocType(), m_pWidgetA, m_pWidgetB ) )
	{
		return true;//in a paste we still need to be true
	}
	//make sure there isn't already the same assoc
   CAssociationWidgetListIt assoc_it(m_Associations);

	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;

		CAssociationWidgetData * pData = assocwidget -> getData();
		if( *AssocData == *pData )
			return true;

	}
	return CreateAssoc( AssocData );
}


/** Creates an association from a CAssociationWidgetData from the clipboard,
Activate must be called to make this assoc visible */
bool UMLView::CreateAssoc(CAssociationWidgetData* AssocData)
{
   if(!AssocData)
	{
		return false;

	}
	CAssociationWidget* assoc = new CAssociationWidget(this, AssocData);
	m_Associations.append(assoc);
	return true;
}

bool UMLView::ActivateAfterSerialize( bool bUseLog )
{
	bool status = true;
	CAssociationWidgetData * assocData = 0;
	CUMLWidgetData * widgetData = 0;
	if ( !m_bActivated )
	{
		if( bUseLog )
			BeginPartialWidgetPaste();
   	//create and activate regular widgets first
		CUMLWidgetDataListIt w_it( m_pData -> m_WidgetList );
		while( ( widgetData = w_it.current() ) )
		{
			++w_it;
			if( bUseLog )
				addWidget( widgetData );
			else
				CreateWidget( widgetData );
		}

		//create the message widgets now
      CUMLWidgetDataListIt m_it( m_pData -> m_MessageList );
		widgetData = 0;
		while( ( widgetData = m_it.current() ) )
		{
			++m_it;
			if( bUseLog )
				addWidget( widgetData );
			else
				CreateWidget( widgetData );
		}

		//now create the associations
		CAssociationWidgetDataListIt a_it( m_pData -> m_AssociationList );
		while( ( assocData = a_it.current() ) )
		{
			++a_it;
			if( bUseLog )
				addAssociation( assocData );
			else
				CreateAssoc( assocData );
		}

		//now activate them all

		status =  Activate();

		if( bUseLog )
			EndPartialWidgetPaste();
	}//end if active
	if(status)
	{
		m_bActivated = true;
	}
  	return true;
}

void UMLView::BeginPartialWidgetPaste()
{
	if(m_pIDChangesLog)


	{
		delete    m_pIDChangesLog;
		m_pIDChangesLog = 0;
	}
	m_pIDChangesLog = new CIDChangeLog();
  m_bPaste = true;
}

void UMLView::EndPartialWidgetPaste()
{
	 if(m_pIDChangesLog)
	{
		delete    m_pIDChangesLog;
		m_pIDChangesLog = 0;
	}
  m_bPaste = false;
}

CIDChangeLog* UMLView::GetLocalIDChangeLog()
{
	return m_pIDChangesLog;
}

void UMLView::RemoveAssoc(CAssociationWidget* pAssoc)
{
  if(!pAssoc)
	return;
	if( pAssoc == m_pMoveAssoc )
	{
	  getDocument() -> getDocWindow() -> updateDocumentation( true );
		m_pMoveAssoc = 0;
	}
  pAssoc->cleanup();
  m_Associations.remove(pAssoc);
	pAssoc -> deleteLater();
  delete pAssoc;
}

bool UMLView::GetAssocWidgets(CAssociationWidgetList & Associations)

{
    Associations = m_Associations;
	return true;
}

bool UMLView::setAssoc(UMLWidget *pWidget)
{
	Association_Type type = convert_TBB_AT(m_CurrentCursor);
  m_bDrawRect = false;
  m_SelectionRect.clear();
  //if this we are not concerned here so return
  if(m_CurrentCursor < WorkToolBar::tbb_Generalization || m_CurrentCursor > WorkToolBar::tbb_Anchor)
  {
    return true;
  }
  clearSelected();

  if(!m_pFirstSelectedWidget)
  {
	if( !AssocRules::allowAssociation( type, pWidget ) )
	{
		KMessageBox::error(0, i18n("Incorrect use of associations."), i18n("Association error!"));
		return true;
	}
    //set up position
    QPoint pos;
    pos.setX((int)pWidget -> x() + (pWidget->width() / 2));

    pos.setY((int)pWidget -> y() + (pWidget->height() / 2));
    setPos(pos);
    m_LineToPos = pos;
    m_pFirstSelectedWidget = pWidget;
    viewport() -> setMouseTracking( true );
    if( m_pAssocLine )
      delete m_pAssocLine;
	m_pAssocLine = 0;
    m_pAssocLine = new QCanvasLine( canvas() );
    m_pAssocLine -> setPoints( pos.x(), pos.y(), pos.x(), pos.y() );
    m_pAssocLine -> setPen( QPen( m_pData -> getLineColor(), 0, DashLine ) );

    m_pAssocLine -> setVisible( true );

    return true;
  }
  else
  {
    //The following reasigment is just to make things clearer
	UMLWidget* widgetA = m_pFirstSelectedWidget;
	UMLWidget* widgetB = pWidget;
	UMLWidget_Type at = widgetA -> getBaseType();
	bool valid = true;
   if( widgetA == widgetB )
		valid = AssocRules::allowSelf( type, at );

	else
		valid =  AssocRules::allowAssociation( type, widgetA, widgetB );
	if( valid )
	{
		CAssociationWidget *temp = new CAssociationWidget(this, widgetA, type, widgetB);
      m_Associations.append(temp);
    }
	else
	{
		KMessageBox::error(0, i18n("Incorrect use of associations."), i18n("Association error!"));
	}
    m_pFirstSelectedWidget = 0;

    if( m_pAssocLine )
    {
      delete m_pAssocLine;

      m_pAssocLine = 0;

    }
    return true;
  }//end if firstAssocSet

  if( m_pAssocLine )
  {
    delete m_pAssocLine;
    m_pAssocLine = 0;
  }
	resetToolbar();
	m_pFirstSelectedWidget = 0;
  return false;
}

/** Removes all the associations related to Widget */
void UMLView::RemoveAssociations(UMLWidget* Widget)
{

	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;
		if(assocwidget->contains(Widget))
		{
			RemoveAssoc(assocwidget);

		}
	}
}

void UMLView::SelectAssociations(bool bSelect)
{
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;
		if(bSelect)
		{
			if(assocwidget->getWidgetA()->getSelected() &&
										assocwidget->getWidgetB()->getSelected())
				assocwidget->setSelected(true);
			else
				assocwidget->setSelected(false);
		}
		else
			assocwidget->setSelected(false);
	}//end while
}

void UMLView::GetWidgetAssocs(UMLObject* Obj, CAssociationWidgetList & Associations)
{
	if( ! Obj )
		return;

	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget * assocwidget;
	while((assocwidget = assoc_it.current()))
	{
		if(assocwidget->getWidgetA()->getUMLObject() == Obj
								|| assocwidget->getWidgetB()->getUMLObject() == Obj)
			Associations.append(assocwidget);
		++assoc_it;
	}//end while
}

void UMLView::closeEvent ( QCloseEvent * e )
{
  QWidget::closeEvent(e);
}

void UMLView::RemoveAllAssociations()
{
	//Remove All associations
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))

	{
		++assoc_it;
		m_Associations.remove(assocwidget);
		assocwidget->cleanup();
		delete assocwidget;
	}
}


void UMLView::RemoveAllWidgets()
{
	QObjectList * l = queryList( "UMLWidget");
	QObjectListIt it( *l );
	UMLWidget * temp = 0;
  	while ( (temp=(UMLWidget*)it.current()) != 0 )
	{
		++it;
		if( !( temp -> getBaseType() == wt_Text && ((FloatingText *)temp)-> getRole() != tr_Floating ) )
		{
			removeWidget( temp );
			RemoveAllWidgets();
			return;
		}
	}
	delete l;

}


void UMLView::SetName(QString& strName)
{
  	m_pData -> m_Name = strName;
}

QString UMLView::GetName()
{
	return m_pData -> m_Name;
}

Uml::Association_Type UMLView::convert_TBB_AT(WorkToolBar::ToolBar_Buttons tbb)
{
  Association_Type at = at_Unknown;
  switch(tbb)
  {
    case WorkToolBar::tbb_Anchor:
      at = at_Anchor;
   break;

   case WorkToolBar::tbb_Association:
      at = at_Association;
  break;

    case WorkToolBar::tbb_UniAssociation:
      at = at_UniAssociation;

    break;

    case WorkToolBar::tbb_Generalization:
      at = at_Generalization;
    break;


    case WorkToolBar::tbb_Realization:
      at = at_Realization;
    break;

    case WorkToolBar::tbb_Composition:
      at = at_Composition;
    break;

    case WorkToolBar::tbb_Implementation:
      at = at_Implementation;
    break;

    case WorkToolBar::tbb_Aggregation:
    at = at_Aggregation;
    break;

    case WorkToolBar::tbb_Dependency:
    at = at_Dependency;
    break;


    case WorkToolBar::tbb_Seq_Message:
    at = at_Seq_Message;
    break;

    case WorkToolBar::tbb_Coll_Message:
    at = at_Coll_Message;
    break;

    case WorkToolBar::tbb_State_Transition:
      at = at_State;
    break;

    case WorkToolBar::tbb_Activity_Transition:
      at = at_Activity;
    break;


		default:
		break;
  }
  return at;
}

Uml::UMLObject_Type UMLView::convert_TBB_OT(WorkToolBar::ToolBar_Buttons tbb)
{
  UMLObject_Type ot = ot_UMLObject;

  switch(tbb)
  {
    case WorkToolBar::tbb_Actor:
      ot = ot_Actor;
    break;

    case WorkToolBar::tbb_UseCase:
      ot = ot_UseCase;
    break;

    case WorkToolBar::tbb_Concept:
      ot = ot_Concept;
    break;
		default:
		break;
  }
  return ot;
}

bool UMLView::allocateMousePressEvent(QMouseEvent * me)
{
  m_pMoveAssoc = 0;
  m_pOnWidget = 0;
	m_pSeqLine = 0;

  QObjectList * l = queryList( "UMLWidget");
  QObjectListIt it( *l );
  UMLWidget * obj, * backup = 0;
  while ( (obj=(UMLWidget*)it.current()) != 0 )
  {
    if( obj -> isVisible() && obj -> onWidget( me -> pos() ) )
    {
      //give text object priority
      //they can easily get into a position where
      //you can't touch them.
      if( obj -> getBaseType() == wt_Text )
      {
        m_pOnWidget = obj;
        obj ->  mousePressEvent( me );
        delete l;
        return true;
      }
      else
        backup = obj;
    }
    ++it;
  }//end while
  //if backup is set, then let it have the event
  delete l;
  if(backup)
  {
    backup -> mousePressEvent( me );
    m_pOnWidget = backup;

    return true;

  }
  ///*******now check associations*************///

  CAssociationWidgetListIt assoc_it(m_Associations);
  CAssociationWidget* assocwidget = 0;
  while((assocwidget=assoc_it.current()))
  {
    if( assocwidget -> onAssociation( me -> pos() ))
    {
      assocwidget->mousePressEvent(me);
      m_pMoveAssoc = assocwidget;
      return true;
    }
    ++assoc_it;
  }
	////**********now check sequence lines******///
	if( m_pData -> getType() == dt_Sequence )
	{
		SeqLineWidget * pLine = 0;
		for( pLine = m_SeqLineList.first(); pLine; pLine = m_SeqLineList.next() )
			if( pLine -> onWidget( me -> pos() ) && pLine -> getMouseDown() )
			{
				m_pSeqLine = pLine;
				m_pSeqLine -> mousePressEvent( me );
				return true;
			}
	}
  m_pMoveAssoc = 0;
	m_pSeqLine = 0;
  m_pOnWidget = 0;
  return false;
}

bool UMLView::allocateMouseReleaseEvent(QMouseEvent * me)

{
	//values will already be set through press event.
	//may not be over it, but should still get the event.

	if( m_pOnWidget )
	{
		m_pOnWidget -> mouseReleaseEvent( me );
		return true;
	}

	if( m_pMoveAssoc )
	{
		m_pMoveAssoc -> mouseReleaseEvent( me );
		return true;
	}
	if( m_pSeqLine )
	{
		m_pSeqLine -> mouseReleaseEvent( me );
		return true;
	}
	m_pSeqLine = 0;
	m_pOnWidget = 0;
	m_pMoveAssoc = 0;
  return false;
}

bool UMLView::allocateMouseDoubleClickEvent(QMouseEvent * me)

{
  //values will already be set through press and release events
  if( m_pOnWidget && m_pOnWidget -> onWidget( me -> pos() ))
  {
    m_pOnWidget -> mouseDoubleClickEvent( me );
    return true;
  }
  if( m_pMoveAssoc && m_pMoveAssoc -> onAssociation( me -> pos() ))
  {
    m_pMoveAssoc -> mouseDoubleClickEvent( me );
    return true;
  }
	m_pSeqLine = 0;
	m_pOnWidget = 0;
	m_pMoveAssoc = 0;
  return false;
}

bool UMLView::allocateMouseMoveEvent(QMouseEvent * me)
{
  //tracking may have been set by someone else
  //only move if we set it.  We only want it if
  //left mouse button down.
  if(m_pOnWidget)
  {
    m_pOnWidget -> mouseMoveEvent( me );
    return true;
  }
  if(m_pMoveAssoc)
  {
    m_pMoveAssoc -> mouseMoveEvent( me );
    return true;
  }
	if( m_pSeqLine )
	{
		m_pSeqLine -> mouseMoveEvent( me );
		return true;
	}
  return false;
}



void UMLView::showDocumentation( UMLObject * object, bool overwrite )
{
	getDocument() -> getDocWindow() -> showDocumentation( object, overwrite );
}

void UMLView::showDocumentation( UMLWidget * widget, bool overwrite )
{
	getDocument() -> getDocWindow() -> showDocumentation( widget, overwrite );
}

void UMLView::showDocumentation( CAssociationWidget * widget, bool overwrite )
{
	getDocument() -> getDocWindow() -> showDocumentation( widget, overwrite );
}

void UMLView::updateDocumentation( bool clear )
{
	getDocument() -> getDocWindow() -> updateDocumentation( clear );
}

void UMLView::createAutoAssociations( UMLWidget * widget )
{
	if( !widget )
		return;
	CAssociationWidgetData * docData = 0;

	UMLWidget * widgetA = 0;
	UMLWidget * widgetB = 0;

	CAssociationWidgetDataList list;

	getDocument() -> getAssciationListAllViews( this, widget -> getUMLObject(), list );
	CAssociationWidgetDataListIt it( list );

	while( ( docData = it.current() ) )
	{
		if( docData -> getAssocType() != at_Anchor )
		{
			//see if both widgets are on this diagram
			//if yes - create an association
			widgetA = widget;
			widgetB = findWidget( docData -> getWidgetBID() );
         if( docData -> getWidgetAID() != widget -> getID() )
			{
				widgetA = findWidget( docData -> getWidgetAID() );
				widgetB = widget;

			}

			if( widgetA && widgetB )
			{
				CAssociationWidget * temp = new CAssociationWidget( this, widgetA ,
				docData -> getAssocType(), widgetB );
				CFloatingTextData * data = docData -> getMultiDataA();
				if( data )
					temp -> setMultiA( data -> getText() );
				data =  docData -> getMultiDataB();

				if( data )

					temp -> setMultiB( data -> getText() );
				data = docData -> getRoleData();
				if( data )
					temp -> setRole( data -> getText() );
      		m_Associations.append( temp );
			}
		}//end if !at_Anchor
		++it;
	}//end while docAssoc
}

void UMLView::copyAsImage(QPixmap*& pix)
{

	int px, py, qx, qy;
	int x, y, x1, y1;
	//get the smallest rect holding the diagram
	QRect rect = getDiagramRect();
	QPixmap diagram(rect.width() - rect.x(), rect.height() - rect.y());
	//only draw what is selected
	m_bDrawSelectedOnly = true;
	SelectAssociations(true);
	getDiagram(diagram);

	//now get the selection cut
	//first get the smallest rect holding the widgets
	px = py = qx = qy = -1;
	UMLWidget * temp = 0;

	for(temp=(UMLWidget *)m_SelectedList.first();temp;temp=(UMLWidget *)m_SelectedList.next())
	{
		x = (int)temp -> x();
		y = (int)temp -> y();
		x1 = x + temp -> width() - 1;
		y1 = y + temp -> height() - 1;
		if(px == -1 || x < px)

			px = x;
		if(py == -1 || y < py)
			py = y;

		if(qx == -1 || x1 > qx)
			qx = x1;
		if(qy == -1 || y1 > qy)

			qy = y1;
	}
	//also take into account any text lines in assocs or messages

	CAssociationWidget *assocwidget;
	CAssociationWidgetListIt assoc_it(m_Associations);
	//get each type of associations
	//This needs to be reimplemented to increase the rectangle
	//if a part of any association is not included
 	while((assocwidget = assoc_it.current()))

	{
		++assoc_it;
  		if(assocwidget->getSelected())

  		{
  			temp = const_cast<FloatingText*>(assocwidget->getMultiAWidget());
  			if(temp && temp -> isVisible())
  			{

  				x = (int)temp -> x();
				y = (int)temp -> y();

				x1 = x + temp -> width() - 1;
				y1 = y + temp -> height() - 1;
				if(px == -1 || x < px)
					px = x;
				if(py == -1 || y < py)
					py = y;

				if(qx == -1 || x1 > qx)
					qx = x1;
				if(qy == -1 || y1 > qy)
					qy = y1;
  			}//end if temp

  			temp = const_cast<FloatingText*>(assocwidget -> getMultiBWidget());

  			if(temp && temp -> isVisible())
  			{
  				x = (int)temp -> x();

				y = (int)temp -> y();
				x1 = x + temp -> width() - 1;
				y1 = y + temp -> height() - 1;
				if(px == -1 || x < px)
					px = x;
				if(py == -1 || y < py)
					py = y;

				if(qx == -1 || x1 > qx)
					qx = x1;
				if(qy == -1 || y1 > qy)

					qy = y1;
  			}//end if temp
  			temp = const_cast<FloatingText*>(assocwidget->getRoleWidget());
  			if(temp && temp -> isVisible())

  			{

  				x = (int)temp -> x();

				y = (int)temp -> y();

				x1 = x + temp -> width() - 1;

				y1 = y + temp -> height() - 1;
				if(px == -1 || x < px)
					px = x;
				if(py == -1 || y < py)
					py = y;


				if(qx == -1 || x1 > qx)
					qx = x1;

				if(qy == -1 || y1 > qy)


					qy = y1;
  			}//end if temp
  		}//end if selected
 	}//end while
	rect.setLeft(px);

	rect.setTop(py);
	rect.setRight(qx);
	rect.setBottom(qy);

	pix = new QPixmap(rect.width(), rect.height());
	bitBlt(pix, QPoint(0, 0), &diagram, rect);
	m_bDrawSelectedOnly = false;
	//SelectAssociations(false);
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void UMLView::setMenu()
{
	slotRemovePopupMenu();
  ListPopupMenu::Menu_Type menu = ListPopupMenu::mt_Undefined;
	switch( m_pData -> m_Type )
	{
		case dt_Class:
			menu = ListPopupMenu::mt_On_Class_Diagram;
		break;

		case dt_UseCase:

			menu = ListPopupMenu::mt_On_UseCase_Diagram;
		break;

		case dt_Sequence:
			menu = ListPopupMenu::mt_On_Sequence_Diagram;
		break;

		case dt_Collaboration:
			menu = ListPopupMenu::mt_On_Collaboration_Diagram;
		break;

		case dt_State:
			menu = ListPopupMenu::mt_On_State_Diagram;
		break;

		case dt_Activity:
			menu = ListPopupMenu::mt_On_Activity_Diagram;
		break;

		default:
			menu = ListPopupMenu::mt_Undefined;
		break;
	}//end switch
	if( menu != ListPopupMenu::mt_Undefined )
	{
		m_pMenu = new ListPopupMenu(this, menu, this );
		connect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotMenuSelection(int)));
		m_pMenu -> popup( mapToGlobal(contentsToViewport( m_Pos ) ) );
	}
}

void UMLView::slotRemovePopupMenu()
{
	if(m_pMenu)
	{
		disconnect(m_pMenu, SIGNAL(activated(int)), this, SLOT(slotMenuSelection(int)));
		delete m_pMenu;
		m_pMenu = 0;
	}
}

void UMLView::slotMenuSelection(int sel)
{
	FloatingText * ft = 0;
  StateWidget * state = 0;
	ActivityWidget * activity = 0;
  bool ok = false;
  QString name = "";




	switch(((ListPopupMenu::Menu_Type)sel))
	{
		case ListPopupMenu::mt_Redo:
		break;

		case ListPopupMenu::mt_Undo :
		break;

		case ListPopupMenu::mt_Clear:

			if(  KMessageBox::Yes == KMessageBox::warningYesNo( this, i18n( "You are about to delete the entire diagram.\nAre you sure?" ),
				i18n( "Delete diagram?" ) ) )
				RemoveAllWidgets();
		break;

		case ListPopupMenu::mt_Export_Image:
			exportImage( KFileDialog::getSaveURL(QDir::currentDirPath(), i18n("*.png|PNG Image\n*.*|All Files"), this, i18n("Save as...") ));
		break;

		case ListPopupMenu::mt_FloatText:
			ft = new FloatingText(this, tr_Floating, "");
    		ft -> changeTextDlg();
    		//if no text entered delete
    		if(!FloatingText::isTextValid(ft -> getText()))

      		delete ft;
    		else
    		{
     			ft -> setX( m_Pos.x() );
				ft -> setY( m_Pos.y() );
				ft -> setVisible( true );
				ft -> setID(getDocument() -> getUniqueID());
   			ft->SetActivated();
   		}
		break;

		case ListPopupMenu::mt_UseCase:
			m_bCreateObject = true;
			getDocument()->createUMLObject( ot_UseCase );
		break;

		case ListPopupMenu::mt_Actor:
			m_bCreateObject = true;
  			getDocument()->createUMLObject( ot_Actor );
		break;

		case ListPopupMenu::mt_Concept:
		case ListPopupMenu::mt_Object:
			m_bCreateObject = true;
  			getDocument()->createUMLObject( ot_Concept );
		break;

		case ListPopupMenu::mt_Paste:
			m_PastePoint = m_Pos;
			m_Pos.setX( 2000 );
			m_Pos.setY( 2000 );
			getDocument() -> EditPaste();

			m_PastePoint.setX( 0 );
			m_PastePoint.setY( 0 );
		break;

		case ListPopupMenu::mt_Initial_State:
			state = new StateWidget( this , StateWidget::Initial );
			state -> setID( getDocument() -> getUniqueID() );//needed for associations
			state -> setX( m_Pos.x() );
			state -> setY ( m_Pos.y() );
			state -> setVisible( true );
			state -> SetActivated();
			break;

		case ListPopupMenu::mt_End_State:
			state = new StateWidget( this , StateWidget::End );
			state -> setID( getDocument() -> getUniqueID() );//needed for associations
			state -> setX( m_Pos.x() );
			state -> setY ( m_Pos.y() );
			state -> setVisible( true );
			state -> SetActivated();
		break;

		case ListPopupMenu::mt_State:
			name = KLineEditDlg::getText( i18n("Enter State name."), i18n("Enter the name of the new State."), i18n("new state"), &ok );
			if( ok )
			{
				state = new StateWidget( this , StateWidget::Normal );
				state -> SetName( name );
				state -> setID( getDocument() -> getUniqueID() );//needed for associations
				state -> setX( m_Pos.x() );
				state -> setY ( m_Pos.y() );
				state -> setVisible( true );
				state -> SetActivated();
			}
		break;

		case ListPopupMenu::mt_Initial_Activity:
			activity = new ActivityWidget( this , ActivityWidget::Initial );
			activity -> setID( getDocument() -> getUniqueID() );//needed for associations
			activity -> setX( m_Pos.x() );
			activity -> setY ( m_Pos.y() );
			activity -> setVisible( true );
			activity -> SetActivated();
		break;


		case ListPopupMenu::mt_End_Activity:
			activity = new ActivityWidget( this , ActivityWidget::End );
			activity -> setID( getDocument() -> getUniqueID() );//needed for associations
			activity -> setX( m_Pos.x() );
			activity -> setY ( m_Pos.y() );
			activity -> setVisible( true );
			activity -> SetActivated();
		break;

		case ListPopupMenu::mt_Branch:
			activity = new ActivityWidget( this , ActivityWidget::Branch );
			activity -> setID( getDocument() -> getUniqueID() );//needed for associations
			activity -> setX( m_Pos.x() );
			activity -> setY ( m_Pos.y() );
			activity -> setVisible( true );
			activity -> SetActivated();
		break;

		case ListPopupMenu::mt_Activity:
			name = KLineEditDlg::getText( i18n("Enter Activity name."), i18n("Enter the name of the new Activity."), i18n("new activity"), &ok );
			if( ok )
			{
				activity = new ActivityWidget( this , ActivityWidget::Normal );
				activity -> SetName( name );
				activity -> setID( getDocument() -> getUniqueID() );//needed for associations
				activity -> setX( m_Pos.x() );
				activity -> setY ( m_Pos.y() );
				activity -> setVisible( true );
				activity -> SetActivated();
			}
		break;

		case ListPopupMenu::mt_SnapToGrid:
			m_pData -> setSnapToGrid( ( m_pData -> getSnapToGrid()?false:true ) );

		break;

		case ListPopupMenu::mt_ShowSnapGrid:
			m_pData -> setShowSnapGrid( ( m_pData -> getShowSnapGrid()?false:true ) );
			canvas() -> setAllChanged();
		break;

		case ListPopupMenu::mt_Properties:
			showPropDialog();
		break;

		default:
		break;
	}
}


void UMLView::SynchronizeData()
{
	//get all the data for associations
	m_pData -> m_AssociationList.clear();
	CAssociationWidgetListIt assoc_it(m_Associations);
	CAssociationWidget* assocwidget = 0;
	while((assocwidget=assoc_it.current()))
	{
		++assoc_it;
		assocwidget -> SynchronizeData();
		m_pData -> m_AssociationList.append( assocwidget -> getData() );
	}
	//get all the data for messagewidgets
	m_pData -> m_MessageList.clear();
	QObjectList * l = queryList( "UMLWidget" );
	QObjectListIt it( *l );
	UMLWidget *obj = 0;
	while ( (obj=(UMLWidget*)it.current()) != 0 )
	{
		++it;
 		if(obj->getBaseType() == wt_Message)
		{
			obj -> SynchronizeData();
			m_pData -> m_MessageList.append( obj -> getData() );
		}
	}//end while

	delete l;
	m_pData -> m_WidgetList.clear();
	//get all the data for the rest of the widgets - for text only store of role tr_Floating
	QObjectList * wl = queryList( "UMLWidget" );
	QObjectListIt wit( *wl );
	obj = 0;
	while ( (obj=(UMLWidget*)wit.current()) != 0 )
	{
		++wit;
		Uml::UMLWidget_Type type = obj -> getBaseType();
		switch( type )
		{
			case wt_Actor:
			case wt_UseCase:
			case wt_Class:
			case wt_Object:
			case wt_Note:
			case wt_State:
			case wt_Activity:
				obj -> SynchronizeData();
				m_pData -> m_WidgetList.append(obj -> getData() );
			break;


			case wt_Text:
				if( ((FloatingText *) obj ) -> getRole() == tr_Floating )
				{
					obj -> SynchronizeData();
					m_pData -> m_WidgetList.append( obj -> getData() );
				}
			break;
			default:
			break;
		}//end switch
	}//end while
	delete wl;
}

void UMLView::slotCutSuccessful()
{
	if( m_bStartedCut )
	{
		deleteSelection();
		m_bStartedCut = false;
	}
}

QPoint UMLView::getPastePoint()
{
   QPoint point = m_PastePoint;
	point.setX( point.x() - m_Pos.x() );
	point.setY( point.y() - m_Pos.y() );
	return point;
}

bool UMLView::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  SynchronizeData();
  return m_pData -> saveToXMI( qDoc, qElement );
}

bool UMLView::showPropDialog()
{
	UMLViewDialog dlg( this, this );
	if( dlg.exec() )
	{
		return true;
	}
	return false;
}


void UMLView::setFont( QFont font )
{
	m_pData -> setFont( font );
	UMLWidget * pWidget = 0;
	QObjectList * wl = queryList( "UMLWidget" );
	QObjectListIt wit( *wl );
	while ( ( pWidget = dynamic_cast<UMLWidget *>( wit.current() ) )  != 0 )
	{
		++wit;
		pWidget -> setFont( font );
	}
	delete wl;
	canvas() -> setAllChanged();
}

void UMLView::setConceptWidgetOptions( ClassOptionsPage * page )
{
	UMLWidget * pWidget = 0;
	QObjectList * wl = queryList( "UMLWidget" );
	QObjectListIt wit( *wl );
	while ( ( pWidget = dynamic_cast<UMLWidget *>( wit.current() ) )  != 0 )
	{
		++wit;
		if( pWidget -> getBaseType() == Uml::wt_Class )
		{
			page -> setWidget( dynamic_cast<ConceptWidget *>( pWidget ) );
			page -> updateUMLWidget();
		}
	}
	delete wl;
}


void UMLView::checkSelections()
{
	UMLWidget * pWA = 0, * pWB = 0, * pTemp = 0;
	//check messages
	for(pTemp=(UMLWidget *)m_SelectedList.first();pTemp;pTemp=(UMLWidget *)m_SelectedList.next())
	{
		if( pTemp->getBaseType() == wt_Message && pTemp -> getSelected() )
		{
			MessageWidget * pMessage = dynamic_cast<MessageWidget *>( pTemp );
			pWA = pMessage -> getWidgetA();
			pWB = pMessage -> getWidgetB();
			if( !pWA -> getSelected() )
			{
				pWA -> m_bSelected = true;
				m_SelectedList.append( pWA );
			}
			if( !pWB -> getSelected() )
			{
				pWB -> m_bSelected = true;
				m_SelectedList.append( pWB );
			}
		}//end if
	}//end for
	//check Associations
	CAssociationWidgetListIt it(m_Associations);
	CAssociationWidget * pAssoc = 0;
	while((pAssoc = it.current()))
	{
		++it;
		if( pAssoc -> getSelected() )
		{
			pWA = pAssoc -> getWidgetA();
			pWB = pAssoc -> getWidgetB();
			if( !pWA -> getSelected() )
			{
				pWA -> m_bSelected = true;
				m_SelectedList.append( pWA );
			}
			if( !pWB -> getSelected() )
			{
				pWB -> m_bSelected = true;
				m_SelectedList.append( pWB );
			}
		}//end if
	}//end while
}

QPtrList<MessageWidget> UMLView::getMessageWidgetList()
{
	UMLWidget * pWidget = 0;
	QObjectList * wl = queryList( "UMLWidget" );
	QObjectListIt wit( *wl );
	QPtrList<MessageWidget> list;
	while ( ( pWidget = dynamic_cast<UMLWidget *>( wit.current() ) )  != 0 )
	{
		++wit;
		if( pWidget -> getBaseType() == Uml::wt_Message )
		{
			list.append( (MessageWidget*)pWidget );
		}
	}
	delete wl;
	return list;
}














