#include <qdatastream.h>
#include <qcolor.h>
#include <kdebug.h>

#include "umlwidgetdata.h"
#include "umlview.h"
#include "associationwidgetdata.h"
#include "umlwidgetdata.h"
#include "usecasewidgetdata.h"
#include "conceptwidgetdata.h"
#include "actorwidgetdata.h"
#include "objectwidgetdata.h"
#include "messagewidgetdata.h"
#include "floatingtextdata.h"
#include "notewidgetdata.h"
#include "statewidgetdata.h"
#include "activitywidgetdata.h"
#include "umlviewdata.h"

UMLViewData::UMLViewData()
{
	m_nID = -1;
	m_Documentation = ""; 
	m_Name = "umlview";
	m_Type = dt_Undefined; 
	m_nLocalID = 30000; 
	m_bUseSnapToGrid = false;
	m_bShowSnapGrid = false;
	m_nSnapX = 10;
	m_nSnapY = 10;
}

UMLViewData::~UMLViewData()
{
}

UMLViewData::UMLViewData( UMLViewData & other )
{
	*this = other;
}

UMLViewData & UMLViewData::operator=( UMLViewData & rhs )
{
	if( this == &rhs )
		return *this;

	m_nID = rhs.m_nID;
	m_Documentation = rhs.m_Documentation; 
	m_Name = rhs.m_Name;
	m_Type = rhs.m_Type; 
	m_nLocalID = rhs.m_nLocalID; 
	m_Options = rhs.m_Options;

	return *this;
}

bool UMLViewData::operator==( UMLViewData & rhs )
{
	if( this == &rhs )
		return true;

	if( m_nID != rhs.m_nID )
		return false;

	if( m_Documentation != rhs.m_Documentation )
		return false;

	if( m_Name != rhs.m_Name )
		return false;

	if( m_Type != rhs.m_Type )
		return false;

	if( m_nLocalID != rhs.m_nLocalID )
		return false;

	return true;
}

bool UMLViewData::serialize( QDataStream * stream, bool bArchive )
{
	bool bStatus = true;
   if( bArchive )
	{
		int nType = m_Type;
		*stream	<<	nType
						<<	m_Name
						<<	m_nID
						<<	m_Documentation
						<<	m_nLocalID
						<<	m_Options.uiState.fillColor
						<<	m_Options.uiState.lineColor
						<<	m_Options.uiState.font
						<<	(int)m_Options.uiState.useFillColor
						<< m_nSnapX
						<< m_nSnapY
						<< (int)m_bShowSnapGrid
						<< (int)m_bUseSnapToGrid
						<< (int)m_Options.classState.showAttSig
						<< (int)m_Options.classState.showAtts
						<< (int)m_Options.classState.showOpSig
						<< (int)m_Options.classState.showOps
						<< (int)m_Options.classState.showPackage
						<< (int)m_Options.classState.showScope
						<< (int)m_Options.classState.showStereoType;
	}
	else
	{
		int nType = 0;
		int nSnapgrid = 0, nShowsnap = 0, nUseFC = 0, nTemp = 0;
		*stream	>>	nType
						>>	m_Name
						>>	m_nID
						>>	m_Documentation
						>>	m_nLocalID
						>>	m_Options.uiState.fillColor
						>>	m_Options.uiState.lineColor
						>>	m_Options.uiState.font
						>>	nUseFC
						>> m_nSnapX
						>> m_nSnapY
						>> nShowsnap >> nSnapgrid;
		*stream >> nTemp;
		m_Options.classState.showAttSig = nTemp;
		*stream >> nTemp;
		m_Options.classState.showAtts = nTemp;
		*stream >> nTemp;
		m_Options.classState.showOpSig = nTemp;
		*stream >> nTemp;
		m_Options.classState.showOps = nTemp;
		*stream >> nTemp;
		m_Options.classState.showPackage = nTemp;
		*stream >> nTemp;
		m_Options.classState.showScope = nTemp;
		*stream >> nTemp;
		m_Options.classState.showStereoType = nTemp;
		m_Options.uiState.useFillColor = nUseFC;
		m_bUseSnapToGrid = nSnapgrid;
		m_bShowSnapGrid = nShowsnap;
		m_Type = (Uml::Diagram_Type)nType;
	}

	bStatus = serializeWidgets( stream, bArchive );
	if( bStatus )
		bStatus = serializeMessages( stream, bArchive );
	if( bStatus )
		bStatus = serializeAssociations( stream, bArchive );

	return bStatus;
}


long UMLViewData::getClipSizeOf()
{
	long l_size = 0;
   l_size += sizeof( int ); //type
	l_size += sizeof( m_Name ); //not allowed to be blank so ok not to test
	l_size += sizeof( m_nID );
	l_size += sizeof( m_nLocalID );
	l_size += sizeof( int ) * 4; //m_Options.classState - docwindow
	l_size += sizeof( int ) * 7;//m_Options.uiState
	l_size += sizeof( m_nSnapX );
	l_size += sizeof( m_nSnapY );
	l_size += sizeof( int) * 2; //m_bUseSnapToGrid m_bShowSnapGrid

	if( m_Documentation.length() == 0 )
		l_size += sizeof( Q_UINT32 );
	else
		l_size += ( sizeof( QChar ) * m_Documentation.length() );

	l_size += ( ( sizeof( int ) * 3 ) );//counts of each the three lists
	l_size += ( m_WidgetList.count() * sizeof( int ) ); // type of each widget as header (assoc. and message don't need it)

	CUMLWidgetData * widgetData = 0;
	CAssociationWidgetData * assocData = 0;
	//get size of each widget data item
	CUMLWidgetDataListIt w_it( m_WidgetList );
	while( ( widgetData = w_it.current() ) )
	{
		++w_it;
		l_size += widgetData -> getClipSizeOf(); 
	}

	//get size of each message widget data item
   CUMLWidgetDataListIt m_it( m_MessageList );
	widgetData = 0;
	while( ( widgetData = m_it.current() ) )
	{
		++m_it;
		l_size += widgetData -> getClipSizeOf(); 
	}

	//get size of each association data item
	CAssociationWidgetDataListIt a_it( m_AssociationList );
	while( ( assocData = a_it.current() ) )
	{
		++a_it;
		l_size += assocData -> getClipSizeOf(); 
	}
	return l_size;
}

QString UMLViewData::getDoc()
{
	return m_Documentation;
}

void UMLViewData::setDoc( QString doc )
{
	m_Documentation = doc;
}

QString UMLViewData::getName()
{
	return m_Name;
}
                          
void UMLViewData::setName( QString name )
{
	m_Name = name;
}

Uml::Diagram_Type UMLViewData::getType()
{
	return m_Type;
}
	
void UMLViewData::setType( Uml::Diagram_Type type )
{
	m_Type = type;
}

QColor UMLViewData::getFillColor()
{
	return m_Options.uiState.fillColor;
}

void UMLViewData::setFillColor( QColor color )
{
	m_Options.uiState.fillColor = color;
}

QColor UMLViewData::getLineColor()
{
	return m_Options.uiState.lineColor;
}

void UMLViewData::setLineColor( QColor color )
{
	m_Options.uiState.lineColor = color;
}

int UMLViewData::getID()
{
	return m_nID;
}

void UMLViewData::setID( int ID )
{
	m_nID = ID;
}

int UMLViewData::getUniqueID()
{
	return --m_nLocalID;
}

bool UMLViewData::serializeWidgets( QDataStream * stream, bool bArchive )
{
	int nCount = 0, nType;
	bool bStatus = true;
	CUMLWidgetData * widgetData = 0;
   if( bArchive )
	{
		nCount = m_WidgetList.count();
		*stream	<<	nCount;
		CUMLWidgetDataListIt w_it( m_WidgetList );
		while( ( widgetData = w_it.current() ) )
		{
			++w_it;
			nType = widgetData -> getType();
			*stream	<<	nType;
			bStatus = widgetData -> serialize( stream, bArchive ); 
			if( !bStatus )
				return false;
		}
	}
	else
	{
		m_WidgetList.clear();
		*stream	>>	nCount;
		for( int i = 0; i < nCount; i++ )
		{
			*stream	>>	nType;	
			Uml::UMLWidget_Type type = (Uml::UMLWidget_Type)nType;
			if( type == wt_UseCase )
			{
					CUseCaseWidgetData * uc = new CUseCaseWidgetData();
					widgetData = dynamic_cast<CUMLWidgetData *>( uc );
			}
         else if( type == wt_Actor )
			{
					CActorWidgetData * a = new CActorWidgetData();
					widgetData = dynamic_cast<CUMLWidgetData *>( a );
			}
			else if( type == wt_Class )
			{
					CConceptWidgetData * c = new CConceptWidgetData();
					widgetData = dynamic_cast<CUMLWidgetData *>( c );
			}
         else if( type == wt_Object )
			{
					CObjectWidgetData * o = new CObjectWidgetData();
					widgetData = dynamic_cast<CUMLWidgetData *>( o );
			}
         else if( type == wt_Note )
			{
					CNoteWidgetData * n = new CNoteWidgetData();
					widgetData = dynamic_cast<CUMLWidgetData *>( n );
			}
			else if( type == wt_Text )
			{
					CFloatingTextData * ft = new CFloatingTextData();
					widgetData = dynamic_cast<CUMLWidgetData *>( ft );
			}
			else if( type == wt_State )
			{
				CStateWidgetData * s = new CStateWidgetData();
				widgetData = dynamic_cast<CUMLWidgetData *>( s );
			}
			else if( type == wt_Activity )
			{
				CActivityWidgetData * a = new CActivityWidgetData();
				widgetData = dynamic_cast<CUMLWidgetData *>( a );
			}
			else
			{
					kdDebug()<<"UMLViewData::error: trying to load bad widget type."<<endl;
					return false;
			}//end else
			bStatus = widgetData -> serialize( stream, bArchive );
			if( !bStatus )
				return false;
			m_WidgetList.append( widgetData );
		}//end for
	}//end else
	return true;
}

bool UMLViewData::serializeMessages( QDataStream * stream, bool bArchive )
{
	int nCount = 0;
	bool bStatus = true;
	CUMLWidgetData * widgetData = 0;
   if( bArchive )
	{
		nCount = m_MessageList.count();
		*stream	<<	nCount;
		CUMLWidgetDataListIt m_it( m_MessageList );
		while( ( widgetData = m_it.current() ) )
		{
			++m_it;
			bStatus = widgetData -> serialize( stream, bArchive ); 
			if( !bStatus )
				return false;
		}
	}
	else
	{
		m_MessageList.clear();
		*stream	>>	nCount;
		for( int i = 0; i < nCount; i++ )
		{
			CMessageWidgetData * messageData = new CMessageWidgetData();
			bStatus = messageData -> serialize( stream, bArchive );
			if( !bStatus )
				return false;
			m_MessageList.append( messageData );
		}
	}
	return true;
}

bool UMLViewData::serializeAssociations( QDataStream * stream, bool bArchive )
{
	int nCount = 0;
	bool bStatus = true;
	CAssociationWidgetData * assocData = 0;
   if( bArchive )
	{
		nCount = m_AssociationList.count();
		*stream	<<	nCount;
		CAssociationWidgetDataListIt a_it( m_AssociationList );
		while( ( assocData = a_it.current() ) )
		{
			++a_it;
			bStatus = assocData -> serialize( stream, bArchive ); 
			if( !bStatus )
				return false;
		}
	}
	else
	{
		m_AssociationList.clear();
		*stream	>>	nCount;
		for( int i = 0; i < nCount; i++ )
		{
			assocData = new CAssociationWidgetData();
			bStatus = assocData -> serialize( stream, bArchive );
			if( !bStatus )
				return false;
			m_AssociationList.append( assocData );
		}
	}
	return true;
}

bool UMLViewData::saveToXMI( QDomDocument & qDoc, QDomElement & qElement )
{
  QDomElement viewElement = qDoc.createElement( "diagram" );
  viewElement.setAttribute( "xmi.id", m_nID );
  viewElement.setAttribute( "name", m_Name );
  viewElement.setAttribute( "type", m_Type );
  viewElement.setAttribute( "documentation", m_Documentation );
	//optionstate uistate
  viewElement.setAttribute( "fillcolor", m_Options.uiState.fillColor.name() );
  viewElement.setAttribute( "linecolor", m_Options.uiState.lineColor.name() );
	viewElement.setAttribute( "usefillcolor", m_Options.uiState.useFillColor );
	viewElement.setAttribute( "font", m_Options.uiState.font.toString() ); 
	//optionstate classstate
	viewElement.setAttribute( "showattsig", m_Options.classState.showAttSig ); 
	viewElement.setAttribute( "showatts", m_Options.classState.showAtts); 
	viewElement.setAttribute( "showopsig", m_Options.classState.showOpSig ); 
	viewElement.setAttribute( "showops", m_Options.classState.showOps ); 
	viewElement.setAttribute( "showpackage", m_Options.classState.showPackage ); 
	viewElement.setAttribute( "showscope", m_Options.classState.showScope ); 
	viewElement.setAttribute( "showstereotype", m_Options.classState.showStereoType ); 
	//misc
  viewElement.setAttribute( "localid", m_nLocalID );
	viewElement.setAttribute( "showgrid", m_bShowSnapGrid );
	viewElement.setAttribute( "snapgrid", m_bUseSnapToGrid );
	viewElement.setAttribute( "snapx", m_nSnapX );
	viewElement.setAttribute( "snapy", m_nSnapY );
  //now save all the widgets
  CUMLWidgetData * widgetData = 0;
  CUMLWidgetDataListIt w_it( m_WidgetList );
  QDomElement widgetElement = qDoc.createElement( "widgets" );
  while( ( widgetData = w_it.current() ) )
  {
    ++w_it;
    widgetData -> saveToXMI( qDoc, widgetElement );
  }
  viewElement.appendChild( widgetElement );
  //now save the message widgets
  CUMLWidgetDataListIt m_it( m_MessageList );
  QDomElement messageElement = qDoc.createElement( "messages" );
  while( ( widgetData = m_it.current() ) )
  {
    ++m_it;
    widgetData -> saveToXMI( qDoc, messageElement );
  }
  viewElement.appendChild( messageElement );
  //now save the associations
  CAssociationWidgetData * assocData = 0;
  CAssociationWidgetDataListIt a_it( m_AssociationList );
  QDomElement assocElement = qDoc.createElement( "associations" );
  while( ( assocData = a_it.current() ) )
  {
    ++a_it;
    assocData -> saveToXMI( qDoc, assocElement );
  }
  viewElement.appendChild( assocElement );
  qElement.appendChild( viewElement );
  return true;
}

bool UMLViewData::loadFromXMI( QDomElement & qElement )
{
  QString id = qElement.attribute( "xmi.id", "-1" );
  m_nID = id.toInt();
  if( m_nID == -1 )
    return false;
	m_Name = qElement.attribute( "name", "" );
	QString type = qElement.attribute( "type", "-1" );
	m_Documentation = qElement.attribute( "documentation", "" );
	  QString localid = qElement.attribute( "localid", "0" );
	//optionstate uistate
	QString font = qElement.attribute( "font", "" );
	if( font != "" )
		m_Options.uiState.font.fromString( font );
  QString fillcolor = qElement.attribute( "fillcolor", "" );
  QString linecolor = qElement.attribute( "linecolor", "" );
	QString usefillcolor = qElement.attribute( "usefillcolor", "0" );
	m_Options.uiState.useFillColor = (bool)usefillcolor.toInt();
	//optionstate classstate
	QString temp = qElement.attribute( "showattsig", "0" );
	m_Options.classState.showAttSig = (bool)temp.toInt();
	temp = qElement.attribute( "showatts", "0" );
	m_Options.classState.showAtts = (bool)temp.toInt();
	temp = qElement.attribute( "showopsig", "0" );
	m_Options.classState.showOpSig = (bool)temp.toInt();
	temp = qElement.attribute( "showops", "0" );
	m_Options.classState.showOps = (bool)temp.toInt();
	temp = qElement.attribute( "showpackage", "0" );
	m_Options.classState.showPackage = (bool)temp.toInt();
	temp = qElement.attribute( "showscope", "0" );
	m_Options.classState.showScope = (bool)temp.toInt();
	temp = qElement.attribute( "showstereotype", "0" );
	m_Options.classState.showStereoType = (bool)temp.toInt();
	//misc 
	QString showgrid = qElement.attribute( "showgrid", "0" );
	m_bShowSnapGrid = (bool)showgrid.toInt();

	QString snapgrid = qElement.attribute( "snapgrid", "0" );
	m_bUseSnapToGrid = (bool)snapgrid.toInt();

	QString snapx = qElement.attribute( "snapx", "10" );
	m_nSnapX = snapx.toInt();

	QString snapy = qElement.attribute( "snapy", "10" );
	m_nSnapY = snapy.toInt();
  m_Type = (Uml::Diagram_Type)type.toInt();
  if( fillcolor != "" )
    m_Options.uiState.fillColor = QColor( fillcolor );
  if( linecolor != "" )
    m_Options.uiState.lineColor = QColor( linecolor );
  m_nLocalID = localid.toInt();
  //load the widgets
  QDomNode node = qElement.firstChild();
  QDomElement element = node.toElement();
  if( !element.isNull() && element.tagName() != "widgets" )
    return false;
  if( !loadWidgetsFromXMI( element ) )
    return false;

  //load the message widgets
  node = element.nextSibling();
  element = node.toElement();
  if( !element.isNull() && element.tagName() != "messages" )
    return false;
  if( !loadMessagesFromXMI( element ) )
    return false;

  //load the associations
  node = element.nextSibling();
  element = node.toElement();
  if( !element.isNull() && element.tagName() != "associations" )
    return false;
  if( !loadAssociationsFromXMI( element ) )
    return false;
  return true;
}

bool UMLViewData::loadWidgetsFromXMI( QDomElement & qElement )
{
  CUMLWidgetData * widgetData = 0;
  QDomNode node = qElement.firstChild();
  QDomElement widgetElement = node.toElement();
  while( !widgetElement.isNull() )
  {
    QString tag = widgetElement.tagName();
    if( tag == "UML:ActorWidget" )
    {
      widgetData = new CActorWidgetData();
    }
    else if( tag == "UML:UseCaseWidget" )
    {
      widgetData = new CUseCaseWidgetData();
    }
    else if( tag == "UML:ConceptWidget" )
    {
      widgetData = new CConceptWidgetData();
    }
    else if( tag == "UML:StateWidget" )
    {
      widgetData = new CStateWidgetData();
    }
    else if( tag == "UML:NoteWidget" )
    {
      widgetData = new CNoteWidgetData();
    }
    else if( tag == "UML:FloatingTextWidget" )
    {
      widgetData = new CFloatingTextData();
    }
    else if( tag == "UML:ObjectWidget" )
    {
      widgetData = new CObjectWidgetData();
    }
		else if( tag == "UML:ActivityWidget" )
		{
			widgetData = new CActivityWidgetData();
		}
    else
    {
      kdDebug()<<"Trying to create an unknown widget"<<endl;
      return false;
    }
    if( !widgetData -> loadFromXMI( widgetElement ) )
      return false;
    m_WidgetList.append( widgetData );
    node = widgetElement.nextSibling();
    widgetElement = node.toElement();
  }

  return true;
}

bool UMLViewData::loadMessagesFromXMI( QDomElement & qElement )
{
  CMessageWidgetData * messageData = 0;
  QDomNode node = qElement.firstChild();
  QDomElement messageElement = node.toElement();
  while( !messageElement.isNull() )
  {
    if( messageElement.tagName() == "UML:MessageWidget" )
    {
      messageData = new CMessageWidgetData();
      if( !messageData -> loadFromXMI( messageElement ) )
        return false;
      m_MessageList.append( messageData );
    }
    node = messageElement.nextSibling();
    messageElement = node.toElement();
  }
  return true;
}

bool UMLViewData::loadAssociationsFromXMI( QDomElement & qElement )
{
  CAssociationWidgetData * assocData = 0;
  QDomNode node = qElement.firstChild();
  QDomElement assocElement = node.toElement();
  while( !assocElement.isNull() )
  {
    if( assocElement.tagName() == "UML:AssocWidget" )
    {
      assocData = new CAssociationWidgetData();
      if( !assocData -> loadFromXMI( assocElement ) )
        return false;
      m_AssociationList.append( assocData );
    }
    node = assocElement.nextSibling();
    assocElement = node.toElement();
  }
  return true;
}





