#include <qdatastream.h>

#include "umlwidgetdata.h"
#include "associationwidget.h"

#include <kdebug.h>

CUMLWidgetData::CUMLWidgetData()
{
	m_bUseFillColor = true;
	m_nId = -1;
	m_Type = wt_UMLWidget;
	m_nX = 0;
	m_nY = 0;
	m_nWidth = 0;
	m_nHeight = 0;
}

CUMLWidgetData::CUMLWidgetData(CUMLWidgetData &Other)
{
	*this = Other;
}

CUMLWidgetData::~CUMLWidgetData()
{
	
}

CUMLWidgetData & CUMLWidgetData::operator=(const CUMLWidgetData & Other)
{
	m_bUseFillColor = Other.m_bUseFillColor;
	m_nId = Other.m_nId;
	m_Type = Other.m_Type;
	m_nX = Other.m_nX;
	m_nY = Other.m_nY;
	m_Assocs = Other.m_Assocs;
	m_Font = Other.m_Font;
	m_nHeight = Other.m_nHeight;
	m_nWidth = Other.m_nWidth;
	return *this;
}

bool	 CUMLWidgetData::operator==(const CUMLWidgetData & Other)
{
	if( this == &Other )
		return true;

	if(m_Type != Other.m_Type)
	{
		return false;
	}

	if(m_Assocs.count() != Other.m_Assocs.count())
	{
		return false;
	}

	CAssociationWidgetListIt assoc_it( m_Assocs );
	CAssociationWidgetListIt assoc_it2( Other.m_Assocs );
	CAssociationWidget * assoc = 0, *assoc2 = 0;
	while ( ((assoc=assoc_it.current()) != 0) &&  ((assoc2=assoc_it2.current()) != 0))
	{
		++assoc_it;
		++assoc_it2;
		if(!(*assoc == *assoc2))
		{
			return false;
		}
	}
	return true;
//NOTE:  In the comparison tests we are going to do, we don't need these values.
//They will actually stop things functioning correctly so if you change these, be aware of that.
/*	if(m_bUseFillColor != Other.m_bUseFillColor)
	{
		return false;
	}*/
/*	if(m_nId != Other.m_nId)
	{
		return false;
	}    */
/*	if( m_Font != Other.m_Font )
		return false;       */	
/*	if(m_nX  != Other.m_nX)
	{
		return false;
	}
	if(m_nY != Other.m_nY)
	{
		return false;
	} */
}

Uml::UMLWidget_Type CUMLWidgetData::getType()
{
	return m_Type;
}

void CUMLWidgetData::setType( UMLWidget_Type Type)
{
	m_Type = Type;
}

int CUMLWidgetData::getX()
{
	return m_nX;
}

void CUMLWidgetData::setX( int X)
{
	m_nX = X;
}

int CUMLWidgetData::getY()
{
	return m_nY;
}

void CUMLWidgetData::setY( int Y)
{
	m_nY = Y;
}

int CUMLWidgetData::getId()
{


	return m_nId;
}

void CUMLWidgetData::setId( int Id)
{
	m_nId = Id;
}

bool CUMLWidgetData::getUseFillColor()
{
	return m_bUseFillColor;
}

void CUMLWidgetData::setUseFillColor( bool UseFillColor)
{
	m_bUseFillColor = UseFillColor;
}

bool CUMLWidgetData::serialize(QDataStream *s, bool archive)
{
	if(archive)
	{
		*s	<< m_Type
				<< m_nX
				<< m_nY
				<< m_nId
				<< m_bUseFillColor
				<< m_Font
				<< m_nHeight
				<< m_nWidth;
	}
	else
	{
		*s	>> (int)m_Type
			 	>> m_nX
			 	>> m_nY
			 	>> m_nId
				>> (int)m_bUseFillColor
				>> m_Font
				>> m_nHeight
				>> m_nWidth;
	}
	return true;
}

uint CUMLWidgetData::getNumAssoc()
{
	return m_Assocs.count();
}

long CUMLWidgetData::getClipSizeOf()
{
	return sizeof(m_Type) + 
		sizeof(m_nId) + 
		sizeof(m_nX) +
		sizeof(m_nY) +
		sizeof(m_bUseFillColor) + 
		sizeof( m_Font ) +
		sizeof( m_nHeight ) +
		sizeof( m_nWidth );
}
/** Prints the data members to standard error */
void CUMLWidgetData::print2cerr()
{
	if(m_bUseFillColor)
	{
		kdDebug() << "m_bUseFillColor = " << "true" << endl;
	}
	else
	{
		kdDebug() << "m_bUseFillColor = " << "false" << endl;
	}
	kdDebug() << "m_nId = " << m_nId << endl;
	kdDebug() << "m_nNumAssoc = " << m_Assocs.count() << endl;
	QString stype;
	switch(m_Type)
	{
	case wt_Actor:
	   	stype = "ACTOR";
	break;
   case wt_UseCase:
    	stype = "USECASE";
	break;
	case wt_Class:
		stype = "CONCEPT";
	break;
	case wt_Text:
		stype = "FLOATTEXT";
	break;
	case wt_Note:
		stype = "NOTE";



	break;
	case wt_Object:
		stype = "OBJECT";
	break;
    case wt_Message:
		stype = "MESSAGEW";
	break;
	default:
		stype = "UNDEFINED";
	break;
	}
	kdDebug() << "m_Type = " << stype <<endl;
	kdDebug() << "m_nX = " << m_nX << endl;
	kdDebug() << "m_nY = " << m_nY << endl;
}

void CUMLWidgetData::AddAssoc(CAssociationWidget* pAssoc)
{
	if(pAssoc)
	{
		m_Assocs.append(pAssoc);
	}
}

void CUMLWidgetData::RemoveAssoc(CAssociationWidget* pAssoc)
{
	if(pAssoc)
	{
		m_Assocs.remove(pAssoc);
	}
}

bool CUMLWidgetData::saveToXMI( QDomDocument & /*qDoc*/, QDomElement & qElement )
{
  /*
    Call after required actions in child class.
    Type must be set in the child class.
  */
  qElement.setAttribute( "xmi.id", m_nId );
  qElement.setAttribute( "font", m_Font.toString() );
  qElement.setAttribute( "usefillcolor", m_bUseFillColor );
  qElement.setAttribute( "x", m_nX );
  qElement.setAttribute( "y", m_nY );
	qElement.setAttribute( "width", m_nWidth );
	qElement.setAttribute( "height", m_nHeight );
  return true;
}

bool CUMLWidgetData::loadFromXMI( QDomElement & qElement )
{
  QString id = qElement.attribute( "xmi.id", "-1" );
  QString font = qElement.attribute( "font", "" );
  QString usefillcolor = qElement.attribute( "usefillcolor", "1" );
  QString x = qElement.attribute( "x", "0" );
  QString y = qElement.attribute( "y", "0" );
  QString h = qElement.attribute( "height", "0" );
  QString w = qElement.attribute( "width", "0" );

  m_nId = id.toInt();
  if( font != "" )
    m_Font.fromString( font );
  m_bUseFillColor = (bool)usefillcolor.toInt();
  m_nX = x.toInt();
  m_nY = y.toInt();
  m_nWidth = w.toInt();
  m_nHeight = h.toInt();
  return true;
}


