#ifndef UMLWIDGETDATA_H
#define UMLWIDGETDATA_H

#include "associationwidgetdatalist.h"
#include "umlnamespace.h"
#include <qdom.h>
#include <qfont.h>

class QDataStream;
class CAssociationWidget;

/**	This class holds all the UMLWidget's Information
*		All this information goes to a file or clipboard
*		when a UMLWidget object is serialized
*		With this class we are trying to achieve isolation
*		between data and display layers.
*		@author Gustavo Madrigal
*/

class CUMLWidgetData : public Uml
{
friend class UMLWidget;
public: 
	/**
	*   Constructor
	*/
	CUMLWidgetData();

	/**
	*		Copy constructor
	*/
	CUMLWidgetData(CUMLWidgetData &Other);

	/**
	*  	Deconstructor
	*/
	virtual ~CUMLWidgetData();

	/**
	* 		Overload '=' operator
	*/
	virtual CUMLWidgetData & operator=(const CUMLWidgetData & Other);

	/**
	* 		Overload '==' operator
	*/
  virtual bool		operator==(const CUMLWidgetData & Other);

  /**
	*	 Write property of int m_Type. 
	*/
  virtual void setType( UMLWidget_Type Type);

  /**
	*	 Read property of int m_Type. 
	*/
  virtual UMLWidget_Type getType();

  /**
	*	 Write property of int m_nX. 
	*/
  virtual void setX( int X);

  /**
	* Read property of int m_nX. 
	*/
  virtual int getX();
	
  /**
	* Write property of int m_nY. 
	*/
  virtual void setY( int Y);

  /**
	* Read property of int m_nY. 
	*/
  virtual int getY();

	/**
	*		Returns the height of widget.
	*		Only valid after load.
	*/
	virtual int getHeight(){ return m_nHeight; }

	/**
	*		Returns the width of the widget.
	*		Only valid after load.
	*/
	virtual int getWidth(){ return m_nWidth; }

	/**
	*		Sets the width of the widget.
	*		Sets before save/copy.
	*/
	virtual void setWidth( int width ){ m_nWidth = width; }

	/**
	*		Sets the height of the widget.
	*		Sets before save/copy.
	*/
	virtual void setHeight( int height ){ m_nHeight = height; }

  /**
	* Write property of int m_Id. 
	*/
  virtual void setId( int Id);

  /**
	* Read property of int m_Id. 
	*/
  virtual int getId();

  /**
	* Write property of bool m_bUseFillColor. 
	*/
  virtual void setUseFillColor( bool UseFillcolor);

  /**
	* Read property of bool m_bUseFillColor. 
	*/
  virtual bool getUseFillColor();

  /** 
	*		Saves/loads the class
	*/
  virtual bool serialize(QDataStream *s, bool archive);

  /**
	* Read property of int m_nNumAssoc. 
	*/
  virtual uint getNumAssoc();
	
 /**
	* Returns the amount of bytes needed to serialize this object 
	*/
  virtual long getClipSizeOf();

  /**
	* Prints the data members to standard error 
	*/
  virtual void print2cerr();

  /**
	* Removes an already created association from the list of associations that include
	*	this UMLWidget 
	*/
  void RemoveAssoc(CAssociationWidget* pAssoc);

  /**
	* Adds an already created association to the list of associations that include
	*	this UMLWidget 
	*/
  void AddAssoc(CAssociationWidget* pAssoc);
	
	/**
	* 		Returns the list of associations connected to this widget.
	*/
	CAssociationWidgetList & getAssocList() { return m_Assocs; }

	/**
	*  	Sets the font the widget is to use.
	*/
	virtual void setFont( QFont font ){ m_Font = font; }

	/**
	* 		Returns the font the widget is too use.
	*/
	virtual QFont getFont(){ return m_Font; }

  virtual bool saveToXMI( QDomDocument & qDoc, QDomElement & qElement );

  virtual bool loadFromXMI( QDomElement & qElement );
protected: // Protected attributes
  /**
	*		Type of widget.
	 */
  UMLWidget_Type m_Type;

  /**
	* X Position of the UMLWidget that holds this instance 
	*/
  int m_nX;

  /**
	* Y Position of the widge 
	*/
  int m_nY;

	/**
	*		Width of width
	*/
	int m_nWidth;

	/**
	*		Height of width
	*/
	int m_nHeight;

  /**
	* This UMLObjectID of the UMLObject related to the UMLWidget holding 
	*	this CUMLWidgetData Instance 
	*/
  int m_nId;

  /**
	* This flag indicates if the UMLWidget holding this CUMLWidgetData instance
	*	uses the Diagram FillColor 
	*/
  bool m_bUseFillColor;

  /**
	*	 A list of CAssociationWidgets between the UMLWidget holding this CUMLWidgetData
	*	instance and other UMLWidgets in the diagram 
	*/
  CAssociationWidgetList m_Assocs;

	/**
	* 		The font the widget will use.
	*/
	QFont m_Font;
};

#endif
