#ifndef USECASEWIDGET_H
#define USECASEWIDGET_H
#include <qpainter.h>
#include "umlwidget.h"

#define UC_MARGIN 5
#define UC_WIDTH 60
#define UC_HEIGHT 30
/**
	*	This class is the graphical version of a UML UseCase.  A UseCaseWidget is created
	* by a @ref UMLView.  An UseCaseWidget belongs to only one @ref UMLView instance.
	* When the @ref UMLView instance that this class belongs to, it will be automatically deleted.
	*
	* If the @ref UseCase class that this UseCaseWidget is displaying is deleted, the @ref UMLView will
	* make sure that this instance is also deleted.
	*
	* The UseCaseWidget class inherits from the @ref UMLWidget class which adds most of the functionality
	* to this class.
	*
	* @short  A graphical version of a UML UseCase.
  *	@author Paul Hensgen <phensgen@techie.com>
  * @version 1.0
  */


class UseCaseWidget : public UMLWidget
{
public:
	/**
		*	Creates a UseCase widget.
		*
		*	@param	view		The parent of the widget.
		*	@param	pData		The CUMLWidgetData to represent.
		*/
	UseCaseWidget(UMLView * view, UMLObject *o, CUMLWidgetData* pData);
	/**
		*	Creates a UseCase widget.
		*
		*	@param	view		The parent of the widget.
		*	@param	o				The UMLObject to represent.
		*/
	UseCaseWidget(UMLView * view, UMLObject *o);
	
	/**
		*	Creates a UseCase widget.
		*
		*	@param	view		The parent of the widget.
		*/
	UseCaseWidget(UMLView * view);
	
	/**
		*	Standard deconstructor	
		*/
	~UseCaseWidget();

  /**
  * Synchronize the Widget's m_pData member with its display properties, for example:
  *the X and Y positions of the widget, etc
  */
  virtual void SynchronizeData();

  /**
		*	Overrides the standard paint event.
		*/
	void draw(QPainter & p, int offsetX, int offsetY);
protected:
	/**
		*	Calculates the size of the widget.
		*/
	void calculateSize();
};

#endif
