
#include <qpixmap.h>
#include <qcursor.h>

#include <kglobal.h>
#include <kstddirs.h>
#include <ktoolbarbutton.h>
#include <kapplication.h>

#include "uml.h"
#include "umlview.h"
#include "worktoolbar.h"

#include <kdebug.h> //remove

WorkToolBar::WorkToolBar(QMainWindow *parentWindow, const char*name) : KToolBar(parentWindow,QMainWindow::Right,true,name)
{
  loadPixmaps();
  m_Type = Uml::dt_Class;//first time in just want it to load arrow, needs anything but dt_Undefined
  setOrientation( Vertical );
  setVerticalStretchable( true );
  //intialize old tool mal, everything starts with select tool (arrow)
  m_map.insert(Uml::dt_UseCase,tbb_Arrow);
  m_map.insert(Uml::dt_Collaboration,tbb_Arrow);
  m_map.insert(Uml::dt_Class,tbb_Arrow);
  m_map.insert(Uml::dt_Sequence,tbb_Arrow);
  m_map.insert(Uml::dt_State,tbb_Arrow);
  m_map.insert(Uml::dt_Activity,tbb_Arrow);
  m_map.insert(Uml::dt_Undefined,tbb_Arrow);

  slotCheckToolBar( Uml::dt_Undefined );
  connect( this, SIGNAL( released( int ) ), this, SLOT( buttonChanged (int ) ) );
}
////////////////////////////////////////////////////////////////////////////////////////////////////
WorkToolBar::~WorkToolBar()
{
  disconnect(this, SIGNAL(released(int)),this,SLOT(buttonChanged(int)));
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void WorkToolBar::slotCheckToolBar(Uml::Diagram_Type dt)
{
  if( dt == m_Type )
    return;
  clear();
  m_Type = dt;

  insertButton( m_Pixmaps.Arrow, tbb_Arrow );
  setToggle( tbb_Arrow,true );
  toggleButton( tbb_Arrow );
  m_CurrentButtonID = tbb_Arrow;

  if( m_Type != Uml::dt_Undefined )
  {

	  insertButton( m_Pixmaps.Note, tbb_Note, true, "note" );
    setToggle( tbb_Note, true );

    insertButton( m_Pixmaps.Anchor, tbb_Anchor, true, "anchor" );
    setToggle( tbb_Anchor, true );

    insertButton( m_Pixmaps.Text, tbb_Text, true, "Line of Text" );
    setToggle( tbb_Text, true );
  }
  if( m_Type == Uml::dt_UseCase )
  {



    insertButton( m_Pixmaps.Actor, tbb_Actor, true, "actor" );
    setToggle( tbb_Actor, true );

    insertButton( m_Pixmaps.UseCase, tbb_UseCase,true, "use case" );
    setToggle( tbb_UseCase, true );

    insertButton( m_Pixmaps.Generalization, tbb_Generalization, true,"generalization" );
    setToggle( tbb_Generalization, true );

    insertButton( m_Pixmaps.Dependency, tbb_Dependency, true,"dependency" );
    setToggle( tbb_Dependency,true );

    insertButton( m_Pixmaps.Association, tbb_Association, true,"association" );
    setToggle( tbb_Association,true );

    insertButton( m_Pixmaps.UniAssociation, tbb_UniAssociation, true,"unidirectional association" );
    setToggle( tbb_UniAssociation, true );
  }
  else if( m_Type == Uml::dt_Class )
  {
    insertButton( m_Pixmaps.Concept, tbb_Concept, true,"concept" );
    setToggle( tbb_Concept,true );

    insertButton( m_Pixmaps.Composition, tbb_Composition, true,"composition" );
    setToggle( tbb_Composition, true );

    insertButton( m_Pixmaps.Implementation, tbb_Implementation, true,"implementation" );
    setToggle( tbb_Implementation, true );

    insertButton( m_Pixmaps.Realization, tbb_Realization, true,"realize" );
    setToggle( tbb_Realization, true );

    insertButton( m_Pixmaps.Generalization, tbb_Generalization, true,"generalization" );
    setToggle( tbb_Generalization, true );

    insertButton( m_Pixmaps.Aggregation, tbb_Aggregation, true,"aggregation" );
    setToggle( tbb_Aggregation, true );

    insertButton( m_Pixmaps.Dependency, tbb_Dependency, true,"dependency" );
    setToggle( tbb_Dependency, true );

    insertButton( m_Pixmaps.Association, tbb_Association, true,"association");
    setToggle( tbb_Association,true );

    insertButton( m_Pixmaps.UniAssociation, tbb_UniAssociation, true,"unidirectional association");
    setToggle( tbb_UniAssociation,true );
  }
  else if( m_Type == Uml::dt_Sequence )
  {
    insertButton( m_Pixmaps.Object, tbb_Object, true,"object" );
    setToggle (tbb_Object, true);

    insertButton( m_Pixmaps.Message, tbb_Seq_Message, true,"message" );
    setToggle( tbb_Seq_Message, true );
  }
  else if( m_Type == Uml::dt_Collaboration )
  {
    insertButton( m_Pixmaps.Object, tbb_Object, true,"object" );
    setToggle( tbb_Object, true );

    insertButton( m_Pixmaps.Message, tbb_Coll_Message, true,"message" );
    setToggle( tbb_Coll_Message, true );
  }
  else if( m_Type == Uml::dt_State )
  {
    insertButton( m_Pixmaps.Initial_State, tbb_Initial_State, true,"initial state" );
    setToggle( tbb_Initial_State, true );

    insertButton( m_Pixmaps.UseCase, tbb_State, true,"state" );
    setToggle( tbb_State, true );

    insertButton( m_Pixmaps.End_State, tbb_End_State, true,"end state" );
    setToggle( tbb_End_State, true );

    insertButton( m_Pixmaps.UniAssociation, tbb_State_Transition, true,"state transition" );
    setToggle( tbb_State_Transition, true );
  }
  else if( m_Type == Uml::dt_Activity )
  {
    insertButton( m_Pixmaps.Initial_State, tbb_Initial_Activity, true,"initial activity" );
    setToggle( tbb_Initial_Activity, true );

    insertButton( m_Pixmaps.UseCase, tbb_Activity, true,"activity" );
    setToggle( tbb_Activity, true );

    insertButton( m_Pixmaps.End_State, tbb_End_Activity, true,"end activity" );
    setToggle( tbb_End_Activity, true );

    insertButton( m_Pixmaps.Branch, tbb_Branch, true,"branch/merge" );
    setToggle( tbb_Branch, true );

    insertButton( m_Pixmaps.Fork, tbb_Fork, true,"fork/join" );
    setToggle( tbb_Fork, true );

    insertButton( m_Pixmaps.UniAssociation, tbb_Activity_Transition, true,"activity transition" );
    setToggle( tbb_Activity_Transition, true );
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
void WorkToolBar::buttonChanged(int b)
{

  UMLView *view = ((UMLApp*) parent()->parent())->getDocument()->getCurrentView();
  QCursor curs;
	//if trying to turn off arrow - stop it
   ToolBar_Buttons tbb = (ToolBar_Buttons)b;
	if(tbb == tbb_Arrow && m_CurrentButtonID == tbb_Arrow)
	{
		toggleButton(tbb_Arrow);
    curs.setShape(Qt::ArrowCursor);
    view -> setCursor(curs);
		return;
	}
	//if toggling off a button set to arrow
	if(tbb == m_CurrentButtonID)
	{ m_map[m_Type] = m_CurrentButtonID;  // store old tool for this diagram type
		toggleButton(tbb_Arrow);
		m_CurrentButtonID = tbb_Arrow;
		emit sigButtonChanged(m_CurrentButtonID);
    curs.setShape(Qt::ArrowCursor);
    view -> setCursor(curs);
		return;
	}
  m_map[m_Type] = m_CurrentButtonID;
	toggleButton(m_CurrentButtonID);
	m_CurrentButtonID = tbb;
	emit sigButtonChanged(m_CurrentButtonID);
  if(m_CurrentButtonID == tbb_Arrow)
    curs.setShape(Qt::ArrowCursor);
  else   //maybe switch and set a different cursor for each tool
    curs.setShape(Qt::CrossCursor);

  view->setCursor(curs);
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void WorkToolBar::slotResetToolBar()
{
	if(m_CurrentButtonID == tbb_Arrow)
		return;//really shouldn't occur
	toggleButton(m_CurrentButtonID);
	m_CurrentButtonID = tbb_Arrow;
	toggleButton(m_CurrentButtonID);
	emit sigButtonChanged(m_CurrentButtonID);
  UMLView *view = ((UMLApp*) parent()->parent())->getDocument()->getCurrentView();
  QCursor curs;
  curs.setShape(Qt::ArrowCursor);
  view -> setCursor(curs);
}
///////////////////////////////////////////////////////////////////////////////////////////
void WorkToolBar::setOldTool()
{
  KToolBarButton *b = (KToolBarButton*) getWidget(m_map[m_Type]);
  if(b) b -> animateClick();
}

void WorkToolBar::setDefaultTool()
{
  KToolBarButton *b = (KToolBarButton*) getWidget(tbb_Arrow);
  if(b) b -> animateClick();
}
////////////////////////////////////////////////////////////////////////////////////////////////////
void WorkToolBar::loadPixmaps()
{
  KStandardDirs * dirs = KGlobal::dirs();
  QString dataDir = dirs -> findResourceDir( "data", "uml/pics/object.xpm" );
  dataDir += "/uml/pics/";

  m_Pixmaps.Object.load( dataDir + "object.xpm" );
  m_Pixmaps.Message.load( dataDir + "message.xpm" );
  m_Pixmaps.Arrow.load( dataDir + "arrow.xpm" );
  m_Pixmaps.Association.load( dataDir + "line.xpm" );
  m_Pixmaps.Anchor.load( dataDir + "anchor.xpm" );
  m_Pixmaps.Text.load( dataDir + "text.xpm" );
  m_Pixmaps.Note.load( dataDir + "note.xpm" );
  m_Pixmaps.Actor.load( dataDir + "actor.xpm" );
  m_Pixmaps.Dependency.load( dataDir + "dependency.xpm" );
  m_Pixmaps.Aggregation.load( dataDir + "aggregation.xpm" );
  m_Pixmaps.UniAssociation.load( dataDir + "uniassoc.xpm" );
  m_Pixmaps.Generalization.load( dataDir + "generalization.xpm" );
  m_Pixmaps.Realization.load( dataDir + "realizes.xpm" );
  m_Pixmaps.Composition.load( dataDir + "composition.xpm" );
  m_Pixmaps.Implementation.load( dataDir + "implements.xpm" );
  m_Pixmaps.UseCase.load( dataDir + "case.xpm" );
  m_Pixmaps.Concept.load( dataDir + "umlclass.xpm" );
  m_Pixmaps.Initial_State.load( dataDir + "initial_state.xpm" );
  m_Pixmaps.End_State.load( dataDir + "end_state.xpm" );
  m_Pixmaps.Branch.load( dataDir + "branch.xpm" );
  m_Pixmaps.Fork.load( dataDir + "fork.xpm" );
}


